/**
* Title: SimpleClient
* Description:
* Copyright:    Copyright (c) 2008-2010
* Company:      Morpheus Quality Technologies
* @author Stefano Adriani
* @version 1.0
*/

package adriani.jws.template.client ;

import adriani.jws.template.db.* ;
import adriani.jws.template.eclipse.* ;
import adriani.jws.template.service.* ;
import adriani.jws.template.messages.* ;
import adriani.jws.template.util.* ;
import adriani.jws.template.Settings ;

import java.util.* ;
import java.math.* ;

/**
* A very simple command line class offering a <code>main()</code> method to allow invocation of the web service
* directly on the command line.<br/> If all the necessary enviroment variables are correctly defined it should
* be possibile to run this client writing:<br/><br/>
*
* <code>java -cp %CLASSPATH% adriani.jws.template.client.SimpleClient</code><br/><br/>
*
* This class can be launched specifying the input parameters for the web service, choosing between two
* different approaches: in the <strong>emulate</strong> mode the client will invoke the business logic
* directly, bypassing the ADB structure. In the <strong>actual</strong> mode the client will invoke
* the actual skeleton class of the service.<br/><br/>
*
* For details run <code>SimpleClient -h</code>.
*/
public class SimpleClient {

	/** Number of input parameters necessary to fill the input SOAP request.								*/
	public 		 			int 		howMany 			= 2 ;

	/** The string array containing the default hardcoded SOAP request parameters.							*/
	public 		 			String[]	default_data 		= null ;

	/**
	* If <code>true</code> invokes the <code>SimpleClient.testServiceLogic()</code> method emulating the
	* web service (without using the <code>ADBBean</code> classes). Otherwise, if <code>false</code>, invokes
	* the actual skeleton class implementing the web service.
	*/
	public		 			boolean 	emulate 			= false ;

	/** Static final value stored as class field to improve performances.									*/
	public static final		String		EMPTY 				= "" ;
	/** Static final value stored as class field to improve performances.									*/
	public static final		String		BLANK 				= " " ;
	/** Static final value stored as class field to improve performances.									*/
	public static final	 	String		ZERO 				= "0" ;

  	/** Beginning of each logging message produced by this class.											*/
	protected static 		String		leadingName 		= "Simple Client - " ;
	
  	/** Logger instance for this class (static): must be set after <code>WSLogger</code> initialization.	*/
	protected static 		WSLogger	logger ;

    /**
	* Main method used to call the web service from the command line, forwarding action to the
	* <code>run(String[] args)</code> method.
	*
	* @param args the standard input string array, usually read from STDIN.
	* @see run(String[] args)
    */
    public static void main(String[] args) {
	
		// Creating the client
		SimpleClient client = new SimpleClient() ;

		// Setting hardcoded default values
		//client.default_data = new String[] {"00000012", "1"} ;
		client.default_data = new String[] {"390498978842", "1"} ;

		// Running the client
		client.run(args) ;
		
		// Comment next line if necessary (e.g. testing threads management)
		System.exit(0) ;
	}

    /**
	* If the client is run in the emulate mode the method forwards action to the <code>testServiceLogic()</code>
	* method, which invokes the service logic using the input parameters (hardcoded or from the command line).<br/>
	* Otherwise, if the client is run in the actual mode, action is forwarded to the <code>testServiceSkeleton()</code>
	* method, which calls the service skeleton passing the input parameters (as above).
	*
	* @param args the standard input string array, usually read from STDIN.
    */
    public void run(String[] args) {

		// 1. Eventually managing options
		if ( (args != null) && (args.length > 0) && (args[0].startsWith("-")) ) {
			if (args[0].length() > 1) manageOptions(args) ;
			else System.out.println("\nPlease specify a valid option, or use -h to get some help") ;
		// 2. Otherwise we test the web service using default settings
		} else {
			StringBuffer[] param = getParameters(args, 0) ;
			if (emulate) {
				testServiceLogic(param) ;
				// PLACE-HOLDER for eventual other SOAP operation
			} else {
				testServiceSkeleton(param) ;
				// PLACE-HOLDER for eventual other SOAP operation
			}
		}
	}

// *********************************************************************************************************
// **********************************		MANAGING OPTIONS		****************************************
// *********************************************************************************************************

    /**
	* Manage all the possibile options available from the command line.
	* The available test offered by the <code>SimpleClient</code> are executed by calling:<br/><br/>
	*
	* <code>java -cp %CLASSPATH% adriani.jws.template.client.SimpleClient -t n</code><br/><br/>
	*
	* Where the <code>n</code> parameter select one of the following test:<br/><br/>
	*
	* 0. Test the logger system offered by the <code>WSLogger</code> class.<br/>
	* 1. Forces the class loader to load the <code>GeneralService</code> class in order to verify that
	* the configuration and initialization sequence is executed correctly.<br/>
	* 2. Verify database enviroment (driver and connection pool) and all active threads.<br/>
	* 3. Print the list of all active threads.<br/>
	*
	* @param buffer the input string array
    */
    public void manageOptions(String[] args) {

		// Adjusting input
		String option = args[0].substring(0,2) ;
		StringBuffer[] buffer = new StringBuffer[args.length] ;
		for (int i = 0 ; i < args.length ; i++) buffer[i] = new StringBuffer(args[i]) ;

		// Calling service in the actual mode
		if (option.equalsIgnoreCase("-a")) {
			StringBuffer[] param = getParameters(args, 1) ;
			testServiceSkeleton(param) ;
			// PLACE-HOLDER for eventual other SOAP operation
		}
		// Calling service in the emulate mode
		else if (option.equalsIgnoreCase("-e")) {
			StringBuffer[] param = getParameters(args, 1) ;
			testServiceLogic(param) ;
			// PLACE-HOLDER for eventual other SOAP operation
		}
		// Printing help screen
		else if (option.equalsIgnoreCase("-h")) {
			printHelp() ;
		}
		// Printing information about default parameters
		else if (option.equalsIgnoreCase("-i")) {
			System.out.println("\nNumber of input exptected parameters: " + howMany + "\n") ;
			System.out.println("Currently default hardcoded parameters:\n") ;
			for (int i = 0 ; i < default_data.length ; i++) System.out.println("Param " + i + " = " + default_data[i]) ;
		}
		// Creating the keywords.properties file from a given txt file
		else if (option.equalsIgnoreCase("-c")) {
			String source = null ;
			if ( (args.length > 1) && (args[1] != null) ) source = args[1] ;
			else source = "name_list.txt" ;
			ServiceData.createKeywords(source) ;
		}
		// Dumping the content of the keywords.properties file into the keywords.java file (source code-block)
		else if (option.equalsIgnoreCase("-d")) {
			System.out.println() ;
			ServiceData.loadKeywords(Settings.KEYWORDS) ;
			ServiceData.produceJavaFile() ;
		}
		// Executing a basic test
		else if (option.equalsIgnoreCase("-t")) {
			initLogger() ;
			int code = getIntValue(buffer, 1) ;
			switch(code) {
				case 	0	: 	testLogger() ; 					break ;
				case 	1	: 	testConfiguration() ;			break ;
				case 	2	: 	testDatabase() ;				// No break, we want cascade behaviuor
				case 	3	: 	ThreadManager.printList() ; 	break ;
				default		: 	System.out.println(leadingName + "Please specify a number from 0 to 3") ;
			}
		}
		// Executing a simple query
		else if (option.equalsIgnoreCase("-q")) {
			initLogger() ;
			testQuery(buffer) ;
			System.exit(0) ;
		}
		// Executing an unit test
		else if (option.equalsIgnoreCase("-u")) {
			initLogger() ;
			int code = getIntValue(buffer, 1) ;
			UnitTester tester = new UnitTester() ;
			tester.runTest(code, buffer) ;
		}
		// Testing a Java5 feature
		else if (option.equalsIgnoreCase("-j")) {
			int code = getIntValue(buffer, 1) ;
			JavaTester tester = new JavaTester() ;
			tester.testJava5(code) ;
		}
		// Handling invalid options
		else {
			System.out.println("Invalid option \n") ;
		}
	}

// ************************************************************************************************************
// **********************************		TESTING SERVICE LOGIC		***************************************
// ************************************************************************************************************

    /**
	* Invokes the <code>ServiceLogic.getOutput()</code> method using the wrappers defined in the <code>messages</code>
	* package. This method is useful when it's necessary to test the web service logic without using the
	* <code>ADBBean</code> classes produces by Eclipse.<br/>
	* Basically speaking, the invocation of the <code>ServiceLogic.getOutput()</code> method performed here
	* should almost reflect the actual implementation of the skeleton java class produced by Eclipse or the
	* Axis2 tool, with just two differences:<br/><br/>
	*
	* 1. Here the <code>WrapperIn</code> object is constructed using the <code>WrapperIn.setXXX()</code> methods,
	* while in the skeleton class a constructor like <code>WrapperIn(ADBBean element)</code> is used.<br/>
	* 2. Here we extract data from the output wrapper using the <code>WrapperOut.getXXX()</code> methods, in
	* order to print these data on the STDOUT, while the skeleton class returns the whole
	* <code>ADBBean</code> object given by <code>WrapperOut.getElement()</code>.<br/><br/>
	*
	* @param param the string buffer array containing all the input data
    */
    public void testServiceLogic(StringBuffer[] param) {

		// 1. Initializing
		if (Settings.DETAILS) System.out.println(leadingName + "Emulating service\n") ;
		initLogger() ;

		// 2. Eventually testing the web service logic
		Wrapper[] record = null ;
		WrapperOut output = null ;
		ServiceLogic logic = null ;

		// 3. Creating the business logic
		logic = new ServiceLogic() ;
		// Emulating the SOAP input message
		WrapperIn input = new WrapperIn() ;
		input.setNumber(param[0]) ;
		input.setCustomer(param[1]) ;
		// Bypassing the skeleton class
		output = logic.getOutput(input) ;
		// Retrieving data
		record = output.getRecords() ;
		if (Settings.DETAILS) {
			System.out.print(leadingName + "Retrieved: " + output) ;
			if (record != null) System.out.println(" (" + record.length + " records)") ;
		}

		// 4. If we found a valid result we print all the retrieved records
		if ( record != null ) {
			int max = record.length ;
			StringBuffer buffer = new StringBuffer() ;
			buffer.append(leadingName + "Status : "	+ output.getStatus()	+ "\n") ;
			buffer.append(leadingName + "Message: " + output.getMessage()	+ "\n\n") ;
			for (int i = 0 ; i < max ; i++) {
				WrapperData envelope = (WrapperData) record[i] ;
				buffer.append(leadingName + "First: "	+ 	envelope.getFirst() 	+ "\n") ;
				buffer.append(leadingName + "Second: "	+ 	envelope.getSecond() 	+ "\n") ;
				buffer.append("\n") ;
			}
			// Printing result on STDOUT and in the LOG FILE
			String response = "OUTPUT DATA:\n\n" + buffer ;
			logger.logInfo(response) ;
		}

		// 5. Eventually printing analysis data for this single service call
		//if (Settings.ANALYZE) logic.getServiceAnalyzer().logResults() ;
	}

    /**
	* This method is invoked only if the client is run in the <em>actual</em> mode.
	* The required input <code>ADBBean</code> objects are produced using the input parameters, in order to
	* respect the structure of the input <code>ADBBean</code> element. In the same way, the resulting
	* output data are retrieved from the output <code>ADBBean</code> object, as returned by the skeleton,
	* then are printed on the STDOUT.
	*
	* @param param the string buffer array containing all the input data
    */
    public void testServiceSkeleton(StringBuffer[] param) {

		// 1. Initializing
		if (Settings.DETAILS) System.out.println(leadingName + "Testing actual skeleton\n") ;
		initLogger() ;

		// 2. Creating the skeleton
		TemplateSkeleton skeleton = new TemplateSkeleton() ;

		// 3. Emulating the SOAP IN parameters
		User user = new User() ;
		// Creating inner primitive types
		UserType userType = new UserType() ;
		PhoneType phoneType = new PhoneType() ;
		// Setting inner primitive values
		phoneType.setPhoneType( param[0].toString() ) ;
		userType.setPhone(phoneType) ;
		userType.setCustomer( new BigInteger(param[1].toString()) ) ;
		user.setUser(userType) ;

		// 4. Performing actual call
		Information information = skeleton.getInformation(user) ;

		// 5. Retrieving the SOAP OUTPUT data
		StringBuffer buffer = new StringBuffer() ;
		InformationType informationType = information.getInformation() ;
		RecordsType[] array = informationType.getRecords() ;
		buffer.append(leadingName + "Status : " + informationType.getStatus() + "\n") ;
		buffer.append(leadingName + "Message: " + informationType.getMessage() + "\n\n") ;

		// 6. Printing the SOAP OUTPUT data
		if (array != null) {
			int max = array.length ;
			for (int i = 0 ; i < max ; i++) {
				RecordsType record 	= array[i] ;
				NameType 	first	= record.getFirst() ;
				NameType 	second	= record.getSecond() ;
				buffer.append(leadingName + "First: " 	+ 	first.getNameType() 	+ "\n") ;
				buffer.append(leadingName + "Second: " 	+ 	second.getNameType() 	+ "\n") ;
				buffer.append("\n") ;
			}
		}
		// Printing result on STDOUT and in the LOG FILE
		String response = "OUTPUT DATA:\n\n" + buffer ;
		logger.logInfo(response) ;

		// 7. Eventually printing analysis data for this single service call
		//if (Settings.ANALYZE) skeleton.getServiceLogic().getServiceAnalyzer().logResults() ;
	}

// *********************************************************************************************************
// **********************************		TEST SECTION			****************************************
// *********************************************************************************************************

    /**
	* Test the logging system.
    */
    public void testLogger() {

		// Initializing test
		System.out.println() ;
		System.out.println(leadingName + "Testing logger: " + SimpleClient.logger + "\n") ;

		// Logging test
		logger.logInfo("testing information level") ;
		logger.logWarning("testing warning level") ;
		logger.logSevere("testing severe level") ;
		if (logger.isInfoEnabled()) System.out.println(leadingName + "Check 'if isInfoEnabled()' is true") ;
		else System.out.println(leadingName + "Check 'if isInfoEnabled()' is false") ;
	}
	
    /**
	* Test method used to verify that the static code block in the <code>GeneralService</code> class is found
	* and correctly exectuted. In this way we verify that all the static configuration and initialization 
	* of the web service is executed in the right order.
    */
    public void testConfiguration() {

		// Forcing the VM to load the GeneralService CLASS (not the instance)
		Class service = null ;
		try									{ service = Class.forName("adriani.jws.template.service.GeneralService") ;	}
		catch(ClassNotFoundException e)		{ e.printStackTrace() ;														}
		System.out.println(leadingName + "Loaded class: " + service) ;
	}

    /**
	* Test method used to verify that the static code block in the <code>Database</code> class is found
	* and correctly exectuted. Moreover, when this method is invoked, some metadata about the database's
	* connections and related pools are printed on the STDOUT.
    */
    public void testDatabase() {

		// Forcing the VM to load the Database CLASS (not the instance)
		Class database = null ;
		try									{ database = Class.forName("adriani.jws.template.db.Database") ;	}
		catch(ClassNotFoundException e)		{ e.printStackTrace() ;												}
		System.out.println(leadingName + "Loaded class: " + database) ;
		System.out.println(leadingName + "DB_01 connections pool: " + Database.getJDBCPool(Database.DB_01)) ;
		System.out.println(leadingName + "DB_02 connections pool: " + Database.getJDBCPool(Database.DB_02)) ;

		// Showing metadata about the database
		StringBuffer msg_01 = Database.showMetaData(Database.DB_01) ;
		StringBuffer msg_02 = Database.showMetaData(Database.DB_02) ;
		System.out.println(msg_01.toString()) ;
		System.out.println(msg_02.toString()) ;
	}

    /**
	* Execute a simple dummy SQL query to verify the format of the JDBC result set retrieved from the database.
	* <br/>
	* Current implementation performs a simple SELECT query in the DB_02 database. The query to be executed
	* is selected by the <code>argument[1]</code> parameter, while the input parameter used  to fill the
	* statement (if available) is specified by the <code>argument[2]</code> parameter. In other words this
	* method must be called using following syntax:<br/><br/>
	* 
	* <code>SimpleClient -q n p</code><br/><br/>
	*
	* The value of the parameter <code>n</code> select of the following DBA methods:<br/><br/>
	*
	* 0	: getRecord(target, label, query, cols)<br/>				
	* 1	: getRecordSet(target, label, query, cols)<br/>				
	* 2	: getFunction(target, label, query, param)<br/>				
	* 3	: getFunctionPipe(target, label, query, cols)<br/>				
	* 4	: getProcedure(target, label, query, param, cols, inout)<br/>				
	*
	* @param buffer the input string array, converted into a <code>StringBuffer</code> object.
	* @see GeneralDBA#getRecord(int targetDB, String label, String query, int cols)
	* @see GeneralDBA#getRecordSet(int targetDB, String label, String query, int cols)
	* @see GeneralDBA#getFunction(int targetDB, String label, String query, StringBuffer[] param)
	* @see GeneralDBA#getFunctionPipe(int targetDB, String label, String query, int cols)
    */
    public void testQuery(StringBuffer[] buffer) {

		// 1. Creating the DBA enviroment
		int	test				= 0 ;
		String key 				= null ;
		int target				= Database.DB_02 ;
		ServiceDBA dba 			= new ServiceDBA() ;
		String label 			= "db_02.testQuery: " ;

		// 2.1 Setting input parameters
		if (buffer != null) {
			if (buffer.length > 1) test = getIntValue(buffer, 1) ;
			if (buffer.length > 2) key	= buffer[2].toString() ;
			System.out.println(leadingName + "Input testQuery() params: " + test + BLANK + key) ;
		}
		// 2.2 Setting hardcoded parameters
		int[] 			inout 	= { 2 } ;
		StringBuffer[]	param	= new StringBuffer[] { new StringBuffer("0323552156"), new StringBuffer("0") } ;

		// 3. Preparing statement
		String query = null ;
		switch(test) {
			case 0	:	query = "SELECT * FROM ACTION WHERE ID_ACTION = '" + key + "'" ;		break ;
			case 1	:	query = "SELECT * FROM ACTION WHERE ID_ACTION < '" + key + "'" ;		break ;
			case 2	:	query = "{? = call GET_MODEM_STATUS(?, ?)}" ;							break ;
			case 3	:	query = "SELECT * FROM TABLE(PKG_GET_CUSTOMER_STATUS.GET_ALL_ENDDATES(0293169084, 521431287, 'T', 30))" ;	break ;
			case 4	:	query = "{call VRFY_CUSTOMER_NUMBER(?,?)}" ;							break ;
			default	:	query = "SELECT SYSDATE FROM DUAL" ;
		}
		System.out.println(leadingName + "Prepared query: " + query) ;

		// 4. Opening connection to the target database
		if (dba.openConnection(target)) {
		
			// 4.1 Executing statement
			Object result = null ;
			switch(test) {
				case 0	:	result = dba.getRecord(target, label, query, 3) ;					break ;
				case 1	:	result = dba.getRecordSet(target, label, query, 2) ;				break ;
				case 2	:	result = dba.getFunction(target, label, query, param) ;				break ;
				case 3	:	result = dba.getFunctionPipe(target, label, query, 2) ;				break ;
				case 4	:	result = dba.getProcedure(target, label, query, param, 1, inout) ;	break ;
				default	:	result = dba.getRecord(target, label, query, 1) ;
			}

			// 4.2 Returning connection
			dba.closeConnection(target) ;

			// 4.3: 1st case (single record)
			if ((result != null) && (result instanceof StringBuffer[])) {
				System.out.print("\nFound record: ") ;
				StringBuffer[] record = (StringBuffer[]) result ;
				for (int i = 0 ; i < record.length ; i++) {
					if (record[i] != null) System.out.print(record[i].toString() + BLANK) ;
				}
			// 4.4: 2nd case (record set)
			} else if ((result != null) && (result instanceof StringBuffer[][])) {
				System.out.println("\nFound record set:") ;
				StringBuffer[][] set = (StringBuffer[][]) result ;
				for (int n = 0 ; n < set.length ; n++) {
					if (set[n] != null) for (int i = 0 ; i < set[n].length ; i++) {
						if (set[n][i] != null) System.out.print(set[n][i].toString() + BLANK) ;
					}
					System.out.println() ;
				}
			// 4.5: 3rd case (no result)
			} else if (result == null) System.out.println("\nTest query with no result") ;
			System.out.println() ;
			
		// 5. Eventually handling no database
		} else System.out.println(leadingName + "No connections to target database") ;
		
	}

// *********************************************************************************************************
// **********************************		UTILITIES			********************************************
// *********************************************************************************************************

    /**
	* Inizializes the logger mechanism BEFORE any class could try to call the WSLogger factory, then
	* obtain the <code>WSLogger</code> object instance for this client.
    */
    public void initLogger() {

		if ( !WSLogger.isReady() ) WSLogger.initialize() ;
		SimpleClient.logger = WSLogger.getLogger(SimpleClient.class) ;
		System.out.println() ;
	}

    /**
	* Return the integer number specified after a general "-option" keyword, for example -t or -u.
	*
	* @param buffer the input string array, converted into a <code>StringBuffer</code> object
	* @param index the index of the element to be extracted
	* @return the integer number corresponding to the index-th element of the array
    */
    public int getIntValue(StringBuffer[] buffer, int index) {

		int code = 0 ;
		if (buffer.length <= index) return code ;
		try 				{ code = Integer.parseInt( buffer[index].toString().trim() ) ;		}
		catch(Exception e) 	{ code = 0 ;														}
		return code ;
	}

    /**
	* Extract the SOAP parameters list from the input <code>args</code> string array, starting from the
	* <code>start</code> index position. If the input array is <code>null</code> the default values, 
	* stored as class fields, will be used.
	* If the number of input parameters is less than the required number of arguments this method will 
	* complete the SOAP request using the default values.
	*
	* @param args the standard input string array, usually read from STDIN
	* @param start the index of the first element to be used as SOAP parameter
	* @return the SOAP input parameters as a <code>StringBuffer</code> array
    */
    public StringBuffer[] getParameters(String[] args, int start) {

		// 1. Initializing
		StringBuffer[] param = new StringBuffer[howMany] ;
		System.out.println(leadingName + "Using option " + (emulate?"-e":"-a")) ;

		// 2. Reading input values from the console line
		if ((args != null) && (args.length > start)) {
			System.out.print(leadingName + "Reading console parameters:") ;
			for (int k = 0 ; k < howMany ; k++) {
				int i = start + k ;
				if (i < args.length) param[k] = new StringBuffer( args[i] ) ;
				else if (k < default_data.length) param[k] = new StringBuffer( default_data[k] ) ;
				else param[k] = new StringBuffer(ZERO) ;
				System.out.print(BLANK + param[k]) ;
			}
		}

		// 3. Setting input values using the default data
		else {
			System.out.print(leadingName + "Setting default parameters:") ;
			for (int k = 0 ; k < howMany ; k++) {
				if (k < default_data.length) param[k] = new StringBuffer( default_data[k] ) ;
				else param[k] = new StringBuffer(ZERO) ;
				System.out.print(BLANK + param[k]) ;
			}
		}

		// 4. Formatting output and returning the array
		System.out.println() ;
		return param ;
	}

    /**
	* Print some help about the <code>SimpleClient</code> command line options on the STDOUT.
    */
    public StringBuffer printHelp() {

		// Preparing text message
		StringBuffer message = new StringBuffer() ;
		message.append("\n") ;
		message.append("Simple Client for running and invoking the web service from the command line, ") ;
		message.append("version " + Settings. VERSION + "\n\n") ;
		message.append("Usage: java adriani.jws.template.client.SimpleClient [options] [ARGS]\n\n") ;
		message.append("ARGS: the sequence of input parameters filling the SOAP request (see below)\n\n") ;
		message.append("Options can be:\n\n") ;
		message.append("-h          print this help screen\n") ;
		message.append("-a          invokes the web service in the actual mode (default)\n") ;
		message.append("-e          invokes the web service in the emulate mode\n") ;
		message.append("-i          print amount of the expected input parameters (and default values)\n") ;
		message.append("-c file     create the keywords.properties file using the specified file as source\n") ;
		message.append("-d          dump the keywords properties into the keywords.java file\n") ;
		message.append("-t n        executes the n-th general test offered by the Simple Client class\n") ;
		message.append("-q n param  executes the n-th SQL query offered by the Simple Client class\n") ;
		message.append("-u n        executes the n-th method from the UnitTester class\n") ;
		message.append("-j n        executes the n-th method from the JavaTester class\n\n") ;
		message.append("ARGS: if no argument is specified when running this client from the command line,\n") ;
		message.append("the default values will be used. To acknowledge the amount and value of the default\n") ;
		message.append("parameters, run this client with the option -i (information).\n") ;
		message.append("Arguments can also be specified directly on the command line, space separated.\n") ;
		message.append("If the number of input parameters is less than the required number of arguments\n") ;
		message.append("the client will complete the SOAP request using the default values.\n\n") ;
		message.append("Example: if the web service requires three parameters, and the hardcoded values are {22, 33, 42},\n") ;
		message.append("running the command 'SimpleClient 17 25' will use the params {17, 25, 42}.\n\n") ;
		message.append("TESTING (-t | -q | -u | -j) : see the API documentations (JavaDocs) for details.\n") ;

		// Producing output
		System.out.println(message) ;
		return message ;
	}

} // END OF CLASS