/**
* Title: Database
* Description:
* Copyright:    Copyright (c) 2009
* Company:      Morpheus Quality Technologies
* @author Stefano Adriani
* @version 1.0
*/

package adriani.jws.template.db ;

import adriani.jws.template.* ;
import adriani.jws.template.util.* ;

import java.sql.* ;
import java.util.* ;

/**
* The databridge class used to retrieve and return JDBC connection from the database. To obtain a connection,
* users should use a code block like this:<br/>
*
* <pre><code>
*  		Connection connection = Database.getJDBCPool().getConnection() ;
*		// Perform your DB operation here
*		Database.getJDBCPool().returnConnection(connection) ;
* </code></pre><br/>
*
* Moreover this class holds some static reference labels and parameters common to all databases.<br/>
* Example: the <code>Database.NAME</code> labels and the <code>timeout</code> parameter.
*/
 public class Database {
 
  	/** Reference code for the DB_01 database.																*/
	public static final 	int 					DB_01 			= 0 ;
  	/** Reference code for the DB_02 database.																*/
	public static final 	int 					DB_02 			= 1 ;
  	/** Total number of databases handled by this class.													*/
	public static final 	int 					DB_NUMBER		= 2 ;
	
	/** 
	* Specifies if connections are retrieved from a pure JDBC pool (the <code>JDBCConnectionPool</code> class)
	* or from a mocked connection pool (the <code>MockConnectionPool</code> class).		
	*/
	protected static		boolean					mock ;

	/** Array of JDBC connection pool for the databases handled by this class.								*/
	protected static		JDBCConnectionPool[]	pool ;

	/** Sleep time for the cleaning thread for ALL the JDBC pools (in seconds).								*/
	protected static		int 					timeout ;
	
	/** Maximum allowed number of connections for ALL the JDBC pools.										*/
	protected static 		int 					maximum ;

	/** Name of the current JDBC driver (for testing or debug).												*/
	protected static  		String 					driver ;

	/** Absolute path of the <code>tnsnames.ora</code> (for testing or debug).								*/
	protected static		String 					tnsnames ;
	
	/** Beginning of each logging message produced by this class.											*/
	protected static 		String 					leadingName		= "Database - " ;

	/** Retrieving the logger instance to be used for this class (static).									*/
	protected static 		WSLogger				logger 			= WSLogger.getLogger(Database.class) ;
	
	// Creation of static parameters, performed when the Database Class is loaded (not instanciated)
	// Remark: these variables are not class fields, hence they will not be visible outside this codeblock
	static {

		// 1. Setting DEFAULT values (class fields, shared by all databases)
		timeout					= 900 ;		
		maximum					= 100 ;		
		driver					= "oracle.jdbc.driver.OracleDriver" ;
		tnsnames				= "/home/wsadm/tnsnames.ora" ;		
		
		// 2. Defining arrays (local variables)
		String[] alias			= new String[DB_NUMBER] ;
		String[] user			= new String[DB_NUMBER] ;
		String[] password		= new String[DB_NUMBER] ;
		String[] address		= new String[DB_NUMBER] ;
		String[] host 			= new String[DB_NUMBER] ;
		String[] schema 		= new String[DB_NUMBER] ;
		
		// 3. Setting DEFAULT values (local variables)
		alias		[0]			= "my_schema" ;
		user		[0]			= "user" ;
		password	[0]			= "password" ;
		address		[0]			= "jdbc:oracle:thin:@192.168.1.10:1522:db_name_01" ;
		alias		[1]			= "another_schema" ;
		user		[1]			= "scott" ;
		password	[1]			= "tiger" ;
		address		[1]			= "jdbc:oracle:thin:@192.168.1.11:1533:db_name_02" ;

		// 4. Parsing properties from resource bundle
		try {

			// 4.1 Retrieving general properties
			ResourceBundle rb	= ResourceBundle.getBundle(Settings.CONFIG) ;
			driver				= rb.getString("jdbc.driver") ;

			// 4.2 Retrieving properties for the DB_01 database
			for (int i = 0 ; i < DB_NUMBER ; i++) {
				int index = i + 1 ;
				host		[i]		= rb.getString("db_0" + index + ".host"		) ;
				schema		[i]		= rb.getString("db_0" + index + ".schema"	) ;
				user		[i]		= rb.getString("db_0" + index + ".user"		) ;
				password	[i]		= rb.getString("db_0" + index + ".password"	) ;
				alias		[i]		= rb.getString("db_0" + index + ".alias"	) ;				
				address		[i]		= "jdbc:oracle:thin:@" + host[i] +":" + schema[i] ;
			}
			
			// 4.3 Parsing boolean properties
			String mocked		= rb.getString("pool.mocked") ;			
			mock				= mocked.equalsIgnoreCase("YES") ;

			// 4.4 Parsing integer properties
			String value		= rb.getString("pool.timeout") ;
			try 				{ timeout = Integer.parseInt(value.trim()) ;	}
			catch(Exception e) 	{ timeout = 900 ;								}	// Default value (15 minutes)
			value				= rb.getString("pool.maximum") ;
			try 				{ maximum = Integer.parseInt(value.trim()) ;	}
			catch(Exception e) 	{ maximum = 100 ;								}	// Default value

			// 4.5 Setting system properties for tns system
			tnsnames = rb.getString("oracle.tnsnames") ;
			System.setProperty("oracle.net.tns_admin", tnsnames) ;
			if (Settings.DETAILS) System.out.println(Database.leadingName + "Loaded configuration file") ;

		} catch(MissingResourceException e) {
			System.err.println(Database.leadingName + "DB configuration properties file not found: using defaults") ;
			e.printStackTrace() ;
		}

		// 5. Creating the array of proper connection pool types (pure JDBC or mocked).
		if (mock) {
			pool = new MockConnectionPool[DB_NUMBER] ;
			logger.logInfo(Database.leadingName + "using mocked JDBC connection pool") ;
		} else {
			pool = new JDBCConnectionPool[DB_NUMBER] ;
			logger.logInfo(Database.leadingName + "using pure JDBC connection pool") ;
		}
		
		// 6. Creating the static reference to the necessary JDBCConnectionPool objects.
		// Note that we register the JDBC driver only when creating the 1st pool.
		for (int i = 0 ; i < DB_NUMBER ; i++) {
			if (i == 0) {
				if (mock) 	pool[i] = new MockConnectionPool(driver, address[i], user[i], password[i], timeout) ;
				else		pool[i] = new JDBCConnectionPool(driver, address[i], user[i], password[i], timeout) ;
			} else {
				if (mock) 	pool[i] = new MockConnectionPool(address[i], user[i], password[i], timeout) ;
				else 		pool[i] = new JDBCConnectionPool(address[i], user[i], password[i], timeout) ;
			}
			// 6.1 Configuring the pool
			pool[i].setTnsName(alias[i]) ;
			pool[i].setPoolSize(maximum) ;
		}
		
	} // End of static code-block

// **********************************************************************************************************
// *****************************			  ACCESSORS				*****************************************
// **********************************************************************************************************

	/** 
	* Specifies if connections are retrieved from the pure JDBC pool (i.e. the <code>JDBCConnectionPool</code>)
	* or from the <code>MockConnectionPool</code>. Notice that all other information about the pool (timeout,
	* maximum limit etc.) can be retrieved by inspecting the proper pool instance as obtained by the method 
	* <code>getJDBCPool(int targetDB)</code> method.
	*
	* @return <code>true</code> if this class relies on an instance of the <code>MockConnectionPool</code>, 
	* <code>false</code> if the standard <code>JDBCConnectionPool</code> is used
	*/
	public static boolean usesMockedPool() 						{		return mock ;						}

	/**
	* Return the <code>JDBCConnectionPool</code> object managing all the JDBC connections to the database 
	* specified in input, according to the static fields reference defined by this class.<br/>
	* The returned pool can be a pure <code>JDBCConnectionPool</code> object or another valid subclass,
	* like the <code>MockConnectionPool</code> class.
	*
	* Example: <code>getJDBCPool(Datase.DB_01)</code> will return the JDBC pool for the DB_01 database.<br/>
	*
	* @param a reference to the database (see static class field definitions)
	* @return the instance of the <code>JDBCConnectionPool</code> for the input database
	* @see usesMockedPool()
	*/
	public static JDBCConnectionPool getJDBCPool(int targetDB) {		return pool[targetDB] ;			}
	
// **********************************************************************************************************
// *****************************			 POOL INTERFACE				*************************************
// **********************************************************************************************************
	
	/**
	* Return a new <code>MockConnection</code> wrapper containing the SQL connection, by retrieving
	* it from the proper pool (pure or mocked).
	*
	* @param a reference to the database (see static class field definitions)
	* @return the instance of the <code>JDBCConnectionPool</code> for the input database
	*/
	public static MockConnection getMockConnection(int targetDB) throws SQLException {
	
		if (mock) {
			MockConnectionPool pool = (MockConnectionPool) getJDBCPool(targetDB) ;
			return pool.getMockConnection() ;
		} else {
			JDBCConnectionPool pool = getJDBCPool(targetDB) ;
			Connection connection = pool.getConnection() ;
			return new MockConnection(connection) ;
		}
	}
	
	/**
	* Return an already opened SQL connection, wrapped by a <code>MockConnection</code> class,
	* to the proper pool that produced the connection.
	*
	* @param a reference to the database (see static class field definitions)
	* @return the instance of the <code>JDBCConnectionPool</code> for the input database
	*/
	public static void returnMockConnection(int targetDB, MockConnection wrapper) {
	
		if (wrapper == null) return ;
	
		if (mock) {
			MockConnectionPool pool = (MockConnectionPool) getJDBCPool(targetDB) ;
			pool.returnMockConnection(wrapper) ;
		} else {
			JDBCConnectionPool pool = getJDBCPool(targetDB) ;
			Connection connection = wrapper.getConnection() ;
			pool.returnConnection(connection) ;
		}
	}

// **********************************************************************************************************
// *****************************			DATABASE METADATA			*************************************
// **********************************************************************************************************

	/**
	* Print in the LOG file and on the STDOUT some metadata information about ALL the databases used
	* by this project.
	*/
	public static void logMetaData() {

		for (int i = 0 ; i < DB_NUMBER ; i++) logMetaData(i) ;
	}

	/**
	* Print in the LOG file and on the STDOUT some metadata information about the database selected by the 
	* input parameter, according to the static fields reference defined by this class.<br/><br/>
	*
	* Example: <code>logMetaData(Datase.DB_01)</code> will show metadata about the DB_01 database.<br/>
	*
	* @param a reference to the database (see static class field definitions)
	*/
	public static void logMetaData(int target) {

		StringBuffer message = showMetaData(target) ;
		if (message != null) logger.logInfo( message.toString() ) ;
	}

	/**
	* Display on the console line (<code>stdout</code>) some metadata information about the database
	* selected by the input <code>target</code> parameter, according to the static fields reference 
	* defined by this class.<br/><br/>
	*
	* Example: <code>showMetaData(Datase.DB_01)</code> will show metadata about the DB_01 database.<br/>
	*
	* @param a reference to the database (see static class field definitions)
	*/
	public static StringBuffer showMetaData(int target) {

		// Initiliazing
		StringBuffer result = null ;
		Connection connection = null ;
		JDBCConnectionPool pool = getJDBCPool(target) ;

		// Retrieving a connection from the target database (via the MockConnection wrapper)
		try {
			connection = getMockConnection(target).getConnection() ;
		} 
		catch(Exception e) {
			System.err.println(Database.leadingName + "Error getting the connection") ;
			e.printStackTrace() ;
		}

		// Getting metadata from the connection and the JDBC pool
		if (connection != null) try {
			result = getMetaData(connection) ;
			result.append("JDBC pool mocked                   : " + mock + "\n") ;
			result.append("JDBC pool using TNS names          : " + pool.supportTnsNames() + "\n") ;
			result.append("Sleep time for the cleaning thread : " + timeout + " s\n") ;			
			result.append("TNS names oracle file              : " + tnsnames + "\n") ;
			result.append("JDBC pool object                   : " + pool + "\n") ;
			result.append("**********************************************************************\n") ;
		} catch(SQLException ex) {
			System.err.println(Database.leadingName + "Error retrieving metadata") ;
			ex.printStackTrace() ;
		}

		return result ;
	}

	/**
	* Produce the metadata information about the input database's connection object and format this data
	* into a <code>StringBuffer</code> object.
	*
	* @return a printable string containing metadata information about the database
	*/
	public static StringBuffer getMetaData(Connection connection) throws SQLException {

		Driver driver = null ;
		String driverPath = null ;
		DatabaseMetaData metadata = connection.getMetaData() ;

		// Detecting the path of the JDBC driver
		try {
			driver = (Driver) Class.forName(Database.driver).newInstance() ;
			java.net.URL url = driver.getClass().getProtectionDomain().getCodeSource().getLocation() ;
			driverPath = url.getPath() ;
		} catch(Exception e) { 	System.err.println(Database.leadingName + "Meta Data error") ;		}

		// Producing the string message
		StringBuffer message = new StringBuffer() ;
		message.append("\n********************* DATABASE INFORMATION ***************************\n") ;
		message.append("Connected to (tsn):       " + metadata.getURL() + "\n") ;
		message.append("Driver:                   " + metadata.getDriverName() + "\n") ;
		message.append("Driver class:             " + driver.getClass() + "\n") ;
		message.append("Driver archive:           " + driverPath + "\n") ;
		message.append("Driver version:           " + metadata.getDriverVersion() + "\n") ;
		message.append("Support Batch Updates?    " + metadata.supportsBatchUpdates() + "\n") ;
		message.append("Support Stored Functions? " + metadata.supportsStoredProcedures() + "\n") ;
		message.append("Auto commit mode?         " + connection.getAutoCommit() + "\n\n") ;
		message.append("Current isolation lvl:    " + connection.getTransactionIsolation() + "\n") ;
		message.append("Default isolation lvl:    " + metadata.getDefaultTransactionIsolation() + "\n") ;
		message.append("Where:\n") ;
		message.append("   Level " + java.sql.Connection.TRANSACTION_READ_UNCOMMITTED + " = TRANSACTION_READ_UNCOMMITTED\n") ;
		message.append("   Level " + java.sql.Connection.TRANSACTION_READ_COMMITTED + " = TRANSACTION_READ_COMMITTED\n") ;
		message.append("   Level " + java.sql.Connection.TRANSACTION_REPEATABLE_READ + " = TRANSACTION_REPEATABLE_READ\n") ;
		message.append("   Level " + java.sql.Connection.TRANSACTION_SERIALIZABLE + " = TRANSACTION_SERIALIZABLE\n\n") ;

		return message ;
	}
	
	/**
	* Logs some information about the status of the JDBC pools used by this database class.<br/>
	* In current implementation this method is executed when the web service receive the command 00000011.
    */
    public static void logPoolsData() {

		StringBuffer message = new StringBuffer() ;
		message.append("\n") ;
		for (int i = 0 ; i < DB_NUMBER ; i++) {
			message.append("JDBCPool " + i + " : " + pool[i].getPoolData() + "\n") ;
		}
		if (message != null) logger.logInfo( message.toString() ) ;
	}
	
} // END OF CLASS