/**
* Title: GeneralDBA
* Description:
* Copyright:    Copyright (c) 2003-2010
* Company:      Morpheus Quality Technologies
* @author Stefano Adriani
* @version 1.0
*/

package adriani.jws.template.db ;

import adriani.jws.template.Settings ;
import adriani.jws.template.util.* ;
import adriani.jws.template.service.* ;

import java.sql.*;
import java.util.*;

/**
* GeneralDBA - general class used to provide some common methods to all database accessors (DBA classes).<br/>
* This class defines only general purposes objects: class fields, double quoting methods, introspection methods,
* standard accessors and benchmark analysis. All connections related issues SHOULD NOT be implemented at this
* level, but should be left to the children classes, in order to reflect the detaileds requirements of the
* web service business logic.
*/
public abstract class GeneralDBA {

  	/** Status of the DBA after execution of the last SQL statement. 										*/
	protected 				int 			status ;

	/** Long timestamp used to evaluate performance time when the ANALYSIS mode is set.						*/
	protected				long 			chronStart ;

	/** Static final <code>String</code> value stored as class fields to improve performances.				*/
	protected static final	String			EMPTY				= "" ;
	/** Static final <code>String</code> value stored as class fields to improve performances.				*/
	public static final 	String			QUOTE 				= "'" ;
	/** Static final <code>String</code> value stored as class fields to improve performances.				*/
	public static final 	String			DOUBLE_QUOTES 		= "''" ;

	/** The message to print in the case of a general Java <code>Exception</code> .									*/
	protected static final	String			ERROR	 			= "Java error:\n\n"  ;
  	/** The message to print in the case of a <code>SQLException</code> .									*/
	protected static final	String			QUERY_NOK 			= "query NOK:\n\n"  ;
  	/** The message to print in the case of a rollback failure.												*/
	protected static final	String			ROLLBACK_NOK 		= "rollback-failed: "  ;
	/** The message to print if preparation of the prepared statement failed.								*/
	protected static final 	String 			PSTMT_NOK 			= "failed preparation of the prepared statement " ;
	/** The message to print if the query returned no data at all.											*/
	protected static final 	String 			NO_RESULT 			= "no data found" ;

	/** The integer representing the Oracle error code ORA-01403: no data found.							*/
	protected static final	int				NO_DATA_FOUND		= 1403  ;

	/** Retrieving the logger instance to be used for this class (static).									*/
	protected static 		WSLogger		logger 				= WSLogger.getLogger(GeneralDBA.class) ;

	/** The leading name of each message produced by thsi class.											*/
	protected 				String 			leadingName 		= "General DBA - " ;

	/**
	* Empty constructor.
    */
    public GeneralDBA() {

		if (Settings.DETAILS) System.out.println(leadingName + "Created object: " + this) ;
	}

	/**
	* Method useful for debug and testing, the verify the scope of the children classes.
    */
    public void finalize() {

		if (Settings.DETAILS) System.out.println(leadingName + "Finalized object: " + this) ;
	}

// **********************************************************************************************************
// *****************************		CONNECTIONS MANAGEMENT		*****************************************
// **********************************************************************************************************

	/**
	* Return the connection currently opened through the target database, for example:
	* <br/><br/>
	* targetDB = 1 --> get the connection opened to the DB_01 database.<br/>
	* targetDB = 2 --> get the connection opened to the DB_02 database.<br/>
	* etc...
	* <br/><br/>
	* For the database index reference see the class fields of the <code>Database</code> class.
	* <br/><br/>
	* It's up to the specific subclass decide how to obtain and return database connections.
	*
	* @param targetDB the number of the database offering the connection
	* @return the currently opened connection to the target database
	* @see Database
    */
    public abstract Connection getConnection(int targetDB) ;

// **********************************************************************************************************
// ************************************		PERFORMANCE ANALYSIS		*************************************
// **********************************************************************************************************

	/**
	* Log performances data on the <em>Performarces Analyser</em> and flush the relative Handler.
	* This mehod automatically adds a detailed timestamp to the logged message, and store the
	* starting timestamp in the <code>chronStart</code> class field.
	*
	* @param msg the string text name of the query that is going to be executed
	*/
	public void startQuery(String msg) {

		this.chronStart = System.currentTimeMillis() ;
		java.util.logging.Logger logger = java.util.logging.Logger.getLogger(Settings.BENCHMARK) ;
		logger.info("(" + chronStart + ") START QUERY: " + msg) ;
		flushLogger(logger) ;
	}

	/**
	* Log performances data on the <em>Performarces Analyser</em> and flush the relative Handler.
	* This mehod compares the query end time with the query starting time and log the query
	* duration time in the performances logger.
	*
	* @param msg the string text name of the query that has just been terminated
	*/
	public void endQuery(String msg) {

		long endTime = System.currentTimeMillis() ;
		java.util.logging.Logger logger = java.util.logging.Logger.getLogger(Settings.BENCHMARK) ;
		logger.info("(" + endTime + ") END QUERY: " + msg + "   Time= " + (endTime - chronStart) +  " ms") ;
		flushLogger(logger) ;
	}

	/**
	* Flush the Handler bound to the given <code>Logger</code>. Notice that the <strong>benchmark</strong>
	* feature relies always on the <code>SystemLogger</code> class, never on the Log4j mechanism.
	*
	* @param logger a reference to the logger object to be flushed
	*/
	public void flushLogger(java.util.logging.Logger logger) {

		java.util.logging.Handler[] handlers = logger.getHandlers() ;
		if ( (handlers.length > 0) && (handlers[0] != null) ) handlers[0].flush() ;
	}

	/**
	* Subclasses SHOULD override this method in order to log the final execution status of a general
	* SQL statement. For example, if the subclass relies on the <code>ServiceAnalyzer</code> mechanism
	* for collecting statistics about SQL execution status, implementation of this method could be
	* something like:<br/><br/>
	*
	* <code>serviceMacro.getServiceAnalyzer().collect(status) ;</code><br/><br/>
	*
	* Current implementation (of this class) does nothing.
	*
	* @param statusValue the integer status as defined by the <code>ErrorMessage</code> class fields.
	* @see ErrorMessage
	*/
	public void collectStatus(int statusValue) {

		//collectStatus(status) ;
	}

// ************************************************************************************************************
// ********************************			GENERAL SQL QUERIES			***************************************
// ************************************************************************************************************

	/**
	* Executes a SELECT query to the target database, returning a <strong>single record</strong> described
	* by a <code>StringBuffer</code> array. The length of the array is defined by the <code>cols</code>
	* input parameter, i.e. the number of columns retrieved from the database.
	*
	* @param targetDB the number of the database where to execute the query
	* @param label the logical name of the statement, only for logging purposes
	* @param query the SQL statement to be executed, already filled with parameters
	* @param cols the number of columns to be retrieved from the database
	* @return a <code>StringBuffer</code> array containing the selected columns (from the same record)
    */
    public StringBuffer[] getRecord(int targetDB, String label, String query, int cols) {

		// Preparing variables
		status = ErrorMessage.DBA ;
		StringBuffer[] output = null ;
		if (Settings.BENCH) startQuery(label) ;

		// Choosing database's connection
		Connection connection = this.getConnection(targetDB) ;

		// Executing statement
		try {
	 		Statement stmt		= connection.createStatement() ;
	  		ResultSet result 	= stmt.executeQuery(query) ;
			//System.out.println("Executed query: " + query) ;

			// Reading data only if we find a valid result set
			if ( result.isBeforeFirst() ) {
				output = new StringBuffer[cols] ;
				status = ErrorMessage.OK ;
				result.next() ;
				for (int i = 0 ; i < cols ; i++) {
					String data = result.getString(i + 1) ;
					if (data != null) output[i] = new StringBuffer(data) ;
				}
				if (Settings.DETAILS) {
					System.out.print(leadingName + label) ;
					for (int i = 0 ; i < cols ; i++) if (output[i] != null) System.out.print(output[i].toString() + " ") ;
					System.out.println() ;
				}
			}
			else status = ErrorMessage.NO_RECORD ;
			stmt.close() ;

		} catch(SQLException e) {
			status = ErrorMessage.SQL ;
			logger.logSevere(leadingName + label + QUERY_NOK + query + "\n" + e.getMessage()) ;
	  		if (Settings.DEBUG) e.printStackTrace() ;
	  		try 					{ 	if (connection != null) connection.rollback() ;			  				}
	  		catch(SQLException rbe) { 	System.err.println(ROLLBACK_NOK + rbe.getMessage()) ;					}
		} catch(Exception e) {
			status = ErrorMessage.DBA ;
			logger.logSevere(leadingName + label + ERROR + query + "\n" + e.getMessage()) ;
	  		if (Settings.DEBUG) e.printStackTrace() ;
		}

		// Eventually logging results
		if (Settings.BENCH) endQuery(label) ;
		if (Settings.ANALYZE) collectStatus(status) ;
		if (status == ErrorMessage.NO_RECORD) logger.logWarning(leadingName + label + ErrorMessage.getMessage(status)) ;

		return output ;
	}

	/**
	* Executes a SELECT query to the target database, returning a <strong>set of records</strong> described
	* by an array which elements are others <code>StringBuffer</code> arrays:<br/><br/>
	*
	* <code>StringBuffer[n][i]</code><br/><br/>
	*
	* contains the i-th column of the n-th record.<br/>
	*
	* @param targetDB the number of the database where to execute the query
	* @param label the logical name of the statement, only for logging purposes
	* @param query the SQL statement to be executed, already filled with parameters
	* @param cols the number of columns to be retrieved from the database
	* @return a <code>StringBuffer</code> double array containing all the matching records
    */
    public StringBuffer[][] getRecordSet(int targetDB, String label, String query, int cols) {

		// Preparing variables
		status = ErrorMessage.DBA ;
		StringBuffer[] record = null ;
		StringBuffer[][] output = null ;
		if (Settings.BENCH) startQuery(label) ;

		// Choosing database's connection
		Connection connection = this.getConnection(targetDB) ;

		// Executing statement
		try {
	 		Statement stmt = connection.createStatement() ;
	  		ResultSet result = stmt.executeQuery(query) ;
			//System.out.println("Executed query: " + query) ;

			// Reading data only if we find a valid result set
			if ( result.isBeforeFirst() ) {
				ArrayList list = new ArrayList() ;
				status = ErrorMessage.OK ;
				while (result.next()) {
					record = new StringBuffer[cols] ;
					for (int i = 0 ; i < cols ; i++) {
						String data = result.getString(i + 1) ;
						if (data != null) record[i] = new StringBuffer(data) ;
						//if (i == 5) logger.logInfo("Column:#" + record[i] + '#') ;
					}
					list.add(record) ;
				}
				int size = list.size() ;
				output = (StringBuffer[][]) list.toArray( new StringBuffer[size][cols] ) ;
				if (Settings.DETAILS) System.out.println(leadingName + label + "found " + size + " records") ;
				// Eventually printing the results: KEEP FOR TESTING OR DEBUG
				//for (int n = 0 ; n < size ; n++) { for (int i = 0 ; i < cols ; i++) if (output[n][i] != null) System.out.print(output[n][i].toString() + " ") ; System.out.println() ; }
			}
			else status = ErrorMessage.NO_RECORD ;
			stmt.close() ;

		} catch(SQLException e) {
			status = ErrorMessage.SQL ;
			logger.logSevere(leadingName + label + QUERY_NOK + query + "\n" + e.getMessage()) ;
	  		if (Settings.DEBUG) e.printStackTrace() ;
	  		try 					{ 	if (connection != null) connection.rollback() ;			  				}
	  		catch(SQLException rbe) { 	System.err.println(ROLLBACK_NOK + rbe.getMessage()) ;					}
		} catch(Exception e) {
			status = ErrorMessage.DBA ;
			logger.logSevere(leadingName + label + ERROR + query + "\n" + e.getMessage()) ;
	  		if (Settings.DEBUG) e.printStackTrace() ;
		}

		// Eventually logging results
		if (Settings.BENCH) endQuery(label) ;
		if (Settings.ANALYZE) collectStatus(status) ;
		if (status == ErrorMessage.NO_RECORD) logger.logWarning(leadingName + label + ErrorMessage.getMessage(status)) ;

		return output ;
	}

// ************************************************************************************************************
// ************************************			PL/SQL FUNCTIONS		***************************************
// ************************************************************************************************************

	/**
	* Executes a PL/SQL FUNCTION on the target database, returning a <strong>single record</strong> wrapped
	* by a <code>StringBuffer</code> array.<br/>
	* Current implementation relies on the <code>{? = call FUNCTION(?, ?, ?, ....)}</code> syntax,
	* because is generally more performing than the alternative approaches:
	* <br/><br/>
	* <code>SELECT FUNCTION(x, y, z, ...) FROM DUAL</code> (ordinary plain java)<br/>
	* <code>SELECT FUNCTION(?, ?, ?, ...) FROM DUAL</code> (JDBC prepared statement)<br/>
	*
	* @param targetDB the number of the database where to execute the query
	* @param label the logical name of the statement, only for logging purposes
	* @param query the prepared call statement to be executed, not yet filled with parameters
	* @param param the <code>StringBuffer</code> array containing the input parameters
	* @return a <code>StringBuffer</code> array containing the selected columns
    */
    public StringBuffer[] getFunction(int targetDB, String label, String query, StringBuffer[] param) {

		// Preparing variables
		int cols = 1 ;
		int errorCode = 0 ;
		status = ErrorMessage.DBA ;
		StringBuffer[] output = null ;
		if (Settings.BENCH) startQuery(label) ;

		// Choosing database's connection
		Connection connection = this.getConnection(targetDB) ;

		// Executing statement
		try {
			CallableStatement call = connection.prepareCall(query) ;
			// Setting output parameters
			call.registerOutParameter(1, java.sql.Types.VARCHAR) ;
			// Setting input parameters
			for (int i = 0 ; i < param.length ; i++) {
				int n = cols + 1 + i ;
				call.setString(n, param[i].toString()) ;
				//System.out.println("Registered input " + n) ;
			}
	  		call.execute() ;
			//System.out.println("Executed call: " + query) ;

			// Reading output data
			status = ErrorMessage.OK ;
			output = new StringBuffer[cols] ;
			for (int i = 0 ; i < cols ; i++) {
				String data = call.getString(i + 1) ;
				if (data != null) output[i] = new StringBuffer(data) ;
					else status = ErrorMessage.NO_RECORD ;
			}
			if (Settings.DETAILS) {
				System.out.print(leadingName + label) ;
				for (int i = 0 ; i < cols ; i++) if (output[i] != null) System.out.print(output[i].toString() + " ") ;
				System.out.println() ;
			}
			call.close() ;

		} catch(SQLException e) {
			status = ErrorMessage.SQL ;
			errorCode = e.getErrorCode() ;
			// Avoiding to log a critical message if we just did not find any data
			if (errorCode != NO_DATA_FOUND) {
				logger.logSevere(leadingName + label + QUERY_NOK + query + "\n" + e.getMessage()) ;
				if (Settings.DEBUG) e.printStackTrace() ;
			} else {
				logger.logSevere(leadingName + label + NO_RESULT) ;
			}
			// Trying to rollback anyway
	  		try 					{ 	if (connection != null) connection.rollback() ;			  				}
	  		catch(SQLException rbe) { 	System.err.println(ROLLBACK_NOK + rbe.getMessage()) ;					}
		} catch(Exception e) {
			status = ErrorMessage.DBA ;
			logger.logSevere(leadingName + label + ERROR + query + "\n" + e.getMessage()) ;
	  		if (Settings.DEBUG) e.printStackTrace() ;
		}

		// Eventually logging results
		if (Settings.BENCH) endQuery(label) ;
		if (Settings.ANALYZE) collectStatus(status) ;
		if (status == ErrorMessage.NO_RECORD) logger.logWarning(leadingName + label + ErrorMessage.getMessage(status)) ;

		return output ;
	}

	/**
	* Executes a PL/SQL FUNCTION on the target database, returning a <strong>set of records</strong> described
	* by an array which elements are others <code>StringBuffer</code> arrays:<br/><br/>
	*
	* <code>StringBuffer[n][i]</code><br/><br/>
	*
	* contains the i-th column of the n-th record.<br/>
	*
	* Current implementation is just a wrapper to the <code>getRecordSet()</code> method, since the best way
	* to execute a function returning a pipe is to use an ordinary SELECT statement.
	*
	* @param targetDB the number of the database where to execute the query
	* @param label the logical name of the statement, only for logging purposes
	* @param query the SQL statement to be executed, already filled with parameters
	* @param cols the number of columns to be retrieved from the database
	* @return a <code>StringBuffer</code> double array containing all the matching records
    */
    public StringBuffer[][] getFunctionPipe(int targetDB, String label, String query, int cols) {

		return getRecordSet(targetDB, label, query, cols) ;
	}

// ************************************************************************************************************
// ************************************			PL/SQL PROCEDURE		***************************************
// ************************************************************************************************************

	/**
	* Executes a PL/SQL PROCEDURE call to the target database, returning a <strong>single record</strong>
	* described by a <code>StringBuffer</code> array. The length of the output array is defined by the
	* <code>cols</code> input parameter (number of columns to be retrieved from the record).<br/>
	* If the procedure does not require any IN parameter the input <code>param</code> array must be null. 
	* If the procedure requires some IN OUT parameters, the index of these parameters MUST BE specified in
	* the <code>inout</code> input integer array. If there are no IN OUT parameters, this array MUST BE
	* <code>null</code>.<br/><br/>
	*
	* Example: to call the PL/SQL procedure <code>FOO(a IN, b IN OUT, c IN, d IN OUT, e OUT)</code>
	* this method must be invoked using: <code>param = {1,2,3,4} ; cols = 3 ; inout = {2,4}</code>.<br/><br/>
	*
	* @param targetDB the number of the database where to execute the query
	* @param label the logical name of the statement, only for logging purposes
	* @param query the SQL statement to be executed, not yet filled with parameters
	* @param param the <code>StringBuffer</code> array containing the input parameters
	* @param cols the number of output columns to be retrieved from the database
	* @param inout an int array specifying the index of the IN OUT parameters (null if there are no IN OUT params)
	* @return a <code>StringBuffer</code> array containing the selected columns
    */
    public StringBuffer[] getProcedure(int targetDB, String label, String query, StringBuffer[] param, int cols, int[] inout) {

		// Preparing variables
		status = ErrorMessage.DBA ;
		StringBuffer[] output = null ;
		if (Settings.BENCH) startQuery(label) ;

		// Choosing database's connection
		Connection connection = this.getConnection(targetDB) ;

		// Executing statement
		try {
			int bidir = 0 ;
			CallableStatement call = connection.prepareCall(query) ;
			// Setting IN parameters
			if (param != null) for (int i = 0 ; i < param.length ; i++) {
				int n = i + 1 ;
				call.setString(n, param[i].toString()) ;
				//System.out.println("Registered input " + n + " : " + param[i]) ;
			}
			// Setting IN OUT parameters (if any)
			if (inout != null) for ( ; bidir < inout.length ; bidir++) {
				call.registerOutParameter(inout[bidir], java.sql.Types.VARCHAR) ;
				//System.out.println("Registered input-output " + inout[bidir]) ;
			}
			// Setting "pure" OUT parameters (if any)
			for (int i = 0 ; i < (cols - bidir) ; i++) {
				int n = param.length + 1 + i ;
				call.registerOutParameter(n, java.sql.Types.VARCHAR) ;
				//System.out.println("Registered output " + n) ;
			}
	  		call.execute() ;
			//System.out.println("Executed call: " + query) ;

			// Reading output data
			status = ErrorMessage.OK ;
			output = new StringBuffer[cols] ;
			// Retrieving IN OUT parameters (if any)
			if (inout != null) for (int i = 0 ; i < inout.length ; i++) {
				String data = call.getString(inout[i]) ;
				if (data != null) output[i] = new StringBuffer(data) ;
				else status = ErrorMessage.NO_RECORD ;
			}
			// Retrieving "pure" OUT parameters (if any)
			for (int i = 0 ; i < (cols - bidir); i++) {
				String data = call.getString(param.length + i + 1) ;
				if (data != null) output[i] = new StringBuffer(data) ;
				else status = ErrorMessage.NO_RECORD ;
			}
			// Eventually logging result on STDOUT
			if (Settings.DETAILS) {
				System.out.print(leadingName + label) ;
				for (int i = 0 ; i < cols ; i++) if (output[i] != null) System.out.print(output[i].toString() + " ") ;
				System.out.println() ;
			}
			call.close() ;

		} catch(SQLException e) {
			status = ErrorMessage.SQL ;
			logger.logSevere(leadingName + label + QUERY_NOK + query + "\n" + e.getMessage()) ;
	  		if (Settings.DEBUG) e.printStackTrace() ;
	  		try 					{ 	if (connection != null) connection.rollback() ;			  			}
	  		catch(SQLException rbe) { 	System.err.println(ROLLBACK_NOK + rbe.getMessage()) ;				}
		} catch(Exception e) {
			status = ErrorMessage.DBA ;
			logger.logSevere(leadingName + label + ERROR + query + "\n" + e.getMessage()) ;
	  		if (Settings.DEBUG) e.printStackTrace() ;
		}

		// Eventually logging results
		if (Settings.BENCH) endQuery(label) ;
		if (Settings.ANALYZE) collectStatus(status) ;
		if (status == ErrorMessage.NO_RECORD) logger.logWarning(leadingName + label + ErrorMessage.getMessage(status)) ;

		return output ;
	}

// ************************************************************************************************************
// ************************************			PREPARED STATEMENTS		***************************************
// ************************************************************************************************************

	/**
	* Executes a <code>PreparedStatement</code> to the target database, returning a <strong>single record</strong>
	* described by a <code>StringBuffer</code> array. The length of the array is defined by the <code>cols</code>
	* input parameter, i.e. the number of columns retrieved from the database.<br/><br/>
	*
	* <strong>Remark</strong>: if the database relies on the mock connection pooling, this method does
	* not close the prepared statement, in order to leave to the <code>Database</code> class the choise
	* to eventually close or cache the prepared statement.<br/>
	*
	* @param pstmt the <code>PreparedStatement</code> to be executed, already filled with parameters
	* @param label the logical name of the statement, only for logging purposes
	* @param cols the number of columns to be retrieved from the database
	* @return a <code>StringBuffer</code> array containing the selected columns
    */
    public StringBuffer[] getPreparedRecord(PreparedStatement pstmt, String label, int cols) {

		// Preparing variables
		status = ErrorMessage.DBA ;
		StringBuffer[] output = null ;
		if (Settings.BENCH) startQuery(label) ;

		// Executing statement
		try {
	  		ResultSet result = pstmt.executeQuery() ;
			//System.out.println("PreparedStatement: " + pstmt) ;

			// Reading data only if we find a valid result set
			if ( result.isBeforeFirst() ) {
				output = new StringBuffer[cols] ;
				status = ErrorMessage.OK ;
				result.next() ;
				for (int i = 0 ; i < cols ; i++) {
					String data = result.getString(i + 1) ;
					if (data != null) output[i] = new StringBuffer(data) ;
				}
				if (Settings.DETAILS) {
					System.out.print(leadingName + label) ;
					for (int i = 0 ; i < cols ; i++) if (output[i] != null) System.out.print(output[i].toString() + " ") ;
					System.out.println() ;
				}
			}
			else status = ErrorMessage.NO_RECORD ;
			// If we don't use the mocked pool we must close the statement to avoid the Oracle
			// error "maximum opened cursors exceeded"
			if ( !Database.usesMockedPool() ) pstmt.close() ;
			else result.close() ;

		} catch(SQLException e) {
			status = ErrorMessage.SQL ;
			logger.logSevere(leadingName + label + QUERY_NOK + "\n" + e.getMessage()) ;
	  		if (Settings.DEBUG) e.printStackTrace() ;
		}

		// Eventually logging results
		if (Settings.BENCH) endQuery(label) ;
		if (Settings.ANALYZE) collectStatus(status) ;
		if (status == ErrorMessage.NO_RECORD) logger.logWarning(leadingName + label + ErrorMessage.getMessage(status)) ;

		return output ;
	}

	/**
	* Executes a <code>PreparedStatement</code> to the target database, returning a <strong>set of records</strong>
	* described by an array which elements are others <code>StringBuffer</code> arrays, where:<br/><br/>
	*
	* <code>StringBuffer[n][i]</code><br/><br/>
	*
	* contains the i-th column of the n-th record.<br/><br/>
	*
	* <strong>Remark</strong>: if the database relies on the mock connection pooling, this method does
	* not close the prepared statement, in order to leave to the <code>Database</code> class the choise
	* to eventually close or cache the prepared statement.<br/>
	*
	* @param pstmt the <code>PreparedStatement</code> to be executed, already filled with parameters
	* @param label the logical name of the statement, only for logging purposes
	* @param cols the number of columns to be retrieved from the database
	* @return a <code>StringBuffer</code> double array containing all the matching records
    */
    public StringBuffer[][] getPreparedRecordSet(PreparedStatement pstmt, String label, int cols) {

		// Preparing variables
		status = ErrorMessage.DBA ;
		StringBuffer[] record = null ;
		StringBuffer[][] output = null ;
		if (Settings.BENCH) startQuery(label) ;

		// Executing statement
		try {
	  		ResultSet result = pstmt.executeQuery() ;
			//System.out.println("PreparedStatement: " + pstmt) ;

			// Reading data only if we find a valid result set
			if ( result.isBeforeFirst() ) {
				status = ErrorMessage.OK ;
				ArrayList list = new ArrayList() ;
				while (result.next()) {
					record = new StringBuffer[cols] ;
					for (int i = 0 ; i < cols ; i++) {
						String data = result.getString(i + 1) ;
						if (data != null) record[i] = new StringBuffer(data) ;
						//if (i == 5) logInfo("Column:#" + record[i] + '#') ;
					}
					list.add(record) ;
				}
				int size = list.size() ;
				output = (StringBuffer[][]) list.toArray( new StringBuffer[size][cols] ) ;
				if (Settings.DETAILS) System.out.println(leadingName + label + "found " + size + " records") ;
				// Eventually printing the results: KEEP FOR TESTING OR DEBUG
				//for (int n = 0 ; n < size ; n++) { for (int i = 0 ; i < cols ; i++) if (output[n][i] != null) System.out.print(output[n][i].toString() + " ") ; System.out.println() ; }
			}
			else status = ErrorMessage.NO_RECORD ;
			// If we don't use the mocked pool we must close the statement to avoid the Oracle
			// error "maximum opened cursors exceeded"
			if ( !Database.usesMockedPool() ) pstmt.close() ;
			else result.close() ;

		} catch(SQLException e) {
			status = ErrorMessage.SQL ;
			logger.logSevere(leadingName + label + QUERY_NOK + "\n" + e.getMessage()) ;
	  		if (Settings.DEBUG) e.printStackTrace() ;
		}

		// Eventually logging results
		if (Settings.BENCH) endQuery(label) ;
		if (Settings.ANALYZE) collectStatus(status) ;
		if (status == ErrorMessage.NO_RECORD) logger.logWarning(leadingName + label + ErrorMessage.getMessage(status)) ;

		return output ;
	}
	
// **********************************************************************************************************
// **************************************		UPDATE QUERIES			*************************************
// **********************************************************************************************************
	
	/**
	* Execute the general "update" query passed in input. This method can be used any time the update is 
	* performed using normal statements.<br/>
	*
	* @param targetDB the number of the database where to execute the query	
	* @param label the logical name of the statement, only for logging purposes
	* @param query the query that must be executed
	* @return the number of rows inserted in the database
	*/
	public int executeUpdate(int targetDB, String label, String query) {

		int 		rows 		= 0 ;
		Statement 	stmt		= null ;
		Connection	connection	= this.getConnection(targetDB) ;		

		try {
	 		stmt = connection.createStatement() ;
	  		rows = stmt.executeUpdate(query) ;
	  		if (rows == 0) 	status = ErrorMessage.NO_RECORD ;
			else 			status = ErrorMessage.OK ;
			stmt.close() ;
			connection.commit() ;
		} catch(SQLException e) {
			status = ErrorMessage.SQL ;
			logger.logSevere(leadingName + label + QUERY_NOK + query + "\n" + e.getMessage()) ;
	  		if (Settings.DEBUG) e.printStackTrace() ;
	  		try 					{ 	if (connection != null) connection.rollback() ;			  				}
	  		catch(SQLException rbe) { 	System.err.println(ROLLBACK_NOK + rbe.getMessage()) ;					}
		} catch(Exception e) {
			status = ErrorMessage.DBA ;
			logger.logSevere(leadingName + label + ERROR + query + "\n" + e.getMessage()) ;
	  		if (Settings.DEBUG) e.printStackTrace() ;			
		}

		// Eventually logging results
		if (Settings.BENCH) endQuery(label) ;						
		if (Settings.ANALYZE) collectStatus(status) ;				
		if (status == ErrorMessage.NO_RECORD) logger.logWarning(leadingName + label + ErrorMessage.getMessage(status)) ;

	  	return rows ;
	}
	
	/**
	* Executes a <code>PreparedStatement</code> to the target database, returning an int number describing the
	* number of affected rows.
	*
	* @param pstmt the <code>PreparedStatement</code> to be executed, already filled with parameters
	* @param label the logical name of the statement, only for logging purposes
	* @return the number of rows inserted in the database
    */
    public int executeUpdate(PreparedStatement pstmt, String label) {

		// Preparing variables
		int rows = 0 ;
		status = ErrorMessage.DBA ;
		if (Settings.BENCH) startQuery(label) ;				
		
		// Executing statement
		try {
			//System.out.println(leadingName + "PreparedStatement: " + pstmt) ;
	  		rows = pstmt.executeUpdate()  ;
	  		if (rows == 0) 	status = ErrorMessage.NO_RECORD ;
			else 			status = ErrorMessage.OK ;
			pstmt.close() ;
			if (Settings.DETAILS) System.out.println(leadingName + label + "update " + rows + " rows") ;
		} catch(SQLException e) {
			status = ErrorMessage.SQL ;
			logger.logSevere(leadingName + label + QUERY_NOK + "\n" + e.getMessage()) ;
	  		if (Settings.DEBUG) e.printStackTrace() ;
		}

		// Eventually logging results
		if (Settings.BENCH) endQuery(label) ;						
		if (Settings.ANALYZE) collectStatus(status) ;
		if (status == ErrorMessage.NO_RECORD) logger.logWarning(leadingName + label + ErrorMessage.getMessage(status)) ;

		return rows ;
	}

// **********************************************************************************************************
// **************************************		ACCESSORS			*****************************************
// **********************************************************************************************************

	/**
	* Return an int code describing the status of the DBA after execution of the last SQL-oriented label.
	*
	* @return an int code describing the status of the DBA after execution of the last SQL statement
	*/
	public int getStatus() 				{		return this.status ;										}

// **********************************************************************************************************
// **************************************		UTILITIES			*****************************************
// **********************************************************************************************************

	/**
	* Adds double quotation marks to the input String to avoid SQL error. In example, if the input String is
	* like <em>Video's</em>, the returned String will be: <em>Video''s</em>.<br/>
	*
	* @param input the string that can contain single quote characters
	* @return the same string with all single quotes converted to double quotes
	*/
	public String doubleQuote(String input) {

		if ( (input == null) || (input.length() <=0) ) return input ;
		String result = input.replaceAll(QUOTE, DOUBLE_QUOTES) ;
		return result ;
	}

	/**
	* Execute a <code>doubleQuote()</code> method on the input string, then surround the result with
	* a pair of single quotes.<br/>
	* Example: a string like <em>Video's</em> will become <em>'Video''s'</em>.
	*
	* @param input the string that can contain single quote characters
	* @return the same string with all single quotes converted to double quotes, enclosed within quotes
	*/
	public String quote(String input) {

		return "'" + doubleQuote(input) + "'" ;
	}
	
	/**
	* Parse the string value of the input <code>String</code> into an integer value.
	* The <code>no_value</code> parameter represent the default integer value to be returned if
	* the parsing is not succesfull.
	*
	* @param string the input string object
	* @param no_value the int value to be returned if conversion fails
	* @return the integer value of the input string (or the <code>no_value</code> number)
	*/
	protected int valueOfString(String string, int no_value) {

		int result = no_value ;
		try 				{	result = Integer.parseInt( string.trim() ) ;								}
		catch(Exception e) 	{ 	result = no_value ;															}
		return result ;
	}

// **********************************************************************************************************
// **************************************		INTROSPECTION		*****************************************
// **********************************************************************************************************
	
	/**
	* This method can be invoked immediately after the <code>stmt.executeQuery(query)</code> call
	* to inspect the retrieved <code>ResultSet</code>. The obtained meta data are printed on the
	* standard output (if available).
	*
	* @param rs the <code>ResultSet</code> as returned by the database
	*/
    public StringBuffer inspectResultSet(ResultSet rs) {

		StringBuffer message = new StringBuffer() ;

		try {
			message.append("\nRESULT SET METADATA:") ;
			message.append("ResultSet object: " + rs) ;
			message.append("ResultSet current row: " + rs.getRow()) ;
			message.append("Is after last: " + rs.isAfterLast()) ;
			message.append("Is before first: " + rs.isBeforeFirst()) ;
			message.append("Is first: " + rs.isFirst()) ;
			// Examining Fetch Direction
			int fd = rs.getFetchDirection() ;
			switch (fd) {
				case ResultSet.FETCH_UNKNOWN	: 	message.append("Fetch direction: UNKNOWN") ; 	break ;
				case ResultSet.FETCH_FORWARD	: 	message.append("Fetch direction: FORWARD") ; 	break ;
				case ResultSet.FETCH_REVERSE	: 	message.append("Fetch direction: REVERSE") ; 	break ;
			}
			// Examining Holdability
			int h = rs.getConcurrency() ;
			switch (h) {
				case ResultSet.CONCUR_READ_ONLY	:	message.append("Concurrency: READ_ONLY") ; 	break ;
				case ResultSet.CONCUR_UPDATABLE	:	message.append("Concurrency: UPDATABLE") ; 	break ;
			}
			// Listing metadata
			ResultSetMetaData metadata = rs.getMetaData() ;
			message.append("Metadata - Columns: " + metadata.getColumnCount()) ;
			message.append("") ;

		} catch(Throwable e) {
	  		System.err.println("Test query - Throwable: " + e.getMessage()) ;
			e.printStackTrace() ;
		}

		return message ;
	}

} // END OF CLASS