/**
* Title: JDBCConnectionPool
* Description:
* Copyright:    Copyright (c) 2009-2010
* Company:      Morpheus Quality Technologies
* @author Stefano Adriani
* @version 1.0
*/

package adriani.jws.template.db ;

import adriani.jws.template.* ;

import java.sql.* ;

/**
* JDBC oriented implementation of the <code>ObjectPool</code> parent class.<br/>
* Before ending or closing the application using the pool, the user should call the
* <code>shutdown()</code> method to properly terminate the cleaner thread.
*
* @see ObjectPool
*/
public class JDBCConnectionPool extends ObjectPool {

	/** DB address, equal to the return value of <code>DatabaseMetaData.getURL()</code>						*/
	protected 				String 			tns ;
	
	/** DB user name																						*/	
	protected 				String 			usr ;

	/** DB user password																					*/		
	protected 				String 			pwd ;
	
	/** DB tns alias, defined in the <code>tnsnames.ora</code> file (if available).							*/
	protected 				String 			alias ;
	
	/** If <code>true</code> this class can obtain SQL connection using the <code>tnsnames.ora</code> file. */
	protected 				boolean 		supportTns 		= false ;
	
	/**
	* Use this constructor if another <code>JDBCConnectionPool</code> has already registered the JDBC
	* driver for the same type of database related to this connection pool.<br/><br/>
	*
	* <strong>Remark</strong>: subclasses should override the method <code>getDatabaseName()</code> to
	* ensure proper logging leading messages.<br/>
	*
	* @param tns the tns string of the db, e.g. jdbc:oracle:thin:@192.168.1.10:1533:tartdati
	* @param usr the user name for the connection
	* @param pwd the password for the connection
	* @param expirationTime sleep time of the Cleaner Thread in seconds
	*/
	public JDBCConnectionPool(String tns, String usr, String pwd, int expirationTime) {

		// Remark: the super class will create and run the cleaner thread
		super(expirationTime) ;
		this.tns = tns ;
		this.usr = usr ;
		this.pwd = pwd ;
		this.leadingName = getDatabaseName(tns) ;
		logger.logInfo(leadingName + "Created JDBC ObjectPool instance: " + this) ;		
	}

	/**
	* Standard constructor, to be used to register the JDBC drivers (only the first time).
	*
	* @param driver the JDBC driver, e.g. oracle.jdbc.driver.OracleDriver
	* @param tns the tns string of the db, e.g. jdbc:oracle:thin:@192.168.1.10:1533:tartdati
	* @param usr the user name for the connection
	* @param pwd the password for the connection
	* @param expirationTime sleep time of the Cleaner Thread in seconds
	*/
	public JDBCConnectionPool(String driver, String tns, String usr, String pwd, int expirationTime) {

		this(tns, usr, pwd, expirationTime) ;

		// Creating the JDBC driver and storing a reference in the DriverManager
		try {
			// This code could be refactored using the "Service Provider" mechanism
			Driver driverJDBC = (Driver) Class.forName(driver).newInstance() ;
			DriverManager.registerDriver(driverJDBC) ;
			if (Settings.DETAILS) System.out.println(leadingName + "Registered JDBC driver") ;
		} catch (Exception e) {
			logger.logSevere(leadingName + "Driver not found:\n" + e) ;
			e.printStackTrace(System.err) ;
		}
		logger.logInfo(leadingName + "Created JDBC ObjectPool instance (with driver): " + this) ;		
	}
	
	/**
	* Set the tns alias for the target database, as defined in the <code>tnsnames.ora</code> oracle file. 
	* This method can be used only with the Oracle database and a JDBC driver 10.2 or higher. 
	* The tnsnames files should be configured in a path like the following:<br/><br/>
	* 
	* <code>C:\\Oracle\\product\\10.1.0\\Client\\NETWORK\\ADMIN</code><br/><br/>
	*
	* This approach allow to get a JDBC connection using only the logical name of the datasource, which must
	* match one of the entries defined in the <code>tnsnames.ora</code> oracle file. For this reason, the
	* <code>Database</code> class takes care to set the proper <code>oracle.net.tns_admin</code> variable.
	*
	* @param the tns alias of the datasource, for example @mydatabase
	* @return <code>true</code> if this feature is supported
	*/
	public boolean setTnsName(String tnsAlias) {
	
		// Storing class field reference
		this.alias =  "jdbc:oracle:thin:@" + tnsAlias ;
		
		try {	
			Connection connection = DriverManager.getConnection(alias, usr, pwd) ;	
			if (connection != null) connection.close() ;
			logger.logInfo(leadingName + "Loaded TNS oracle configuration for " + this.alias) ;		
			this.supportTns = true ;
		} catch(Exception e) {
			logger.logWarning(leadingName + "TNS configuration not available: " + e.getMessage()) ;
			this.supportTns = false ;
		}
		
		return this.supportTns ;
	}

// **********************************************************************************************************	
// ************************				CONNECTION MANAGEMENT		*****************************************
// **********************************************************************************************************

	/**
	* Users should call this method each time a connection is needed. The method relies on the mechanism
	* written in the parent <code>ObjectPool</code> class for checking out a valid object from the pool.
	* If the super method throws an exception, this is catched, casted into a <code>SQLException</code>
	* type and rethrow.<br/>
	* This approach should preserve all the information stored in the original general exception, hence,
	* if the upper layers gets an "empty exception" (i.e. without a stack trace) probabily there is 
	* something wrong in the object itself (in this case the Oracle connection).
	*
	* @return a connection obtained from the <code>super.checkOut()</code> method
	*/
	public Connection getConnection() throws SQLException {

		try {
			return (Connection) super.checkOut() ;
		} catch(Exception e) {
			throw ((SQLException) e) ;
		}
	}

	/**
	* Users should call this method each time a connection is not needed anymore.
	* This method forwards action to the parent <code>super.checkIn()</code> method.
	*
	* @param connection the connection to be returned in the <em>unlocked</em> hashtable
	*/
	public void returnConnection(Connection connection)	{

		super.checkIn(connection) ;
	}

	/**
	* This method just terminates the cleaner thread.
	*/
	public void shutdown() {

		super.shutdown() ;
	}

// ************************************************************************************************************
// ************************		METHODS REQUIRED BY ABSTRACT CLASS		***************************************
// **********************************************************************************************************

	/**
	* Users should not call this method directly. It's the <code>checkOut()</code> method of the parent
	* class that will invoke this method ONLY IF there isn't any unlocked object in the pool, in order
	* to create a new one. Notice that this is the only codeblock creating the objects managed by this
	* pool class (in this case a <code>Connection</code> object).
	*
	* @return an object representing a brand new connection to the database
	*/
	public Object create() throws SQLException	{

		Connection connection = null ;
		if (supportTns) connection = DriverManager.getConnection(alias, usr, pwd) ;
			else connection = DriverManager.getConnection(tns, usr, pwd) ;
		connection.setAutoCommit(false) ;
		return connection ;
	}

	/**
	* Method used by the <code>checkOut()</code> method of the parent class before checking out an
	* object from the unlocked pool.
	*
	* @param object the object to be validated
	*/
	public boolean validate(Object object) {

		try {
			return( ! ((Connection) object).isClosed() ) ;
		} catch(SQLException e) {
			logger.logWarning(leadingName + "Failed validation for " + object) ;
			e.printStackTrace(System.err) ;
			return false ;
		}
	}

	/**
	* Method invoked by the parent class in two different situations: when the object is found invalid
	* (during a check out operation) or when the Cleaner Thread discards the object (timeout).
	*
	* @param object the object to be destroyed
	*/
	public void expire(Object object) {

		try {
			((Connection) object).close() ;
		} catch (SQLException e) {
			logger.logSevere(leadingName + "Failed expiration for " + object) ;
			e.printStackTrace(System.err) ;
		}
	}
	
// **********************************************************************************************************	
// ************************					 MUTATORS			*********************************************
// **********************************************************************************************************
	
	/**
	* Mutator method.
	*/
	public void setTNSEntryName(String value) 				{		this.alias = value ;					}	
	
// **********************************************************************************************************	
// ************************					 ACCESSORS			*********************************************
// **********************************************************************************************************

	/**
	* Accessor method.
	*/
	public String getTNSEntryName() 						{		return this.alias  ;					}	
	
	/**
	* Accessor method.
	*/
	public boolean supportTnsNames() 						{		return this.supportTns  ;				}	

// **********************************************************************************************************
// ********************************			UTILITIES			*********************************************
// **********************************************************************************************************

	/**
	* Split the full <code>tns</code> string and return only the last token, describing the database name.
	* This method is used only to obtain a user-friendly name for logging messages.
	*
	* @param tns the tns string of the db, e.g. jdbc:oracle:thin:@192.168.3.12:1588:vardati
	* @return the last part of the database name, e.g. vartdati
	*/
	protected String getDatabaseName(String tns) {
		
		String[] result = tns.split(":") ;
		int last = result.length - 1 ;
		String name = "jdbcpool " + result[last] + " - " ;
		return name ;
	}
	
	/**
	*
	* WORK IN PROGRESS: NOT USED AT THE MOMENT
	*
	*/
	public boolean logConnection(Connection connection) {
	
		if (connection == null) return false ;
		StringBuffer msg = new StringBuffer() ;
	
		try {
			// Logging basic information
			msg.append("Current status for connection " + connection + ":\n\n") ;		
			msg.append("Closed: " + connection.isClosed() + "\n") ;		
			//msg.append("Valid : " + connection.isValid(1000) + "\n") ;		// Timeout = 1000 ms
		
			// Eventually loggig client properties
			int n = 1 ;
			java.util.Properties properties = connection.getClientInfo() ;
			java.util.Enumeration e = properties.propertyNames() ;
			while ( e.hasMoreElements() ) {
				String name = (String) e.nextElement() ;
				msg.append(n++ + ") " + name + ": " + properties.getProperty(name) + "\n") ;
			}
		} catch (Exception e) {
			logger.logWarning(leadingName + "Error analyzing connections: " + e) ;
			e.printStackTrace(System.err) ;
		}
		
		// Printing results
		logger.logInfo(leadingName + msg) ;
		return true ;
	}
	
} // END OF CLASS