/**
* Title: MockConnection
* Description:
* Copyright:    Copyright (c) 2010
* Company:      Morpheus Quality Technologies
* @author Stefano Adriani
* @version 1.0
*/

package adriani.jws.template.db ;

import adriani.jws.template.util.* ;
import adriani.jws.template.Settings ;

import java.sql.* ;

/**
* Wrapper class for the <code>java.sql.Connection</code> object. These wrappers can be pooled using the
* <code>MockConnectionPool</code> class, in substitution of the couple "pure connections" pooled by the
* simple <code>JDBCConnectionPool</code> class.<br/>
* Usually it should be not necessary to use these wrappers, since the JDBC connection pool guarantees the
* same behaviuor and performances. Selecting this "pool wrapper architecture" could be the best choice only
* in few situations, such as:<br/><br/>
*
* <ul>
* <li>The JDBC paradigm implies that prepared statements are precompiled and stored directly in the
* database. Some databases do not offer this feature, hence prepared statement are stored in the JDBC 
* driver. If even this case does not occur a solution could be caching the prepared statement in the
* inner hash table of <code>MockConnection</code> class.</li>
* <li>JDBC does not allow to retrieve the original SQL query used to create a given prepared statement.
* If this becomes necessary, the original SQL query can be stored using the <code>MockConnection</code> 
* wrapper class, and retrieved even after execution.</li>
* </ul>
*/
public class MockConnection {

  	/** The JDBC SQL connection object wrapped by this class.												*/
	protected 				Connection 				connection 		= null ;
	
	/** The hash table tracking all the <code>PreparedStatement</code> bound to this connection.			*/
	private static 			java.util.Hashtable		table 			= null ;
	
	/** Static final <code>String</code> value stored as class field to improve performances.				*/
	public static final 	String					THIN_DRIVER		= "jdbc:oracle:thin:@" ;	
	
	/** Beginning of each logging message produced by this class.											*/
	protected static 		String 					leadingName		= "MockConnection - " ;
	
	/** Retrieving the logger instance to be used for this class (static).									*/
	protected static 		WSLogger				logger 			= WSLogger.getLogger(MockConnection.class) ;
	
	/**
	* Standard constructor.
	*
	* @param connection the <code>java.sql.Connection</code> to be wrapped by this class
	*/
	public MockConnection(Connection connection) {
	
		this.connection = connection ;
	}	
	
	/**
	* Use this constructor if another class has already registered the JDBC driver for the same type of 
	* database hosting these connections. This constructor retrieve a brand new connection from the
	* <code>DriverManager</code> and stores it as a class field.
	*
	* @param alias the tns string of the db, e.g. jdbc:oracle:thin:@192.168.1.10:1533:tartdati
	* @param usr the user name for the connection
	* @param pwd the password for the connection
	*/
	public MockConnection(String alias, String usr, String pwd) {
	
		try {	
			this.connection = DriverManager.getConnection(THIN_DRIVER + alias, usr, pwd) ;	
		} catch(Exception e) {
			this.connection = null ;
			logger.logSevere(leadingName + "Error opening connection to " + THIN_DRIVER + alias) ;
			e.printStackTrace() ;
		}
	}
	
// **********************************************************************************************************	
// ************************				PREPARED STATEMENT		*********************************************
// **********************************************************************************************************

	/**
	* Use always this method to create a JDBC <code>PreparedStatement</code> from a <code>xxxDBA</code>
	* class, in order to uncouple database pooling structure from DBA query logic.<br/>
	* This method takes care to inspect the <code>Database</code> class and then decide if the prepared
	* statement must be produced brand new, or retrieved from the inner cache.
	*
	* @param query the original SQL query, filled with question marks ('?') characters
	* @see #returnPreparedStatement(PreparedStatement pstmt)
	*/
	public PreparedStatement getPreparedStatement(String query) throws SQLException {
		
		PreparedStatement pstmt = null ;
		
		// Trying to retrieve an already used prepared statement from the inner cache
		if ( Database.usesMockedPool() ) {
			if (table == null) table = new java.util.Hashtable() ;
			pstmt = (PreparedStatement) table.get(query) ;
			if (pstmt == null) {
				pstmt = connection.prepareStatement(query) ;			
				table.put(query, pstmt) ;
				if (Settings.DETAILS) System.out.println(leadingName + "Cached new pstmt: " + pstmt) ;
			}
			if (Settings.DETAILS) System.out.println(leadingName + "Using cached pstmt: " + pstmt) ;
		// Otherwise we create a brand new prepared statement (not stored)
		} else {
			pstmt = connection.prepareStatement(query) ;			
			if (Settings.DETAILS) System.out.println(leadingName + "Using temporary pstmt: " + pstmt) ;
		}
		
		return pstmt ;
	}	
	
	/**
	* Use always this method to close a JDBC <code>PreparedStatement</code> from a <code>xxxDBA</code>
	* class, in order to uncouple database pooling structure from DBA query logic.<br/>
	* This method takes care to inspect the <code>Database</code> class and then decide if the prepared
	* stament must be closed or cached in this class's inner hashtable.
	*
	* @param pstmt the <code>PreparedStatement</code> obtained using <code>getPreparedStatement()</code>
	* @return <code>true</code> if the prepared statement has been closed (i.e. released)
	* @see #getPreparedStatement(String query)
	*/
	public boolean returnPreparedStatement(PreparedStatement pstmt) {
		
		// If we cache prepared statement, we leave it open
		if ( Database.usesMockedPool() ) return false ;
		// Otherwise we try to close it
		else {
			try	{			
				pstmt.close() ;
				return true ;				
			}catch(Exception e) {
				logger.logSevere(leadingName + "Impossible to close prepared statement: " + pstmt) ;
				return false ;
			}
		}
	}	
	
// **********************************************************************************************************	
// ************************					 MUTATORS			*********************************************
// **********************************************************************************************************
	
	/**
	* Mutator method.
	*/
	public void setConnection(Connection connection)		{		this.connection = connection ;			}	
	
// **********************************************************************************************************	
// ************************					 ACCESSORS			*********************************************
// **********************************************************************************************************

	/**
	* Accessor method.
	*/
	public Connection getConnection() 						{		return this.connection  ;				}	
	
} // END OF CLASS