/**
* Title: MockConnectionPool
* Description:
* Copyright:    Copyright (c) 2010
* Company:      Morpheus Quality Technologies
* @author Stefano Adriani
* @version 1.0
*/

package adriani.jws.template.db ;

import adriani.jws.template.* ;

import java.sql.* ;

/**
* JDBC oriented implementation of the <code>ObjectPool</code> parent class.<br/>
* Before ending or closing the application using the pool, the user should call the
* <code>shutdown()</code> method to properly terminate the cleaner thread.
*
* @see ObjectPool
*/
public class MockConnectionPool extends JDBCConnectionPool {

	/**
	* Use this constructor if the JDBC driver has been already registered.<br/>
	* Current implementation just calls the super constructor, which forward action to the super-super class 
	* (i.e. <code>ObjectPool</code>) that takes care to create and run the cleaner thread.
	*
	* @param tns the tns string of the db, e.g. jdbc:oracle:thin:@192.168.1.10:1533:tartdati
	* @param usr the user name for the SQL connection
	* @param pwd the password for the SQL connection
	* @param expirationTime sleep time of the Cleaner Thread in seconds
	*/
	public MockConnectionPool(String tns, String usr, String pwd, int expirationTime) {

		super(tns, usr, pwd, expirationTime) ;
		//System.out.println(leadingName + "Created mocked ObjectPool instance: " + this) ;		
	}

	/**
	* Use this constructor when it's necessary to register the JDBC driver (only the first time).<br/>
	* Current implementation just calls the super constructor, which is responsible for
	* creating and registering the proper <code>Driver</code> class.
	*
	* @param driver the JDBC driver, e.g. oracle.jdbc.driver.OracleDriver.
	* @param tns the tns string of the db, e.g. jdbc:oracle:thin:@192.168.1.10:1533:tartdati
	* @param usr the user name for the SQL connection
	* @param pwd the password for the SQL connection
	* @param expirationTime sleep time of the Cleaner Thread in seconds
	*/
	public MockConnectionPool(String driver, String tns, String usr, String pwd, int expirationTime) {

		super(driver, tns, usr, pwd, expirationTime) ;
		//System.out.println(leadingName + "Created mocked ObjectPool instance (with driver): " + this) ;		
	}

// **********************************************************************************************************	
// ************************				CONNECTION MANAGEMENT		*****************************************
// **********************************************************************************************************

	/**
	* Users should call this method each time a mock connection is needed.<br/><br/>
	*
	* <strong>Remark</strong>: DO NOT USE the <code>getConnection()</code> method inherited from the
	* super class, since any <code>java.sql.Connection</code> object stored in a <code>MockConnectionPool</code>
	* class is always null.
	*
	* @return a mock obtained from the <code>super.checkOut()</code> method
	*/
	public MockConnection getMockConnection() throws SQLException {

		try {
			return (MockConnection) super.checkOut() ;
		} catch(Exception e) {
			throw ((SQLException) e) ;
		}
	}

	/**
	* Users should call this method each time a mock is not needed anymore.<br/><br/>
	*
	* <strong>Remark</strong>: DO NOT USE the <code>returnConnection()</code> method inherited from the
	* super class, since any <code>java.sql.Connection</code> object stored in a <code>MockConnectionPool</code>
	* class is always null.
	*
	* @param mock the MockConnection to be returned in the <em>unlocked</em> hashtable 
	* (using <code>super.checkIn()</code>)
	*/
	public void returnMockConnection(MockConnection mock)	{

		super.checkIn(mock) ;
	}

// ************************************************************************************************************
// ************************		METHODS REQUIRED BY ABSTRACT CLASS (OVERIDDEN)		***************************
// ************************************************************************************************************

	/**
	* Users should not call this method directly. It's the <code>checkOut()</code> method of the 
	* <code>ObjectPool</code> class that will invoke this method ONLY IF there isn't any unlocked object in the
	* pool, in order to create a new one. Notice that this is the only codeblock creating the objects managed 
	* by this pool class (in this case a <code>MockConnection</code> object).
	*
	* @return an object representing a brand new <code>MockConnection</code> to the database
	*/
	public Object create() throws SQLException	{
		
		Connection connection = (Connection) super.create() ;
		return new MockConnection(connection) ;
	}

	/**
	* Method used by the <code>checkOut()</code> method of the <code>ObjectPool</code> class before checking 
	* out an object from the unlocked pool.
	*
	* @param object the object (in this case a <code>MockConnection</code> object) to be validated
	*/
	public boolean validate(Object object) {
		
		Connection connection = getConnectionFromMock(object) ;	
		return super.validate(connection) ;	
	}

	/**
	* Method used by the <code>ObjectPool</code> class in two different situations: when the object is found
	* invalid (during a check out operation) or when the Cleaner Thread discards the object (timeout).
	*
	* @param object the object (in this case a <code>MockConnection</code> object) to be destroyed
	*/
	public void expire(Object object) {
	
		Connection connection = getConnectionFromMock(object) ;
		super.expire(connection) ;	
	}
	
// ************************************************************************************************************
// **********************************		OVERRIDED UTILITIES			***************************************
// ************************************************************************************************************
	
	/**
	* Cast the input object into a <code>MockConnection</code> wrapper and the retrieves the 
	* <code>java.sql.Connection</code> stored in the wrapper.
	*
	* @param object an object managed by this pool class (i.e. a <code>MockConnection</code> instance)
	* @return the <code>java.sql.Connection</code> stored in the <code>MockConnection</code> wrapper
	*/
	public Connection getConnectionFromMock(Object object) {
	
		MockConnection mock = (MockConnection) object ;
		return mock.getConnection() ;
	}	
	
// **********************************************************************************************************
// ********************************			GENERAL UTILITIES			*************************************
// **********************************************************************************************************

	/**
	* Split the full <code>tns</code> string and return only the last token, describing the database name.
	* This method is used only to obtain a user-friendly name for logging messages.
	*
	* @param tns the tns string of the db, e.g. jdbc:oracle:thin:@192.168.3.12:1588:tardati
	* @return the last part of the database name, e.g. tartdati
	*/
	protected String getDatabaseName(String tns) {
		
		String[] result = tns.split(":") ;
		int last = result.length - 1 ;
		String name =  "mockpool " + result[last] + " - " ;
		return name ;
	}
	
} // END OF CLASS