/**
* Title: ServiceDBA
* Description:
* Copyright:    Copyright (c) 2008-2010
* Company:      Morpheus Quality Technologies
* @author Stefano Adriani
* @version 1.0
*/

package adriani.jws.template.db ;

import adriani.jws.template.* ;
import adriani.jws.template.service.* ;
import adriani.jws.template.util.* ;

import java.sql.* ;
import java.util.* ;

/**
* The class wrapping all the SQL queries and statement necessary to retrieve data from the databases.
* This class stores the necessary references to the database connections. These fields are not present in the
* <code>GeneralDBA</code> parent class because usually it's better to avoid storing DB connection as class fields.
* Here we store the connections to improve performances by sharing the same connection when invoking
* a sequence of queries from the <code>ServiceMacro</code> class.
* <br/><br/> 
* The connections are obtained from the <code>Database</code> class using the <code>MockConnection</code> 
* wrapper, which allows to perform some specific test/debug operations can improve perfomances when
* using prepared statements in a connection pooling enviroment.<br/>
* Note that it's up to the <code>Database</code> class to manage the eventual connections pooling, in order
* to keep completely uncoupled implementations of this class from the connections management technique:
* whatever of the pool mechanism, a DBA class should always retrieve connections just calling:<br/><br/>
* 
* <code>Database.getMockConnection(targetDB)</code><br/><br/>
*
* and returning them by calling:<br/><br/>
*
* <code>Database.returnMockConnection(targetDB, this.mock[targetDB])</code>
*/
public class ServiceDBA extends GeneralDBA {

  	/** The array of mock connections for the necessary databases (i.e. accessed by this DBA).				*/
	protected 				MockConnection[]	mock ;

  	/** A reference to the <code>ServiceMacro</code> object which created this DBA.							*/
	protected 				ServiceMacro 	serviceMacro  ;
	
	/** Retrieving the logger instance to be used for this class (static).									*/
	protected static 		WSLogger		logger 				= WSLogger.getLogger(ServiceDBA.class) ;
	
	/**
	* This constructor DOES NOT require (nor create) the <code>serviceMacro</code> object, hence the 
	* configuration files and properties will NOT be loaded. Default values (hard coded) will be used 
	* from the <code>Settings</code> class.
    */
    public ServiceDBA() {

		this.leadingName 	= "Service DBA - " ;
		this.mock 			= new MockConnection[Database.DB_NUMBER] ;
	}

	/**
	* Constructor.
	*
	* @param macro the <code>ServiceMacro</code> class creating and using this DBA.
    */
    public ServiceDBA(ServiceMacro macro) {

		this() ;
		this.serviceMacro = macro ;
	}

// **********************************************************************************************************
// *****************************		CONNECTION MANAGEMENT		*****************************************
// **********************************************************************************************************

	/**
	* Open the connections to two databases: the DB_01 database and the DB_02 database. Both connections
	* are retrieved from the <code>Database</code> class using the <code>JDBCConnectionPool</code> mechanism.
	* The connection to be opened is selected by the input <code>targetDB</code> parameter.<br/>
	* For example:
	* <br/><br/>
	* targetDB = 1 --> open a connection to the DB_01 database.<br/>
	* targetDB = 2 --> open a connection to the DB_02 database.<br/>
	* etc...
	* <br/><br/>
	*
	* Current implementation immediately QUIT execution if we get an error opening the connection, 
	* by logging a SEVERE error message and returning <code>false</code>. Usually this means that 
	* the web service will not produce a valid response: the client could receive a SOAP response
	* containing a message like "database not found".<br/>
	*
	* @param targetDB the reference code of the database where to open the connection
	* @return <code>true</code> if the connection has been opened succesfully
	* @see Database#getJDBCPool(int targetDB)
    */
    public boolean openConnection(int targetDB) {
		
		// 1. Checking if general Database enviroment is properly set
		if (mock == null) {
			logger.logSevere(leadingName + "MockConnection array not initialized") ;
			return false ;
		} else if (targetDB >= mock.length) {
			logger.logSevere(leadingName + "Database reference number " + targetDB + " can not be handled") ;
			return false ;
		}
		
		// 2. Trying to retrieve a valid connection wrapper
		try {
			this.mock[targetDB] = Database.getMockConnection(targetDB) ;
			return (mock[targetDB] != null) ;
		}
		catch(SQLException e) {
			logger.logSevere(leadingName + "Connection to database " + targetDB + " not found") ;
	  		e.printStackTrace() ;
			return false ;
		}
		catch(Exception e) {
			logger.logSevere(leadingName + "Java error opening connections to database " + targetDB) ;
	  		e.printStackTrace() ;
			return false ;
		}
	}

	/**
	* Close the connection specified by the input target parameter, for example:
	* <br/><br/>
	* targetDB = 1 --> close the connection to the DB_01 database.<br/>
	* targetDB = 2 --> close the connection to the DB_02 database.<br/>
	* etc...
	* <br/><br/>
	*
	* Current implementation DOES NOT quit execution if we get an error returning the connection. 
	* This means that the web service could provide a valid output, but connections to the database 
	* are left "hunged up" and can cause a database crash, hence such a situation is logged as a SEVERE
	* error by the log system.<br/>
	*
	* @param targetDB the number of the database where to return the connection
    */
    public void closeConnection(int targetDB) {
	
		if ( (mock != null) && (targetDB < mock.length) ) {
			Database.returnMockConnection(targetDB, this.mock[targetDB]) ;
			this.mock[targetDB] = null ;
		} else {
			StringBuffer message = new StringBuffer(leadingName + "Wrong database enviroment: ") ;
			message.append("Impossible to return connection for database " + targetDB) ;
			logger.logSevere(message.toString()) ;
		}
	}
	
	/**
	* Return the connection currently opened through the target database, for example:
	* <br/><br/>
	* targetDB = 1 --> get the connection opened to the DB_01 database.<br/>
	* targetDB = 2 --> get the connection opened to the DB_02 database.<br/>
	* etc...
	*
	* @param targetDB the number of the database offering the connection
	* @return the currently opened connection to the target database
    */
    public Connection getConnection(int targetDB) {
	
		Connection connection = null ;
		
		if ( (mock != null) && (targetDB < mock.length) ) connection = mock[targetDB].getConnection() ;
		else {
			StringBuffer message = new StringBuffer(leadingName + "Wrong database enviroment: ") ;
			message.append("Impossible to use connection for database " + targetDB) ;
			logger.logSevere(message.toString()) ;
		}
		
		return connection ;
	}
	
// **********************************************************************************************************
// *****************************			TEST & DEBUG			*****************************************
// **********************************************************************************************************

	/**
	* Produces a <code>StringBuffer[][]</code> object created from some hard-coded data. 
	* This method should be used only during development to obtain a valid object from this DBA
	* as long as the actual database is not available. The advantage to use this method during development
	* is that we emulate the full DBA behaviour, including error handling and the analyzer mechanism
	* (execution of this method will be counted as execution of one SQL query).
	*
	* @param param the array containing all the input data
	* @param howMany the number of record to be retrieved
	* @param cols the number of columns for each record
	* @return a <code>StringBuffer[][]</code> object obtained from hard-coded data
    */
    public StringBuffer[][] fakeQuery(StringBuffer[] param, int howMany, int cols) {

		// Emualating the "no data" situation
		if ((howMany == 0) || (cols == 0)) return null ;
	
		// Preparing variables
		String label = "fakeQuery" ;
		if (Settings.BENCH) startQuery(label) ;					
		StringBuffer[][] output = new StringBuffer[howMany][cols] ;

		// Eventually printing input paramters on the STDOUT
		if (Settings.DETAILS) {
			System.out.print(leadingName + "Running fake query using input: ") ;
			for (int i = 0 ; i < param.length ; i++) System.out.print(param[i] + " ") ;
			System.out.println() ;
		}
		
		// Producing the whole result set
		StringBuffer[] record = null ;
		ArrayList list = new ArrayList() ;							
		for (int n = 0 ; n < howMany ; n++) {
			record = new StringBuffer[cols] ;	
			// Producing a single record
			for (int i = 0 ; i < cols ; i++) {
				int index = (i < param.length) ? i : (param.length - 1) ;
				if (n == 0)	{
					record[0] = new StringBuffer("Mickey") ;
					record[1] = new StringBuffer("Mouse") ;
				} else {
					//record[i] = new StringBuffer("foo") ;				
					record[i] = new StringBuffer("Echo " + param[index]) ;
				}
			}
			list.add(record) ;						
		}
		
		// Creating output array
		int size = list.size() ;				
		output = (StringBuffer[][]) list.toArray( new StringBuffer[size][cols] ) ;									
		
		// Eventually logging results
		if (Settings.BENCH) endQuery(label) ;						
		if (Settings.ANALYZE) serviceMacro.getServiceAnalyzer().collect(ErrorMessage.OK) ;		
		if (logger.isInfoEnabled()) logger.logInfo(leadingName + "Fake query produced: " + size + " records") ; 		
		
		return output ;
	}
	
} // END OF CLASS