/**
* Title: WrapperIn
* Description:
* Copyright:    Copyright (c) 2009-2010
* Company:      Sirti S.P.A.
* @author Stefano Adriani
* @version 1.0
*/

package adriani.jws.template.messages ;

import adriani.jws.template.eclipse.* ;

import java.math.* ;

/**
* WrapperIn - general class wrapping the root <code>ADBBean</code> element containing the input SOAP request.
*/
public class WrapperIn {

  	/** Static <strong>int</strong> value depecting a undefined number field.<br/>							*/
	public static final 	BigInteger		NOT_AVAILABLE		= BigInteger.valueOf(-1) ;

  	/**
	* Static <strong>string</strong> value depecting a undefined number field.<br/>
	* Example: if validation fails for some input numeric field, and if the corresponding class attribute
	* (of this wrapper) is a string type, the <code>NO_NUMBER</code> string is stored into the class attribute
	* in order to acknowledge this situation.<br/>
	* Current implementation stores the text <code>UNSET</code> in the NO_NUMBER field.
	*/
	public static final 	StringBuffer 	NO_NUMBER 			= new StringBuffer("UNSET") ;

   	/** Wrapped input value, used to store the SOAP request value after validation.							*/
	protected 				StringBuffer	number ;
	/** Wrapped input value, used to store the SOAP request value after validation.							*/
	protected 				BigInteger		customer ;

  	/** Beginning of each logging message produced by this class.											*/
	protected static 		String 			leadingName 		= "WrapperIn - " ;

	/**
	* Constructor used by the <code>SimpleClient</code> to test the service (empty).
	*/
	public WrapperIn() {

	}

	/**
	* Constructor used by the actual web service logic, used to set the wrapper's class fields reading
	* the values from the <code>ADBBean</code> element of the input SOAP request. For this reason
	* implementation of this constructor is strongly coupled to the WSDL definition.
	*
	* @param element the <code>ADBBean</code> object as received by the service skeleton class
	*/
	public WrapperIn(User user) {

		// Retrieving ADBBean root element
		UserType userType 	= user.getUser() ;

		// Retrieving ADBBean inner types
		PhoneType phoneType = userType.getPhone() ;

		// Adjusting input value (optional)
		BigInteger 		customer 					= userType.getCustomer() ;
		String 			phone						= phoneType.getPhoneType().trim() ;

		// Setting class fields
		this.customer								= customer ;
		this.number									= new StringBuffer( phone ) ;
		//System.out.println(leadingName + "Set customer ID: " + this.customer) ;
	}

// **********************************************************************************************************
// **********************************		ELEMENT MANAGEMENT		*****************************************
// **********************************************************************************************************

	/**
	* This method should be called after that all the wrapper's fields have been properly set, in order to
	* adjust any string requiring a particular format. For example, if the datasource required that all the
	* accent vowels must be replaced with the corresponding single quote notation, this method should take
	* care to implement the necessary operations (converting <em></em> into <em>a'e'e'i'o'u'</em>).
	* <br/><br/>
	* Current implementation does nothing.
	*
	* <strong>Remark:</strong> to grant LOOSE COUPLING to the WSDL, the <code>ADBBean</code> Java classes
	* should NOT be used within this method.
	*/
	public void validate() {

		// Example code
		//if (customer < 0) customer = - customer ;
	}

// **********************************************************************************************************
// **********************************			ACCESSORS		*********************************************
// **********************************************************************************************************

	/**
	* Accessor method.
	*/
	public BigInteger getCustomer()						{		return this.customer ;						}

	/**
	* Accessor method.
	*/
	public StringBuffer getNumber()						{		return this.number ;						}

 // **********************************************************************************************************
// **********************************			MUTATORS		*********************************************
// **********************************************************************************************************

	/**
	* Mutator method.
	*/
	public void setCustomer(BigInteger value)			{		this.customer = value ;						}

	/**
	* Mutator method (overloaded).
	*/
	public void setCustomer(StringBuffer value) {

		try 				{ this.customer = new BigInteger( value.toString() ) ;							}
		catch(Exception e) 	{ this.customer = NOT_AVAILABLE ;												}
		//try 				{ this.customer = Integer.parseInt( value.toString() ) ;						}
		//catch(Exception e) 	{ this.customer = NOT_AVAILABLE ;												}
		//System.out.println(leadingName + "Set customer ID: " + this.customer) ;
	}

	/**
	* Mutator method.
	*/
	public void setNumber(StringBuffer value)			{		this.number = value ;						}

// **********************************************************************************************************
// **********************************			BOOLEANS			*****************************************
// **********************************************************************************************************

	/**
	* Helper method, used just to improve performance by reducing multiple checks (only for frequent calls).
	*/
	public boolean isGoodNumber() 				{	return ! compareSB(number, NO_NUMBER) ;					}

// **********************************************************************************************************
// **********************************			UTILITIES			*****************************************
// **********************************************************************************************************

	/**
	* Serialize all the data contained by this wrapper as a <code>String</code> object, dot separated.
	*
	* @return a string of the type "scott.tiger.42.prince consort road.london"
	*/
	public String toString() {

		String sep = "." ;
		return number + sep + customer ;
	}

	/**
	* Produce a <code>StringBuffer[]</code> array containing all data contained by this wrapper.
	*
	* @return a string buffer array with the data contained by this wrapper
	*/
	public StringBuffer[] toArray() {

		StringBuffer customer = new StringBuffer( "" + this.customer ) ;

		return new StringBuffer[] {number, customer} ;
	}

	/**
	* Helper method, used from all the boolean <code>isXXX()</code> methods above.
	*
	* @param first a <code>StringBuffer</code> object
	* @param second another <code>StringBuffer</code> object
	* @return <code>true</code> if the two string object contains the same text
	*/
	public boolean compareSB(StringBuffer first, StringBuffer second) {

		String string = null ;
		boolean result = false ;
		if (first != null) string = first.toString() ;
		if (string != null) result = string.equalsIgnoreCase( second.toString() ) ;
		return result ;
	}

	/**
	* Special accessor used to retrieve the field used to send commands to the web service. This special field
	* must be a text type (String or StringBuffer) to allow inserting a "leading password" in the field, for
	* example "000". Whatever is the type of the special field, this method returns always a
	* <code>StringBuffer</code> to ensure encapsulation.
	*
	* @see manageService(WrapperIn input)#ServiceLogic
	* @return a <code>StringBuffer</code> object containing the original field value
	*/
	public StringBuffer getCommand() 					{		return new StringBuffer(this.number) ;		}

} // END OF CLASS