/**
* Title: WrapperOut
* Description:
* Copyright:    Copyright (c) 2010
* Company:      Sirti S.P.A.
* @author Stefano Adriani
* @version 1.0
*/

package adriani.jws.template.messages ;

import adriani.jws.template.service.* ;

/**
* WrapperOut - general class wrapping the root <code>ADBBean</code> element containing the status flag,
* the status message and the array of records representing the results.<br/>
* This class can wrap an actual array of <code>Wrapper</code> objects as well as a single
* <code>Wrapper</code> object. In this way if the WSDL specifications changes the whole architecture
* should not change: to change from an array response to a single record response just refactory
* the <code>validate()</code> method.
*/
public abstract class WrapperOut {

  	/** Static reference value for the STATUS field.														*/
	public static  			int 				STATUS_OK			=  1 ;
	/** Static reference value for the STATUS field.														*/
	public static  			int					STATUS_WRONG		=  0 ;
	/** Static reference value for the STATUS field.														*/
	public static 	 		int 				STATUS_UNSET		= -1 ;

	/** Static reference values for the MESSAGE field.														*/
	public static 			StringBuffer 		MESSAGE_OK			= new StringBuffer( ErrorMessage.getMessage(ErrorMessage.DATA_OK) ) ;
	/** Static reference values for the MESSAGE field.														*/
	public static 			StringBuffer 		MESSAGE_WRONG		= new StringBuffer( ErrorMessage.getMessage(ErrorMessage.GENERAL) ) ;
	/** Static reference values for the MESSAGE field.														*/
	public static 			StringBuffer 		MESSAGE_UNSET		= null ;

 	/** Static default value for the STATUS field (should match one of the static reference values).		*/
	public static 			int			 		STATUS	 			= STATUS_UNSET ;
	/** Static default value for the MESSAGE field (should match one of the static reference values).		*/
	public static 			StringBuffer 		MESSAGE 			= MESSAGE_UNSET ;

 	/** The wrapped <code>ADBBean</code> element.															*/
	protected 				Object 				element ;

  	/**
	* The array of the "inner wrappers" stored as CLASS FIELD before validation.<br/>
	* If the SOAP response contains just one single output record, the <code>validate()</code> should
	* take into consideration only the <code>records[0]</code> element.
	*/
	protected 				Wrapper[] 			records ;

	/** The integer flag describing the status of the response.												*/
	protected 				int 				status ;

  	/** The text message describing the status of the response.												*/
	protected 				StringBuffer 		message ;

  	/** Beginning of each logging message produced by this class.											*/
	protected static 		String 				leadingName 	= "WrapperOut - " ;

// **********************************************************************************************************
// **********************************		ELEMENT MANAGEMENT		*****************************************
// **********************************************************************************************************

	/**
	* Validation is performed by the Eclipse's <code>ADBBean</code> objects (e.g. TownType.setTownType() etc.)
	* If the validation process is succesfull, the resulting <code>ADBBean</code> object is stored as
	* class field to save computation time, since after validation we expect that the method
	* <code>getElement()</code> should be invoked.<br/><br/>
	*
	* <strong>Remark:</strong> to grant LOOSE COUPLING to the WSDL, the <code>ADBBean</code> Java classes
	* should be used only within this method and in the <code>forceMessage()</code> method, not elsewhere.
	*
	* @return a reference to the <code>ADBBean</code> object wrapped by this class
	*/
	public abstract Object validate() throws WrapperOutFault ;

	/**
	* Load all the STATIC default values (defined in this class) into the corresponding class attributes.
	* This method should also populate the <code>records</code> array of <code>Wrapper</code> objects
	* as expected by the default empty response.<br/>
	* Example: if the empty SOAP response contains just one single <code>Wrapper</code> object, this
	* method should create an array with just one record.<br/>
	*
	* <strong>Hint</strong>: subclasses should call this method inside the constructor
	*/
	public abstract void setDefaults() ;

	/**
	* Set an integer value in the wrapped ADBBean element, without checking for validation and ignoring the
	* value of the <code>status</code> class attribute. This method is used when validation fails and it's
	* necessary to set the response status in the status field of the wrapped ADBBean element.
	*
	* @param status the response status value
	*/
	public abstract void forceStatus(int value) ;

	/**
	* Set a string message in the inner <code>ADBBean</code> element, without checking for validation and
	* overwriting the previous value of the <code>message</code> class attribute (if any).
	* This method is used when validation fails and it's necessary to communicate the error in a message field
	* of the wrapped <code>ADBBean</code> element (usually the <em>message</em> field).<br/><br/>
	*
	* <strong>Remark:</strong> this method MUST be called only after validation, because we want to
	* put all the building logic of the <code>ADBBean</code> structure in the <code>validate()</code> method.
	*
	* @param message the error message that must be set in the proper message field of the response
	*/
	public abstract void forceMessage(String message) ;

	/**
	* Return the <code>ADBBean</code> element wrapped by this class.
	*
	* @return a reference to the <code>ADBBean</code> object wrapped by this class
	*/
	public Object getElement() 							{		return this.element ;						}

// **********************************************************************************************************
// **********************************			ACCESSORS			*****************************************
// **********************************************************************************************************

	/**
	* Return the value of the status flag wrapped by this class.<br/>
	* Default implementation uses <code>1</code> for a good valid SOAP response, <code>0</code> elsewhere.
	* Subclasses can change this definitions by overriding the static <code>STATUS_XXX</code> fields.
	*
	* @return the integer status flag (the wrapper's class field)
	*/
	public int getStatus()								{		return this.status  ;						}

	/**
	* Return the value of the text message wrapped by this class.<br/>
	* The message details the quality of the SOAP response described by the status flag.
	* Subclasses can change this definitions by overriding the static <code>MESSAGE_XXX</code> fields.
	*
	* @return the text status messagge (the wrapper's class field)
	*/
	public StringBuffer getMessage()					{		return this.message  ;						}

 	/**
	* Return the array of <code>Wrapper</code> records wrapped by this class.<br/>
	* It the web service returns just a single <code>Wrapper</code> object, this should be stored
	* into the <code>records[0]</code> element.
	*
	* @return the array of <code>Wrapper</code> records (the wrapper's class field)
	*/
	public Wrapper[] getRecords()						{		return this.records  ;						}

// **********************************************************************************************************
// **********************************			MUTATORS		*********************************************
// **********************************************************************************************************

	/**
	* Set the value of the status flag wrapped by this class.<br/>
	* Default implementation uses <code>1</code> for a good valid SOAP response, <code>0</code> elsewhere.
	* Subclasses can change this definitions by overriding the static <code>STATUS_XXX</code> fields.
	*
	* @param value the integer status flag (will be stored as class field)
	*/
	public void setStatus(int value)					{		this.status = value ;						}

	/**
	* Set the value of the text message wrapped by this class.<br/>
	* The message details the quality of the SOAP response described by the status flag.
	* Subclasses can change this definitions by overriding the static <code>MESSAGE_XXX</code> fields.
	*
	* @param value the text status messagge (will be stored as class field)
	*/
	public void setMessage(StringBuffer value)			{		this.message = value ;						}

	/**
	* Set the array of <code>Wrapper</code> records wrapped by this class.<br/>
	* It the web service returns just a single <code>Wrapper</code> object, this should be stored
	* into the <code>records[0]</code> element.
	*
	* @param value the array of <code>Wrapper</code> records ((will be stored as class field)
	*/
	public void setRecords(Wrapper[] value)				{		this.records = value ;						}

} // END OF CLASS