/**
* Title: ErrorMessage
* Description:
* Copyright:    Copyright (c) 2007-2010
* Company:      Morpheus Quality Technologies
* @author Stefano Adriani
* @version 1.0
*/

package adriani.jws.template.service ;

import adriani.jws.template.* ;

import java.util.* ;

/**
* This class is responsible for mapping all the error codes into user-friendly text messages.
* In order to improve performance, all text messages are instanciated as STATIC class attributes, in this
* way each text message is created only once (the firt time class is loaded).
* The actual mapping between error codes and text messages is defined by some custom properties files
* having the base name <strong><code>message_list.properties</code></strong>.<br/><br/>
*
* <strong>Remark:</strong> the <code>ResourceBundle.getBundle()</code> method retrieves the base file only
* when the system Locale does not match any locale-specific file.<br/><br/>
*
* Example: consider a machine using an ENGLISH Locale and finding only two properties files:
* <code>message_list.properties</code> (base) and <code>message_list_en.properties</code> (english).
* In this case calling <code>ResourceBundle.getBundle(Settings.MESSAGES, Locale.ITALY)</code>
* will load the <code>message_list_en.properties</code> file because it matches the system Locale!
* Hence, in order to force loading of the italian properties, it's necessary to provide also a file
* named: <code>message_list_it.properties</code>.
*/
public class ErrorMessage {

	/** GENERAL ERROR CODES																					*/
	public static final 	int		 	OK					= 1 ;
	public static final 	int 		DBA					= 2 ;
	public static final 	int 		MACRO				= 3 ;
	public static final 	int 		LOGIC				= 4 ;
	public static final 	int 		GENERAL				= 5 ;
	public static final		int 		COMMAND				= 6 ;

	/** DATABASE ERROR CODES																				*/
	public static final		int 		SQL					= 100 ;
	public static final 	int 		NO_DB				= 101 ;
	public static final 	int 		NO_RECORD			= 102 ;

	/** BUSINESS LOGIC ERROR CODES																			*/
	public static final 	int 		VALIDATION			= 200 ;

	/** OUTPUT MESSAGGES CODES																				*/
	public static final 	int			DATA_OK				= 300 ;
	public static final 	int 		NO_RESULT			= 301 ;

	/** SETTING DEFAULT MESSAGGES																			*/
	public static 			String		OK_MESSAGE 			= "OK" ;
	public static 			String		DBA_MESSAGE 		= "JAVA ERROR AT DBA LEVEL" ;
	public static 			String		MACRO_MESSAGE 		= "JAVA ERROR AT MACRO LEVEL" ;
	public static 			String		LOGIC_MESSAGE 		= "JAVA ERROR AT LOGIC LEVEL" ;
	public static 			String		GENERAL_MESSAGE 	= "GENERAL ERRROR" ;
	public static 			String		COMMAND_MESSAGE 	= "RECEIVED SERVICE COMMAND" ;
	public static 			String		SQL_MESSAGE 		= "SQL ERROR" ;
	public static 			String		NODB_MESSAGE 		= "NO CONNECTION TO THE DATABASE" ;
	public static 			String		NORECORD_MESSAGE 	= "NO RECORD" ;
	public static 			String		VALIDATION_MESSAGE 	= "VALIDATION FAILURE" ;
	public static 			String		DATAOK_MESSAGE		= "NO MESSAGE" ;
	public static 			String		NORESULT_MESSAGE 	= "NO DATA INTO DATABASE" ;

  	/** Beginning of each logging message produced by this class.											*/
	protected static final	String 		leadingName 		= "ErrorMessage - " ;

	// Static code block
	static {

		try {
			//ResourceBundle rb = ResourceBundle.getBundle(Settings.MESSAGES, Locale.ITALY) ;
			ResourceBundle rb = ResourceBundle.getBundle(Settings.MESSAGES, Locale.ENGLISH) ;

			// Retrieving string properties
			OK_MESSAGE				= rb.getString("error.ok") ;
			DBA_MESSAGE				= rb.getString("error.dba") ;
			MACRO_MESSAGE			= rb.getString("error.macro") ;
			LOGIC_MESSAGE			= rb.getString("error.logic") ;
			GENERAL_MESSAGE			= rb.getString("error.general") ;
			SQL_MESSAGE				= rb.getString("error.sql") ;
			NORECORD_MESSAGE		= rb.getString("error.no_record") ;
			NODB_MESSAGE			= rb.getString("error.no_db") ;
			VALIDATION_MESSAGE		= rb.getString("error.validation") ;
			COMMAND_MESSAGE			= rb.getString("service.command") ;
			DATAOK_MESSAGE			= rb.getString("output.data_ok") ;
			NORESULT_MESSAGE		= rb.getString("output.no_result") ;

			if (Settings.DETAILS) System.out.println(ErrorMessage.leadingName + "Loaded multilanguage file") ;

		} catch(MissingResourceException e) {
			System.err.println(ErrorMessage.leadingName + "Message properties file not found: using defaults") ;
			e.printStackTrace() ;
		}
	}

	/**
	* Return the text message mapped into the integer code as defined in the loaded properties file.
	*
	* @param code the integer code of the error
	* @return the string text messagge describing the error
	*/
	public static String getMessage(int code){

		switch(code) {
			case OK 				: return OK_MESSAGE ;
			case DBA 				: return DBA_MESSAGE ;
			case MACRO 				: return MACRO_MESSAGE ;
			case LOGIC 				: return LOGIC_MESSAGE ;
			case GENERAL		 	: return GENERAL_MESSAGE ;
			case COMMAND		 	: return COMMAND_MESSAGE ;
			case SQL 				: return SQL_MESSAGE ;
			case NO_RECORD			: return NORECORD_MESSAGE ;
			case NO_DB				: return NODB_MESSAGE ;
			case VALIDATION			: return VALIDATION_MESSAGE ;
			case DATA_OK			: return DATAOK_MESSAGE ;
			case NO_RESULT			: return NORESULT_MESSAGE ;
			default 				: return GENERAL_MESSAGE ;
		}
	}

} // END OF CLASS