/**
* Title: GeneralService
* Description:
* Copyright:    Copyright (c) 2009-2010
* Company:      Morpheus Quality Technologies
* @author Stefano Adriani
* @version 1.0
*/

package adriani.jws.template.service ;

import adriani.jws.template.* ;
import adriani.jws.template.db.* ;
import adriani.jws.template.util.* ;

import java.io.* ;
import java.util.* ;

import org.apache.axis2.context.* ;
import org.apache.axis2.addressing.* ;
import org.apache.axis2.description.* ;

/**
* The parent class for all the classes implementing the service business logic: <code>ServiceLogic</code>
* and <code>ServiceMacro</code>. At the moment this class offers following features for a general
* web service class:<br/><br/>
*
* <ol>
* <li>Load the general configuration from the properties file.</li>
* <li>Static configuration (on start time) of the required logger objects.</li>
* <li>Load keywords from the corresponding properties file (if required).</li>
* <li>Implementation of the remote command manager feature.</li>
* </ol>
*/
public abstract class GeneralService {

	/**
	* Reference to the <code>serviceAnalyzer</code> created in the static code block.
	* This object MUST BE static to ensure that all the subclasses of the <code>GeneralService</code> class
	* do refer to the same analyzer.
	*/
	protected static 		GeneralAnalyzer 	serviceAnalyzer ;
	
	/**
	* Reference to the <code>benchLogger</code> created in the static code block. 
	* The <code>benchLogger</code> logs on file single detailed information about the SQL execution times, this
	* is different from the <code>serviceAnalyzer</code> object, which collects only global and statistical data. 
	* <br/><br/>
	* Remark: this logger is always an istance of the <code>SystemLogger</code> class.
	*/
	protected static 		LoggerManager 		benchLogger ;

  	/** Logger instance for this class (static): must be set after WSLogger initialization.					*/
	protected static 		WSLogger			logger ;

  	/** Beginning of each logging message produced by this class.											*/
	protected static 		String 				leadingName 	= "GeneralService - " ;

	/**
	* Stateless constructor.
    */
    public GeneralService() {

		if (Settings.DETAILS) System.out.println(leadingName + "Created object: " + this) ;
	}

	/**
	* Method useful for debug and testing, the verify the scope of the web service objects.
    */
    public void finalize() {

		if (Settings.DETAILS) System.out.println(leadingName + "Finalized object: " + this) ;
	}

// **************************************************************************************************************
// **********************************		STATIC CODE BLOCK		*********************************************
// **************************************************************************************************************

	/**
	* We create the LOGGER within a static code block in order to execute this code only the first time
	* this class is loaded by the class loader. This is necessary because usually all the web service
	* Java objects are instanciated each time the service is invoked: see the <code>scope="request"</code>
	* attribute of the <code>services.xml</code> configuration file for details.
	*/
	static {

		// 1. Loading data from the configuration.properties file
		try {
			// Opening resource bundle
			ResourceBundle rb	= ResourceBundle.getBundle(Settings.CONFIG) ;

			// Parsing boolean properties
			String verbose		= rb.getString("service.verbose") ;
			String debug 		= rb.getString("service.debug") ;
			String analyze		= rb.getString("service.analyze") ;
			String manager		= rb.getString("service.manager") ;
			String benchmark	= rb.getString("service.benchmark") ;
			Settings.DETAILS	= verbose.equalsIgnoreCase("YES") ;
			Settings.DEBUG		= debug.equalsIgnoreCase("YES") ;
			Settings.ANALYZE	= analyze.equalsIgnoreCase("YES") ;
			Settings.MANAGER	= manager.equalsIgnoreCase("YES") ;
			Settings.BENCH		= benchmark.equalsIgnoreCase("YES") ;

			// Showing status
			System.out.println(GeneralService.leadingName + "Loaded configuration file: " + Settings.CONFIG + ".properties") ;

		} catch(MissingResourceException e) {
			System.err.println(GeneralService.leadingName + "Log configuration properties file not found: using defaults") ;
			e.printStackTrace() ;
		}

		// 4. Inizialize logger BEFORE any class could try to call the WSLogger factory
		if ( !WSLogger.isReady() ) WSLogger.initialize() ;
		else System.out.println(GeneralService.leadingName + "WSLogger already initialized") ;
		GeneralService.logger = WSLogger.getLogger(GeneralService.class) ;

		// 3. Eventually preparing the analysis environment
		if (Settings.ANALYZE) serviceAnalyzer = (GeneralAnalyzer) new ServiceAnalyzer() ;

		// 4. Printing general configuration on STDOUT
		String blank = "" ;
		System.out.print(GeneralService.leadingName + "Settings: " + (Settings.DEBUG?"Debug ":blank)) ;
		System.out.print((Settings.DETAILS?"Verbose ":blank) + (Settings.ANALYZE?"Analyze ":blank)) ;
		System.out.print((Settings.MANAGER?"Manager ":blank) + (Settings.BENCH?"Benchmark ":blank)) ;
		System.out.println() ;

		// 5. Loading service custom data
		ServiceData.loadLogicFlags(Settings.CONFIG) ;
		ServiceData.loadKeywords(Settings.KEYWORDS) ;
		//ServiceData.printKeywords() ;
	}

// **************************************************************************************************************
// **********************************			ACCESSORS		*************************************************
// **************************************************************************************************************

	/**
	* Return the <code>GeneralAnalyzer</code> object, used during development or benchmark testing to
	* collect statistical data about the service's performances.
	*
	* @return a reference to the <code>GeneralAnalyzer</code> object
    */
    public GeneralAnalyzer getServiceAnalyzer() 		{			return serviceAnalyzer ; 					}

// **************************************************************************************************************
// ***********************************			SERVICE MANAGER			*****************************************
// **************************************************************************************************************

	/**
	* Prints in the LOG file some information about the service configuration.<br/>
	* Generally speaking, the service configuration depends by the service actual duties, hence it's not
	* possibile to implement the method at this level, becuase this is a general purposes class. It's up
	* to the <code>ServiceLogic</code> or <code>ServiceMacro</code> subclass to implement this method in order
	* to show the actual service configuration.
	*/
	protected abstract void showConfiguration() ;

	/**
	* Allow to configure the service dynamically, without reloading the AAR package on the application server.<br/>
	* Following commands are available:<br/><br/>
	* <ul>
	* <li>1: Log service configuration.</li>
	* <li>2: Log system properties and runtime data.</li>
	* <li>3: Log the list of threads in this group.</li>
	* <li>4: Log metadata information about the database connection.</li>
	* <li>5: Log result of the <code>ServiceAnalyzer</code> class.</li>
	* <li>6: Switch the <code>Settings.ANALYZE</code> flag on/off.</li>
	* <li>7: Switch the <code>Settings.DETAILS</code> flag on/off.</li>
	* <li>8: Switch the <code>Settings.DEBUG</code> flag on/off.</li>
	* <li>9: Switch the <code>Settings.MANAGER</code> flag on/off.</li>
	* <li>10: Switch the <code>Settings.BENCH</code> flag on/off (and open/close the related logger).</li>
	* <li>11: Log the status of the JDBC connection pool.</li>
	* <li>12: Log some metadata information about the Axis2 enviroment (place-holder).</li>
	* </ul>
	* <br/><br/>
	* <strong>Remark:</strong> this method can be invoked only when the MANAGER mode is on. Hence, when
	* this method is called using the option number 9 the MANAGER mode will be turned off and this method can
	* not be invoked again! This can be used to "close" this access when security issues are required.
	*
	* @param key the integer command code as described above
	*/
	protected void manageService(int key) {

		switch(key) {
			case 1  :		showConfiguration() ;		// Abstract method: see the ServiceMacro class
							break ;
			case 2  :		showRuntime() ;
							break ;
			case 3  :		ThreadManager.logList() ;
							break ;
			case 4  :		Database.logMetaData() ;
							break ;
			case 5  :		// Can throw unpredictable exceptions if the analysis goes wrong
							try 				{ serviceAnalyzer.logResults() ;	}
							catch(Exception e)	{ e.printStackTrace() ;				}
							break ;
			case 6  :		Settings.ANALYZE = ! Settings.ANALYZE ;
							logger.logInfo(leadingName + "Analyzer mode: " + Settings.ANALYZE) ;
							break ;
			case 7  :		Settings.DETAILS = ! Settings.DETAILS ;
							logger.logInfo(leadingName + "Verbose mode: " + Settings.DETAILS) ;
							break ;
			case 8  :		Settings.DEBUG = ! Settings.DEBUG ;
							logger.logInfo(leadingName + "Debug mode: " + Settings.DEBUG) ;
							break ;
			case 9  :		Settings.MANAGER = ! Settings.MANAGER ;
							logger.logInfo(leadingName + "Service manager: " + Settings.MANAGER) ;
							break ;
			case 10 :		Settings.BENCH = ! Settings.BENCH ;
							if (Settings.BENCH) {
								benchLogger = WSLogger.getBenchmarkLogger() ;
								logger.logInfo(leadingName + "Activated benchmark: " + benchLogger) ;
							} else {
								benchLogger.close() ;
								logger.logInfo(leadingName + "Closed benchmark: " + benchLogger) ;
								benchLogger = null ;
							}
							break ;
			case 11  :		Database.logPoolsData() ;
							break ;
			case 12  :		logIntrospection() ;
							break ;
			default	:		logger.logInfo(leadingName + "Invalid command code") ;
		}
	}

	/**
	* Prints in the LOG file some information about the system run time. This method should be used only for
	* testing or debug.
	*/
	protected void showRuntime() {

		Runtime runtime = Runtime.getRuntime() ;
		ClassLoader loader = getClass().getClassLoader() ;
		Properties properties = System.getProperties() ;

		StringBuffer message = new StringBuffer("\nRUNTIME ENVIROMENT:\n") ;
		message.append("ClassLoader               : " 	+ loader + "\n") ;
		message.append("Runtime                   : " 	+ runtime + "\n") ;
		message.append("Java Home                 : " 	+ properties.getProperty("java.home") + "\n") ;
		message.append("Java VM Version           : " 	+ properties.getProperty("java.vm.version") + "\n") ;
		message.append("Java Runtime Version      : " 	+ properties.getProperty("java.specification.version") + "\n\n") ;
		message.append("Java Classpath            : " 	+ properties.getProperty("java.class.path") + "\n\n") ;
		message.append("Java Library Path         : " 	+ properties.getProperty("java.library.path") + "\n\n") ;
		message.append("Java Temp Directory       : " 	+ properties.getProperty("java.io.tmpdir") + "\n") ;
		message.append("Java JIT Compiler         : " 	+ properties.getProperty("java.compiler") + "\n") ;
		message.append("Java Extensions Directory : " 	+ properties.getProperty("java.ext.dirs") + "\n") ;
		message.append("Operative System          : " 	+ properties.getProperty("os.name") + "\n") ;
		message.append("Operative Architecture    : " 	+ properties.getProperty("os.arch") + "\n") ;
		message.append("Operative Version         : " 	+ properties.getProperty("os.version") + "\n") ;
		message.append("User Name                 : " 	+ properties.getProperty("user.name") + "\n") ;
		message.append("User Home                 : " 	+ properties.getProperty("user.home") + "\n") ;
		// Remark: user.dir is the directory used to launch the Java application. For example, if this web service
		// runs within Tomcat, and Tomcat has been started using a BATCH file from the path "C:\foo", then the
		// user.dir properties will be "C:\foo".
		message.append("User working Directory    : " 	+ properties.getProperty("user.dir") + "\n") ;
		message.append("Processors                : " 	+ runtime.availableProcessors()+ "\n") ;
		message.append("Free Java RAM             : " 	+ runtime.freeMemory()/1024 + " KB\n") ;
		message.append("Max Java RAM              : " 	+ runtime.maxMemory()/1024 + " KB\n") ;
		message.append("Total Java RAM            : " 	+ runtime.totalMemory()/1024 + " KB\n") ;
		// Optional properties (not always set)
		message.append("Oracle TSN names          : " 	+ properties.getProperty("oracle.net.tns_admin") + "\n") ;
		logger.logInfo(message.toString()) ;
	}

// **************************************************************************************************************
// **************************************		UTILITIES			*********************************************
// **************************************************************************************************************

   /**
    * Stops this thread for few milliseconds.<br/>
    *
    * @param time the time (in milliseconds) that this thread must be stopped
    */
    public void delay(long time) {

		try {
			Thread.currentThread().sleep(time) ;
		} catch (InterruptedException interrupted) {
			logger.logSevere(leadingName + ": error while sleeping") ;
		  	interrupted.printStackTrace() ;
		}
    }
	
	/**
	* Parse the string value of the input <code>String</code> into an integer value.
	* The <code>no_value</code> parameter represent the default integer value to be returned if
	* the parsing is not succesfull.
	*
	* @param string the input string object
	* @param no_value the int value to be returned if conversion fails
	* @return the integer value of the input string (or the <code>no_value</code> number)
	*/
	protected int valueOfString(String string, int no_value) {

		int result = no_value ;
		try 				{	result = Integer.parseInt( string.trim() ) ;								}
		catch(Exception e) 	{ 	result = no_value ;															}
		return result ;
	}

// **************************************************************************************************************
// ***********************************			INTROSPECTION			*****************************************
// **************************************************************************************************************

	/**
	* PLACE HOLDER: not used at the moment (sample code).
    */
    public void logIntrospection() {

		// Initialize
		StringBuffer text 				= new StringBuffer("\nAXIS2 ENVIROMENT:\n") ;
		MessageContext messageContext 	= MessageContext.getCurrentMessageContext() ;

		// Retrieving metadata
		if (messageContext != null) {

			// Basic data
			AxisService service 			= messageContext.getAxisService() ;
			AxisMessage message 			= messageContext.getAxisMessage() ;
			AxisOperation operation 		= messageContext.getAxisOperation() ;
			text.append("AxisService: " + service + "\n") ;
			text.append("AxisMessage: " + message + "\n") ;
			text.append("AxisOperation: " + operation + "\n") ;

			// EndPointreference metadata (if any)
			EndpointReference epr_1 = messageContext.getFaultTo() ;
			if (epr_1 != null) {
				ArrayList list = epr_1.getMetaData() ;
				text.append("Fault EPR: " + epr_1.getAddress() + "\n") ;
				if (list != null) for (Iterator i = list.iterator() ; i.hasNext() ; ) {
					text.append( (String) i.next() ) ;
				}
			}
			else text.append("Found null EndpointReference") ;

		}
		else text.append("Found null Message Context") ;

		// Logging metadata info
		text.append("\n") ;
		logger.logInfo(text.toString()) ;
	}

} // END OF CLASS