/**
* Title: ServiceData
* Description:
* Copyright:    Copyright (c) 2009-2010
* Company:      Morpheus Quality Technologies
* @author Stefano Adriani
* @version 1.0
*/

package adriani.jws.template.service ;

import adriani.jws.template.* ;

import java.io.* ;
import java.util.* ;

/**
* The container for all STATIC settings used by the business service logic (string values etc.).<br/>
* Instantiation of this class should not be necessary since these methods are called only once, within a static
* code block of the <code>ServiceMacro</code> class, in order to retrieve all the static settings used by the
* business logic. All these settings are stored into static attributes of this class, hence these parameters
* are available without the need to instanciate this class.
*/
public class ServiceData {

   	/** List of the keywords used by this service (loaded from the property file).							*/
	protected static 		StringBuffer[]		data_buffer ;

	/** If <code>true</code> the <code>ServiceMacro</code> will append two fake records to each call.		*/
	public static 			boolean 			FAKE 			= true ;		// Default value

	/** Time (in milliseconds) used to sleep the HTTP-request thread when testing concurrent call.			*/
	public static 			int					DELAY 			= 0 ;			// Default value

   	/** Beginning of each logging message for this class.													*/
	protected static 		String 				leadingName 	= "ServiceData - " ;

	/**
	* Load the logical settings for the business logic from the <code>configuration.properties</code> file and
	* store them into this class attributes. The actual name of the configuration file is defined by the
	* <code>Settings.CONFIG</code> parameter.
	*
	* @param the name of the properties file (usually is "configuration.properties")
	* @return <code>true</code> if all the logic flags have been loaded succesfully
    */
    public static boolean loadLogicFlags(String source) {

		// 1. Loading logical flags from the properties file
		try {
			ResourceBundle rb	= ResourceBundle.getBundle(source) ;

			// Retrieving boolean properties
			String fakedata		= rb.getString("logic.fake") ;
			FAKE				= fakedata.equalsIgnoreCase("YES") ;

			// Parsing integer properties
			String value		= rb.getString("logic.delay") ;
			try 				{ DELAY		= Integer.parseInt(value.trim()) ;		}
			catch(Exception e) 	{ DELAY 	= 0 ;									}	// Default value


		} catch(MissingResourceException e) {
			System.err.println(leadingName + "Configuration file not found: using defaults") ;
			e.printStackTrace() ;
			return false ;
		}

		// 2. Eventually showing logical settings on the STDOUT
		if (Settings.DEBUG) {
			String blank = "none " ;
			System.out.print(leadingName + "Logical flags: ") ;
			System.out.print(FAKE?"FAKE ":blank) ;
			System.out.println() ;
		}

		return true ;
	}

	/**
	* Load all the keywords from the <code>keywords.properties</code> file and store them into the
	* <code>data_buffer</code> class attribute. The actual name of the configuration file is defined by the
	* <code>Settings.KEYWORDS</code> parameter.
	*
	* @param the name of the properties file (usually is "keywords.properties")
	* @return <code>true</code> if the keywords have been succesfully loaded from the properties file
    */
    public static boolean loadKeywords(String source) {

		boolean fromFile ;
		String value = null ;
		Vector v = new Vector(100) ;

		// 1. Loading all values from the properties file
		try {
			ResourceBundle rb = ResourceBundle.getBundle(source) ;
			Enumeration e = rb.getKeys() ;
			while ( e.hasMoreElements() ) {
				String key = (String) e.nextElement() ;
				value = rb.getString(key) ;
				v.add( new StringBuffer(value) ) ;
			}
			fromFile = true ;
			if (Settings.DETAILS) System.out.println(leadingName + "Loaded data from: " + source + ".properties") ;

		} catch(MissingResourceException e) {
			fromFile = false ;
			System.err.println(leadingName + "Service data file not found") ;
			e.printStackTrace() ;
		}

		// 2. If succesfull loads the keywords into this class's array
		if (fromFile) {
			data_buffer = (StringBuffer[]) v.toArray( new StringBuffer[v.size()] ) ;
			if (Settings.DETAILS) System.out.println(leadingName + "Loaded " + data_buffer.length + " keywords") ;
		}	else {
			// 2.a Otherwise we use the hard-coded values (see below)
			data_buffer = getHardCodedValues() ;
			if (Settings.DETAILS) System.out.println(leadingName + "Using " + data_buffer.length + " hardcoded keywords") ;
		}
		return fromFile ;
	}

// *************************************************************************************************************
// ***************************					ACCESSORS				****************************************
// *************************************************************************************************************

	/**
	* Accessor method.
	*/
	public static StringBuffer[] getKeywords()				{		return data_buffer ;						}

// *************************************************************************************************************
// ***************************			COMMAND LINE UTILITIES				************************************
// *************************************************************************************************************

	/**
	* Create a <code>keywords.properties</code> data file containing the list of all the necessary keywords.
	* The input source file can be specified running the <code>SimpleClient</code> from the command line,
	* using the option <code>-c <em>file name</em></code>. If no path it's specified, the source file
	* will be searched in the current directory. Eventually it's possibile to specify an absolute path,
	* for example: <code>C:/Develop/name_list.txt</code>.
	*
	* @param source the source file name
	* @return true if the <code>keywords.properties</code> has been produced succesfully
    */
    public static boolean createKeywords(String source) {

		// 1. Initializing
		BufferedReader reader = null ;
		BufferedWriter writer = null ;
		String sink = "keywords.properties" ;
		System.out.println() ;

		// 2. Opening input stream using a BufferedReader
		try {
			FileReader fileReader = new FileReader(source) ;
			reader = new BufferedReader(fileReader) ;
			System.out.println(leadingName + "Opened file: " + source) ;
        } catch (FileNotFoundException filenotfound) {
            System.err.println(leadingName + "Could not open file: " + source) ;
			return false ;
        }

		// 3. Reading and writing data into a BufferedWriter
		int i = 0 ;
		String line = null ;
		try {
            FileWriter fileWriter = new FileWriter(sink) ;
			writer = new BufferedWriter(fileWriter) ;
			// Main cycle running the input file
			while ( (line = reader.readLine()) != null ) {
				line = i++ + "=" + line ;
				writer.write(line, 0, line.length()) ;
				writer.newLine() ;
			}
			reader.close() ;
			writer.close() ;
        } catch (IOException exception) {
            System.err.println(leadingName + "Error reading file") ;
			return false ;
        }

		if (Settings.DETAILS) System.out.println(leadingName + "Created data file: " + sink) ;
		return true ;
	}

	/**
	* Create a <code>keywords.java</code> data file containing the Java source definition of all the keywords.
	* This method should be used only during development, when the properties file should be always available,
	* to produce the hard-coded list that will be retrieved by the <code>getHardCodedValues()</code> method.
	*
	* @return true if the <code>keywords.java</code> has been produced succesfully
	* @see getHardCodedValues()
    */
    public static boolean produceJavaFile() {

		// 1. Initializing
		String sink = "keywords.java" ;
		BufferedWriter writer = null ;

		if (data_buffer == null) {
			System.err.println(leadingName + "Keywords not loaded") ;
			return false ;
		}

		try {
			int howMany = data_buffer.length ;
            FileWriter fileWriter = new FileWriter(sink) ;
			writer = new BufferedWriter(fileWriter) ;
			for (int i = 0; i < howMany; i++) {
				writer.write("buffer[" + i + "] = new StringBuffer(\"" + data_buffer[i] + "\") ;\n") ;
			}
			writer.close() ;
        } catch (IOException exception) {
            System.err.println(leadingName + "Error reading file") ;
			return false ;
        }

		if (Settings.DETAILS) System.out.println(leadingName + "Produced Java file: " + sink) ;
		return true ;
	}

	/**
	* Prints on the STDOUT the list of all the keywords loaded by this class.
	* This method should be used only during development, for testing or debug.
	*
	* @return true if the keywords list has been printed succesfully
    */
    public static boolean printKeywords() {

		if (data_buffer == null) return false ;
		int howMany = data_buffer.length ;

		System.out.println(leadingName + "KEYWORD LIST:\n") ;
		for (int i = 0; i < howMany; i++) System.out.println("buffer[" + i + "] = \"" + data_buffer[i] + "\"") ;
		System.out.println() ;

		return true ;
	}

// *************************************************************************************************************
// ***************************				HARD-CODED VALUES				************************************
// *************************************************************************************************************

	/**
	* Return an hard-coded list of all the necessary keywords.
	*
	* @return the <code>StringBuffer</code> array containing all the required keywords
	* @see produceJavaFile()
    */
    public static StringBuffer[] getHardCodedValues() {

		StringBuffer[] buffer = new StringBuffer[11] ;

		buffer[0]  = new StringBuffer("Axis2") ;
		buffer[1]  = new StringBuffer("Service") ;
		buffer[2]  = new StringBuffer("Web") ;
		buffer[3]  = new StringBuffer("Bug fixing") ;
		buffer[4]  = new StringBuffer("Java") ;
		buffer[5]  = new StringBuffer("Run") ;
		buffer[6]  = new StringBuffer("Deploy") ;
		buffer[7]  = new StringBuffer("Develop") ;
		buffer[8]  = new StringBuffer("Tomcat") ;
		buffer[9]  = new StringBuffer("Publish") ;
		buffer[10] = new StringBuffer("Test") ;

		return buffer ;
	}

} // END OF CLASS