/**
* Title: ServiceLogic
* Description:
* Copyright:    Copyright (c) 2007-2010
* Company:      Morpheus Quality Technologies
* @author Stefano Adriani
* @version 1.0
*/

package adriani.jws.template.service ;

import adriani.jws.template.* ;
import adriani.jws.template.db.* ;
import adriani.jws.template.eclipse.* ;
import adriani.jws.template.messages.* ;
import adriani.jws.template.util.* ;

import java.util.* ;

/**
* The main business logic class, implementing the high level behaviour of the service. All the issues
* concerning the preparation of the parameters for the DBA layer should not be handled here, but should be
* delegated to the <code>ServiceMacro</code> instead.<br/>
* Using this class it should be possibile to implement the service's skeleton just adding a codeblock
* like this:<br/><br/>
*
* <pre><code>
*	ServiceLogic logic      	= new ServiceLogic() ;
*	WrapperIn input             = new WrapperIn(adbBean) ;
*	WrapperDataOut output       = (WrapperXXXOut) logic.getOutput(input) ;
*	ADBBean element             = output.getElement() ;
* </code></pre>
* <br/><br/>
* Hence this class should implement a <code>getOutputXXX()</code> method for each method offered by the
* service skeleton. Every <code>getOutputXXX()</code> method should return the proper
* <code>WrapperXXXOut</code>, i.e. a custom subclass of the <code>WrapperOut</code> class.
*/
public class ServiceLogic extends GeneralService {

  	/** A reference to the <code>ServiceMacro</code> managing all the calls the necessary DBA's.			*/
	protected 					ServiceMacro 	serviceMacro ;

  	/** Static value identifying a request of the Information object (1st web service method).				*/
	protected static final		int  			INFORMATION		= 1 ;
  	/** Static value identifying a request of the 2nd web service method (PLACE-HOLDER).					*/
	//protected static final	int				SECOND			= 2 ;

  	/** The static value used to identify the INFO empty response.											*/
	protected static final		int  			SILENT 			= 1 ;
  	/** The static value used to identify the INFO empty response.											*/
	protected static final		int  			INFO 			= 2 ;
	/** The static value used to identify the WARNING empty response.										*/
	protected static final		int  			WARN 			= 3 ;
	/** The static value used to identify the ERROR empty response.											*/
	protected static final		int  			ERROR 			= 4 ;

  	/** Beginning of each logging message produced by this class.											*/
	protected static			String 			leadingName 	= "ServiceLogic - " ;

	/** Retrieving the logger instance to be used for this class (static).									*/
	protected static 			WSLogger		logger 			= WSLogger.getLogger(ServiceLogic.class) ;

	/**
	* Constructor, creating also the <code>ServiceMacro</code> object and storing it as class attribute.
    */
    public ServiceLogic() {

		this.serviceMacro = new ServiceMacro() ;
	}

	/**
	* Business logic for the first operation offered by the web service (currently the only one).<br/>
	* When the web service is invoked the <code>invokeBusinessLogic()</code> method of the
	* <code>XXXMessageReceiverInOut</code> class forwards the action to
	* <code>XXXSkeleton.operation()</code>, which should just calls this method.<br/>
	*
	* @param input the input <code>ADBBean</code> element wrapped by the <code>WrapperIn</code> class
	* @return the full output data wrapped by the <code>WrapperDataOut</code> class
	* @see TemplateSkeleton#getInformation(User user)
    */
    public WrapperDataOut getOutput(WrapperIn input) {

		// 1. Starting call
		ArrayList list = null ;
		if (Settings.ANALYZE) serviceAnalyzer.startCall() ;
		if (logger.isInfoEnabled()) logger.logInfo(leadingName + "Input INFORMATION request: " + input) ;

		// 2. Eventually managing the web service
		if ( Settings.MANAGER && manageService(input) )
		return (WrapperDataOut) getEmptyResponse(INFORMATION, SILENT, ErrorMessage.COMMAND) ;

		// 3. Adjusting input data
		input.validate() ;

		// 4. Retrieving data from the database
		if (ServiceData.FAKE) {
			try 					{ 	list = serviceMacro.getDummyResponse(input) ;						}
			catch(Exception e)		{	return (WrapperDataOut) handleMacroError(INFORMATION, e, true) ;	}
		} else {
			try 					{ 	list = serviceMacro.getData(input) ;								}
			catch(Exception e)		{	return (WrapperDataOut) handleMacroError(INFORMATION, e, true) ;	}
		}

		// 5. Checking if the database was not found
		int status = serviceMacro.getStatus() ;
		if (status == serviceMacro.NO_DATABASE)
		return (WrapperDataOut) getEmptyResponse(INFORMATION, SILENT, ErrorMessage.NO_DB) ;

		// 6. Eventually sleeping current thread to easy concurrency test or debug
		if (ServiceData.DELAY > 0) {
			if (logger.isInfoEnabled()) logger.logInfo(leadingName + "Stopping current thread for " + ServiceData.DELAY + " ms") ;
			delay(ServiceData.DELAY) ;
		}

		// Keep for testing or debug (forces a NO RESULT case)
		//list = null ;

		// 7. Creating output element
		WrapperData[] record = null ;
		WrapperDataOut output = null ;
		if (list != null) {
			record = (WrapperData[]) list.toArray( new WrapperData[list.size()] ) ;
			output = new WrapperDataOut(record) ;
			if (logger.isInfoEnabled()) logger.logInfo(leadingName + "Retrieved data from database") ;
		} else {
			// 7.a We get "list == null" only in the NO RECORD situation (no data in the database)
			return (WrapperDataOut) getEmptyResponse(INFORMATION, INFO, ErrorMessage.NO_RESULT) ;
		}

		// 8. Validating output element (and all his inner elements)
		//output = null ;	// Keep for testing or debug (forces a LOGIC ERROR)
		if (output != null) {
			output.setStatus(WrapperOut.STATUS_OK) ;
			if (logger.isInfoEnabled()) logger.logInfo(leadingName + "Validating output data") ;
			try 						{	output.validate() ;												}
			catch(WrapperOutFault e) 	{	return (WrapperDataOut) handleNoValidation(output, e) ;			}
		}
		// 8.a If we reach this block there's something wrong in the above logic
		if (output == null) {
			return (WrapperDataOut) getEmptyResponse(INFORMATION, ERROR, ErrorMessage.LOGIC) ;
		}

		// 9. Eventually stopping the chronwatch.
		if (Settings.ANALYZE) serviceAnalyzer.endCall(ErrorMessage.OK) ;

		// 10. Returning result
		return output ;
	}

// *************************************************************************************************************
// **********************************		WRAPPERS FACTORY		********************************************
// *************************************************************************************************************

	/**
	* The <strong>factory</strong> creating the proper <code>WrapperXXXOut</code> object that will be returned 
	* by the corresponding <code>getOutputXXX()</code> method. Every possible <code>WrapperXXXOut</code> class
	* must extend the parent <code>WrapperOut</code> class.<br/>
	* This factory should return the proper <code>WrapperXXXOut</code> object corresponding to the input
	* <em>method</em> parameter, which describes one of the operations offered by the skeleton
	* (listed using static integer values defined as class fields).
	* <br/><br/>
	* In current implementation this factory is accessed only the the <code>getEmptyResponse()</code>
	* method.
	*
	* @param method the logic method that invoked this method (see class fields)
	* @return the proper <code>WrapperOut</code> object for the corresponding logic method
    */
    public static WrapperOut getWrapperOut(int method) {

		// 1. Preparing the proper output object
		WrapperOut wrapper = null ;
		switch(method) {
			case	INFORMATION	:	wrapper = new WrapperDataOut() ;		break ;
			//case	SECOND		:	wrapper = new WrapperXXXOut() ;			break ;
		}

		// 2. Eventually logging creation failure
		if (wrapper == null) {
			logger.logSevere(leadingName + "WrapperOut factory failed for method " + method) ;
		}

		return wrapper ;
	}

// *************************************************************************************************************
// **********************************		ERRORS MANAGEMENT		********************************************
// *************************************************************************************************************

	/**
	* Prepare a valid <code>WrapperOut</code> subclass when the exception comes from a lower level, like the
	* <code>ServiceMacro</code> class or the <code>ServiceDBA</code> class. In this case the error message is
	* obtained by reading the necessary <code>StackTraceElement</code> objects.<br/>
	* Since the macro processing has been interrupted before producing any SOAP output, here we produce 
	* a brand new <code>WrapperOut</code> wrapper to ensure a correct SOAP output response, containing 
	* a message like "Java Macro Error".
	*
	* @param method the logic method that invoked this method (see class fields)
	* @param exception the <code>Exception</code> thrown from a lower level (<code>null</code> otherwise)
	* @param last if <code>true</code> the <code>getEmptyResponse()</code> method is invoked
	* @return a valid <code>WrapperOut</code> subclass with a message describing this situation
    */
    protected WrapperOut handleMacroError(int method, Exception exception, boolean last) {

		// 1. Closing connections to databases (we don't know if they have been left opened)
		ServiceDBA serviceDBA = serviceMacro.getServiceDBA() ;
		if (serviceDBA != null) {
			serviceDBA.closeConnection(Database.DB_01) ;
			serviceDBA.closeConnection(Database.DB_02) ;
			logger.logWarning(leadingName + "Forced connections check in for all databases") ;
		}

		// 2. Retrieving exception message
		String error = null ;
		if (exception != null) try {
			exception.printStackTrace() ;
			StackTraceElement[] stackTrace = exception.getStackTrace() ;
			error = stackTrace[0].getClassName() + "." + stackTrace[0].getMethodName() + " " + exception ;
		}
		catch(Exception e) {
			e.printStackTrace() ;
			error = "STACKTRACE ERROR (" + e.getMessage() + ") for " + exception.getMessage() ;
		}

		// 3. Logging error message
		String message = ErrorMessage.getMessage(ErrorMessage.LOGIC) ;
		logger.logSevere(leadingName + message + ": " + error) ;

		// If this is the last macro operation, we create the output wrapper in order to send a complete
		// SOAP message back to the client (the main logic will quit processing).
		if (last) return getEmptyResponse(method, SILENT, ErrorMessage.MACRO) ;
		else return null ;
	}

	/**
	* Prepare a valid <code>WrapperOut</code> subclass when we fail validation of the original
	* <code>WrapperOut</code> object produced by the main logic. The output message will cointain a message
	* with a reference to all the <code>ADBBean</code> objects responsible for the validation error.
	*
	* @param wrapper the instance of the <code>WrapperOut</code> subclass wrapping the output data
	* @param exception the <code>Exception</code> thrown when validating the <code>WrapperOut</code> element
	* @return a valid <code>WrapperOut</code> subclass with a message describing all the validation errors
    */
    protected WrapperOut handleNoValidation(WrapperOut wrapper, Exception e) {

		// Logging only the validation error message due to the <code>Wrapper</code> inner wrappers,
		// because any other error (if any) has already been logged.
		String invalid = ErrorMessage.getMessage(ErrorMessage.VALIDATION) ;
		String message = invalid + ": " + e.getMessage() ;
		logger.logSevere(leadingName + message) ;

		// Forcing message in the wrapper class field and in the ADBBean element
		wrapper.forceStatus(WrapperOut.STATUS_WRONG) ;
		wrapper.forceMessage(message) ;

		// Eventually ending analysis
		if (Settings.ANALYZE) serviceAnalyzer.endCall(ErrorMessage.VALIDATION) ;
		return wrapper ;
	}

	/**
	* Prepare a valid <code>WrapperOut</code> object when it's not possible to produce a valid result.
	* This method can be called in many different situations, each situation corresponding to a different
	* message to be logged while producing the empty response. These situations are identified using the
	* <code>state</code> param, which should be one of the integer values mapped by the
	* <code>ErrorMessage</code> class. In current implementation this method is called in following
	* situations: COMMAND, NO_DB, NO_RESULT, ERROR_LOGIC, ERROR_MACRO.
	* <br/><br/>
	* The <code>level</code> parameter can be: SILENT, INFO, WARN or ERROR.<br/>
	* If the SILENT constant it's used when no log message must be produced here, because we assume the caller
	* has already logged a message describing the situation. Otherwise we produce a log message with
	* the specified level.
	* Current implementation relies on the wrapper factory, implemented by the <code>getWrapperOut()</code>,
	* to produce the proper <code>WrapperOut</code> method for the current method.
	*
	* @param method a reference to one the SOAP methods exposed in the WSDL (see class fields)
	* @param level the integer value defining the level of the logged message
	* @param state the code of the message as described by the <code>ErrorMessage</code> class
	* @return a valid <code>WrapperOut</code> object with a message describing this situation
	* @see #getWrapperOut(int method)
    */
    protected WrapperOut getEmptyResponse(int method, int level, int state) {

		// 1. Ending analysis and getting error message
		if (Settings.ANALYZE) serviceAnalyzer.endCall(state) ;
		String message = ErrorMessage.getMessage(state) ;

		// 2. Eventually logging the error message
		switch(level) {
			case 	SILENT	:	break ;
			case 	INFO	:	if (logger.isInfoEnabled()) logger.logInfo(leadingName + message) ;		break ;
			case 	WARN	:	logger.logWarning(leadingName + message) ;								break ;
			case 	ERROR	:	logger.logSevere(leadingName + message) ;								break ;
			default			:	if (logger.isInfoEnabled()) logger.logInfo(leadingName + "EMPTY RESPONSE") ;
		}

		// 3. Getting the proper wrapper
		WrapperOut wrapper = getWrapperOut(method) ;

		// 4. Returning an empty valid response
		try 				{ wrapper.validate() ;						 								}
		catch(Exception e)	{ logger.logWarning(leadingName + "Failed validation of empty response") ;	}
		wrapper.forceStatus(WrapperOut.STATUS_WRONG) ;
		if (message != null) wrapper.forceMessage(message) ;
		return wrapper ;
	}

// *************************************************************************************************************
// **********************************		WEBSERVICE MANAGEMENT		****************************************
// *************************************************************************************************************

	/**
	* Allow to configure the service dynamically, without reloading the AAR package on the application server.
	* To send a valid command the service must be invoked passing a parameter starting with a sequence of
	* zeros followed by the command code. The number of zeros depend by the constrain defined in the WSDL
	* for the input parameter.<br/>
	* For example, if the input number parameter must be at least three digit long, then a valid command code
	* could be something like "0001", which will be interpreted as the command <strong>1</strong>.<br/><br/>
	*
	* @param input the <code>WrapperIn</code> wrapper containing the input data
	* @return <code>true</code> if the input number have been recognized as valid command code
	*/
    private boolean manageService(WrapperIn input) {

		// Retrieving input field
		String pwd = "000000" ;
		String leading = null ;
		StringBuffer cmd = input.getCommand() ;		// See WrapperIn for the command field definition

		// Checking if we have a valid command code
		int position = pwd.length() ;
		if (cmd.length() >= position) leading = cmd.substring(0, position) ; else return false ;
		if (! leading.equals(pwd) ) return false ;
		String action = cmd.substring(position) ;

		// Parsing input string
		int key = 0 ;
		try 				{ key = Integer.parseInt(action.trim()) ;	}
		catch(Exception e) 	{ key = 0 ;									}

		// Logging multilanguage message
		String message = ErrorMessage.getMessage(ErrorMessage.COMMAND) ;
		if (logger.isInfoEnabled()) logger.logInfo(leadingName + message + ": " + key) ;

		// Calling the super method
		manageService(key) ;
		return true ;
	}

	/**
	* Prints in the LOG file some information about the service configuration.<br/>
	* Notice the all classes extending the <code>GeneralService</code> class MUST implement this method.
	* Hence, in order to save coding this method forwards action to the <code>showConfiguration()</code> method
	* of the <code>ServiceMacro</code> class.
	*
	* @see ServiceMacro#showConfiguration()
	*/
	protected void showConfiguration() 			{		serviceMacro.showConfiguration() ;						}

} // END OF CLASS