/**
* Title: ServiceMacro
* Description:
* Copyright:    Copyright (c) 2007-2010
* Company:      Morpheus Quality Technologies
* @author Stefano Adriani
* @version 1.0
*/

package adriani.jws.template.service ;

import adriani.jws.template.* ;
import adriani.jws.template.messages.* ;
import adriani.jws.template.db.* ;
import adriani.jws.template.util.* ;

import java.util.* ;
import java.util.regex.* ;

/**
* This class provides all the logic used to prepare the calls to the necessary DBA, hence it's an intermediate
* layer between the upper main business logic (the <code>ServiceLogic</code> class) and the underlying
* datasource layer (the <code>ServiceDBA</code> class).<br/>
* The "interface policy" of this class is the following:<br/><br/>
*
* EXCEPTION: an unexpected exception is thrown, hence nothing is returned.<br/>
* DATA OK: the method return an <code>ArrayList</code> containing a set of <code>Wrapper</code> objects.<br/>
* NO DATA: the method return a <code>null</code> object.<br/>
* NO DATABASE: the method return a <code>null</code> object.<br/><br/>
*
* The specific case can be acknowledged by calling the <code>getStatus()</code> method after execution,
* which contains a text message only after a NO DATABASE case or a NO DATA case.<br/><br/>
*
* Implementaton guidelines:<br/><br/>
*
* <em>Architecture:</em> to grant loose coupling to the WSDL document and to the database, all the database
* issues should be handled by the <code>ServiceDBA</code> class, and all the <code>ADBBean</code> data
* conversion issues should be handled by the related wrapper in the <code>messages</code> package.<br/>
*/
public class ServiceMacro extends GeneralService {

	/** A reference to the <code>ServiceDBA</code> class accessing the necessary database's.				*/
	protected 				ServiceDBA 	serviceDBA ;

	/** Static final <code>String</code> value stored as class field to improve performances.				*/
	public static final 	String		EMPTY			= "" ;
	/** Static final <code>String</code> value stored as class field to improve performances.				*/
	public static final 	String		BLANK			= " " ;
	/** Static final <code>String</code> value stored as class field to improve performances.				*/
	public static final 	String		COLON			= ":" ;
	/** Static final <code>String</code> value stored as class field to improve performances.				*/
	public static final 	String		UNDER 			= "_" ;

  	/** An integer code describing the status of this macro after the last operation.						*/
	protected 				int 		status ;

  	/** A static code describing the status of this macro after the last operation.							*/
	protected static final	int  		OK	 			= 1 ;
  	/** A static code describing the status of this macro after the last operation.							*/
	protected static final	int  		NO_DATA			= 2 ;
  	/** A static code describing the status of this macro after the last operation.							*/
	protected static final 	int  		NO_DATABASE		= 3 ;

  	/** Beginning of each logging message printed on the STDOUT	.											*/
	protected 				String 		leadingName 	= "ServiceMacro - " ;

	/** Retrieving the logger instance to be used for this class (static).									*/
	protected static 		WSLogger	logger 			= WSLogger.getLogger(ServiceMacro.class) ;

	/**
	* Constructor, creating also the <code>ServiceDBA</code> object and storing it as class attribute.
    */
    public ServiceMacro() {

		this.serviceDBA = new ServiceDBA(this) ;
	}

	/**
	* MUST BE IMPLEMENTED WITH YOUR SERVICE LOGIC OR DATABASE QUERIES !
	* <br/><br/>
	* This means that setting "logic.fake = no" in the properties file will always produce a macro error!
	* <br/><br/>
	* @param input the input parameters wrapped by the <code>WrapperIn</code> class
	* @return an array list containing all the found records
    */
    public ArrayList getData(WrapperIn input) {

		// 1. Retrieving input parameters
		StringBuffer[] param = input.toArray() ;

		// 2. Opening connections to the database's
		if (! serviceDBA.openConnection(Database.DB_01) ) return handleNoDatabase(Database.DB_01) ;

		// 3. Retrieving records from the database
		StringBuffer[][] records = null ;
		// TO DO: CALL YOUR DBA LAYER FROM HERE !!!
		String message = "Please implement " + this.getClass().getName() + "#getData(WrapperIn input)" ;
		if (true) throw new UnsupportedOperationException(message) ;

		// 4. Preparing return object
		ArrayList list = null ;
		if (records != null) {
			// Keep for testing or debug (forces a VALIDATION ERROR)
			//records[0][0] = new StringBuffer("x") ;
			list = new ArrayList() ;
			for (int n = 0 ; n < records.length ; n++) {
				WrapperData output = new WrapperData() ;
				output.setFirst(records[n][0]) ;
				output.setSecond(records[n][1]) ;
				list.add(output) ;
			}
			status = OK ;
			if (logger.isInfoEnabled()) logger.logInfo(leadingName + "Fetched: " + list.size() + " records from database DB_01") ;
		}
		else status = NO_DATA ;

		// Keep for testing or debug (forces a ERROR AT MACRO LEVEL)
		//Object foo = null ;		foo.toString() ;

		// 7. Closing connections to databases
		serviceDBA.closeConnection(Database.DB_01) ;

		// 9. Returning result
		return list ;
	}

// *************************************************************************************************************
// **********************************		ERRORS MANAGEMENT		********************************************
// *************************************************************************************************************

	/**
	* Prepare a valid <code>ArrayList</code> object when the service logic fails because there is no
	* connection to one of the required databases. The output message will cointain a message like
	* "No connection to the database".
	*
	* @param database a reference to the code number of the missing database
	* @return a valid <code>ArrayList</code> object with a message describing this situation
	* @see Database
    */
    protected ArrayList handleNoDatabase(int database) {

		// Trying to close connections (we can have more than a database)
		serviceDBA.closeConnection(Database.DB_01) ;
		if (Settings.DEBUG) System.out.println(leadingName + "Forced connections check in for all databases") ;

		// Writing error on the log system
		status = NO_DATABASE ;
		String message = ErrorMessage.getMessage(ErrorMessage.NO_DB) + " (" + database + ')' ;
		logger.logSevere(leadingName + message) ;
		return null ;
	}

// *************************************************************************************************************
// **********************************			MUTATORS			********************************************
// *************************************************************************************************************

	/**
	* Accessor method.
	*/
	public void setStatus(int value)				{	this.status = value ;									}

// *************************************************************************************************************
// **********************************			ACCESSORS			********************************************
// *************************************************************************************************************

	/**
	* Accessor method.
	*/
	public ServiceDBA getServiceDBA() 				{	return this.serviceDBA ;								}

	/**
	* Accessor method.
	*/
	public int getStatus() 							{	return this.status ;									}

// *************************************************************************************************************
// **********************************		SERVICE MANAGEMENT		********************************************
// *************************************************************************************************************

	/**
	* Prints in the LOG file some information about the service configuration.
	* This method should be used only for testing or debug.
	*/
	protected void showConfiguration() {

		// Preparing output message
		StringBuffer message = new StringBuffer("\nSERVICE CONFIGURATION (") ;
		message.append(Settings.class + ")\n\n") ;
		message.append("Version           : " + Settings.VERSION	+ "\n") ;
		message.append("Debug flag        : " + Settings.DEBUG 		+ "\n") ;
		message.append("Verbose flag      : " + Settings.DETAILS 	+ "\n") ;
		message.append("Analyze flag      : " + Settings.ANALYZE 	+ "\n") ;
		message.append("Manager flag      : " + Settings.MANAGER 	+ "\n") ;
		message.append("Benchmark flag    : " + Settings.BENCH 		+ "\n") ;
		message.append("\n") ;
		message.append("Logger system         : " + (Settings.JAVA?"Java":EMPTY) + (Settings.LOG4J?"Log4j":EMPTY) + "\n") ;
		message.append("Log path              : " + logger.getParentDir() 		+ "\n") ;
		message.append("Log rotation time     : " + logger.getRotationTime()	+ "\n") ;
		if (Settings.JAVA || Settings.BENCH){
			message.append("Java Logger directory : " + Settings.LOGDIR + "\n") ;
			if (Settings.JAVA) message.append	("Java Logger name      : " + Settings.LOGGER + "\n") ;
			if (Settings.BENCH) message.append	("Benchmark logger      : " + Settings.BENCHMARK + "\n") ;
		}
		message.append("\n") ;
		message.append("Configuration file           : " + Settings.CONFIG 			+ ".properties\n"	) ;
		message.append("Log4j configuration file     : " + Settings.LOG_CONFIG 		+ ".properties\n"	) ;
		message.append("Multilanguage file           : " + Settings.MESSAGES		+ ".properties\n"	) ;
		message.append("Fake dba queries             : " + ServiceData.FAKE 		+ "\n"				) ;
		message.append("Logic delay time             : " + ServiceData.DELAY		+ " [ms]\n"			) ;
		message.append("\n") ;

		logger.logInfo(message.toString()) ;
	}

// *************************************************************************************************************
// **********************************			TEST & DEBUG			****************************************
// *************************************************************************************************************

	/**
	* Return an <code>ArrayList</code> emulating the actual macro response, but filled with dummy data.
	* This method is called by the business logic when the FAKE configuration flag is set to true,
	* this means that the actual macro call (<code>getData()</code>) it's not executed and no
	* database connections are opened.
	*
	* @param input the input parameters wrapped by the <code>WrapperIn</code> class
	* @return an array list containing some dummy records
    */
    public ArrayList getDummyResponse(WrapperIn input) {

		// 1. Selecting number of records to be returned (just for training purposes)
		StringBuffer[] param = input.toArray() ;
		int howMany = valueOfString(param[1].toString(), 1) ;
		if (howMany > 10) howMany = 10 ;
		
		// 2. Retrieving fake records
		StringBuffer[][] records = serviceDBA.fakeQuery(param, howMany, 2) ;
		if (logger.isInfoEnabled()) logger.logInfo(leadingName + "Fetched " + records.length + " fake fault records") ;

		// 3. Preparing return object
		ArrayList list = new ArrayList() ;

		// 4. Wrapping records and adding to list
		for (int n = 0 ; n < records.length ; n++) {
			WrapperData output = new WrapperData() ;
			output.setFirst(records[n][0]) ;
			output.setSecond(records[n][1]) ;
			list.add(output) ;
		}

		// 5. Returning array
		return list ;
	}

} // END OF CLASS