/**
* Title: ExternalLogger
* Description:
* Copyright:    Copyright (c) 2009-2010
* Company:      Morpheus Quality Technologies
* @author Stefano Adriani
* @version 1.0
*/

package adriani.jws.template.util ;

import adriani.jws.template.* ;

import java.util.* ;
import org.apache.log4j.* ;

/**
* The class managing the logging mechanism when using the Log4j library. Notice that in the Log4j paradigm
* this class is not necessary, since any class should contain a class attribute like the following:<br/><br/>
*
* <code>protected static Logger logger = Logger.getLogger(MyApp.class) ;</code><br/><br/>
*
* This project needs this class only to ensure compatibility with the logging system offered by the
* <code>SystemLogger</code> class, in this way it's possibile to switch between the two logging system
* simply changing the <code>logger.type</code> property in the service configuration file.
*
* @see SystemLogger
*/
public class ExternalLogger implements LoggerManager {

	/** The absolute log path retrieved from the configuration file.										*/
	protected 				String 		loggerPath ;

  	/** The rotation date pattern set for this logger.														*/
	protected 				String		datePattern ;

  	/** Beginning of each logging message printed on the STDOUT.											*/
	protected static 		String 		leadingName 		= "ExternalLogger - " ;

	/**
	* This method forces the Logger to load configuration from the <code>log4j.properties</code> file.<br/>
	* If the properties file is not found, this method set the basic configuration for all the
	* Log4j logger objects.
	*
	* @param path not used (required only to implement the <code>LoggerManager</code> interface)
	* @param logLevel not used (required only to implement the <code>LoggerManager</code> interface)
	* @param rotate not used (required only to implement the <code>LoggerManager</code> interface)
	* @param mode not used (required only to implement the <code>LoggerManager</code> interface)
	*/
	public void init(String path, String logLevel, int rotate, boolean mode) {

		try {
			ResourceBundle bundle = ResourceBundle.getBundle(Settings.LOG_CONFIG) ;
			Properties properties = getProperties(bundle) ;
			PropertyConfigurator.configure(properties) ;
			if (Settings.DEBUG) System.out.println(leadingName + "Using Log4j configuration: " + Settings.LOG_CONFIG + ".properties") ;
		} catch(Exception e) {
			BasicConfigurator.configure() ;
			if (Settings.DEBUG) System.out.println(leadingName + "Using Log4j default configuration") ;
		}
	}

	/**
	* NOT USED AT THE MOMENT (place holder).
	*
	* @param path the path reference for the log file
	* @param level the logging minimal level as a Logger.Level value (INFO, WARNING or SEVERE)
	* @param mode if <code>true</code> creates the standard log file, if <code>false</code> creates the
	* benchmark analysis file
	* @see getLogFile(String optionalPath, boolean mode)
	*/
	public void createLogger(String path, java.util.logging.Level level, boolean mode) {

		System.err.println(leadingName + "Method not implemented: createLogger()") ;
	}

	/**
	* NOT USED AT THE MOMENT (place holder).
	*/
	public void rotate() {

		System.err.println(leadingName + "Method not implemented: rotate()") ;
	}

	/**
	* NOT USED AT THE MOMENT (place holder).
	*/
	public boolean close() {

		System.err.println(leadingName + "Method not implemented: close()") ;
		return false ;
	}

// **********************************			ACCESSORS		*********************************************

	/**
	*
	* Return the <code>parentDir</code> path used as base directory to locate the log file.<br/>
	* Usually this parameter is retrieved from the <em>RootLogger</em> reference, because the method
	* <code>root.getAllAppenders()</code> returns only the EXPLICIT appenders, and not the appenders
	* obtained from a parent logger. In this case we do not use the <em>RootLogger</em>, because we
	* assume the Log4j configuration file is something like:
	* <br/><br/>
	* <pre><code>
	* log4j.additivity.template         = false
	* log4j.additivity.template.dba     = false
	* log4j.additivity.template.logic   = false
	* log4j.logger.template             = INFO, stdout, myfile
	* log4j.logger.template.dba         = INFO, stdout, myfile
	* log4j.logger.template.logic       = INFO, stdout, myfile
	* </code></pre>
	* <br/><br/>
	* Hence each logger object has his own explicit reference to the output file (<em>myfile</em>), for this
	* reason we can use direcyly: <code>smartLogger.getAllAppenders()</code>.
	*
	* @param logger a valid <code>org.apache.log4j.Logger</code> instance
	* @return the current parent directory for logging on the file system
	*/
	public String getParentDir(Object logger) {

		if (loggerPath == null) {

			Enumeration e = null ;
			Logger root = (Logger) logger ;
			// Logger root = logger.getRootLogger() ;			// See Javadoc documentation above
			if (root != null) e = root.getAllAppenders() ;
			if (Settings.DETAILS) System.out.println(leadingName + "Getting parent directory for : " + root) ;

			// Looking for the first instance of a FileAppender object
			if (e != null) while (e.hasMoreElements()) {
				Appender appender = (Appender) e.nextElement() ;
				if (appender instanceof FileAppender) {
					FileAppender fileAppender = (FileAppender) appender ;
					loggerPath = fileAppender.getFile() ;
					return loggerPath ;
				}
			}
		}

		return loggerPath ;
	}

	/**
	* Not used in current implementation, since roll over and rotation of the log file is performed
	* automatically by the Log4j external module.
	*
	* @return a <code>null</code> reference, since logger rotation is performed by Log4j
	*/
	public RotateThread getRotateThread() {

		System.err.println(leadingName + "Method not implemented: getRotateThread()") ;
		return null ;
	}

	/**
	* Return the rotation time set for this logger, already formatted and ready to print.<br/>
	* Usually this parameter is retrieved from the <em>RootLogger</em> reference, because the method
	* <code>root.getAllAppenders()</code> returns only the EXPLICIT appenders, and not the appenders
	* obtained from a parent logger. In this case we do not use the <em>RootLogger</em>, because we
	* assume the Log4j configuration file is something like:
	* <br/><br/>
	* <pre><code>
	* log4j.additivity.template         = false
	* log4j.additivity.template.dba     = false
	* log4j.additivity.template.logic   = false
	* log4j.logger.template             = INFO, stdout, myfile
	* log4j.logger.template.dba         = INFO, stdout, myfile
	* log4j.logger.template.logic       = INFO, stdout, myfile
	* </code></pre>
	* <br/><br/>
	* Hence each logger object has his own explicit reference to the output file (<em>myfile</em>), for this
	* reason we can use direcyly: <code>smartLogger.getAllAppenders()</code>.
	*
	* @param logger a valid <code>org.apache.log4j.Logger</code> instance.
	* @return a string describing the rotation time set for this logger.
	*/
	public String getRotationTime(Object logger) {

		if (datePattern == null) {

			Enumeration e = null ;
			Logger root = (Logger) logger ;
			// Logger root = smartLogger.getRootLogger() ;	// See Javadoc documentation above
			if (root != null) e = root.getAllAppenders() ;
			if (Settings.DETAILS) System.out.println(leadingName + "Getting rotation time for : " + root) ;

			// Assuming the file is the first instance of a DailyRollingFileAppender object
			if (e != null) while (e.hasMoreElements()) {
				Appender appender = (Appender) e.nextElement() ;
				if (appender instanceof DailyRollingFileAppender) {
					DailyRollingFileAppender daily = (DailyRollingFileAppender) appender ;
					datePattern = daily.getDatePattern() ;
					return datePattern ;
				}
			}
		}

		return datePattern ;
	}

// ******************************************************************************************************
// **********************************		UTILITIES		*********************************************
// ******************************************************************************************************

	/**
	* Convert ResourceBundle into a Properties object.<br/>
	* This method is necessary because Axis2 uses the Log4j mechanism to log events, as defined in the
	* <code>log4j.properties</code> file in the <code>axis2\WEB-INF\classes</code> directory. This means that
	* all the Logger objects instanciated by the Axis2 classloader will use the configuration speficied by
	* the <code>log4j.properties</code> mentioned above. Hence, to avoid setting the logging configuration in
	* that file, we load our Log4j settings as a <code>ResourceBundle</code> object and then we convert it
	* into a <code>Properties</code> object.
	*
	* @param resource the resource bundle to convert
	* @return the properties version of the resource bundle
	*/
	protected Properties getProperties(ResourceBundle resource) {

		Enumeration keys = resource.getKeys() ;
		Properties properties = new Properties() ;

		while ( keys.hasMoreElements() ) {
			String key = (String) keys.nextElement() ;
			properties.put(key, resource.getString(key)) ;
			//System.out.println("Set " + key + " = " + resource.getString(key)) ;
		}

		return properties ;
	}

} // END OF CLASS