/**
* Title: SystemLogger
* Description:
* Copyright:    Copyright (c) 2007-2010
* Company:      Morpheus Quality Technologies
* @author Stefano Adriani
* @version 1.0
*/

package adriani.jws.template.util ;

import adriani.jws.template.* ;

import java.io.*;
import java.net.* ;
import java.util.* ;
import java.util.logging.* ;

/**
* This class creates an unique Java <code>Logger</code> object shared by the whole application.
* This object is created within a static code block in the <code>ServiceLogic</code>, in this way we create
* only one logger no matters how many children of <code>ServiceLogic</code> are used in the project.<br/><br/>
*
* This <code>Logger</code> writes directly on the file system. By default the logs files will be produced into 
* the <code>logs</code> folder in the <code>parentDir</code> directory, which is usually the path where the 
* Virtual Machine in launched. If this class is created by a webapp within the Apache-Tomcat server, 
* the <code>parentDir</code> should be the <code>TOMCAT_HOME</code> directory itself. 
* If the class is used within a web service using Axis2, then the <code>parentDir</code> should be 
* <code>work\Catalina\localhost\axis2\_axis2</code>.<br/>
* For this reason is <strong>strongly</strong> recommented to specify a custom log path when calling
* the <code>init()</code> method.
*
* @see Settings#LOGDIR
*/
public class SystemLogger implements LoggerManager {

  	/** The absolute log path retrieved from the configuration file.										*/	
	protected 					String 			loggerPath ;

  	/** The threshold logging level (default is <code>Level.INFO</code>).									*/	
	protected 					Level 			level 			= Level.INFO ;

  	/** The thread class responsibile for rotation of the log file.											*/	
	protected 					RotateThread 	rotateThread ;
	
  	/** 
	* If <code>true</code> this object handles the standard logger, if <code>false<code> it refers to 
	* the benchmark logger.
	*/	
	protected 					boolean 		mode ;

  	/** 
	* Used only when the <code>loggerPath</code> is not specified and the logging path must be generated
	* automatically by this class. This value is stored as class field to ease test or debug of the 
	* path generation mechanism.
	*/
	protected 					String 			parentDir ;	

  	/** Beginning of each logging message printed on the STDOUT.											*/
	protected 					String 			leadingName 	= "SystemLogger - " ;
	
	/**
	* This method should be called when initializing the application, webapp or service, in order to create
	* and configure the proper <code>Logger</code> object.<br/><br/>
	*
	* Input parameters are not mandatory. Calling this method passing a null <code>path</code> will create
	* the logger file in the default directory (stored in the <code>parentDir</code> class field).<br/>
	* This method does not create the log folder abruptely in the <code>parentDir</code> directory, 
	* instead the log folder is created some levels above this directory, using the <code>climbPath()</code> 
	* method.<br/><br/>
	*
	* This method takes also care to create and run the thread used by <code>RotateThread</code> class.
	*
	* @param path an ALREADY existing path from which logging folders must be created, in the form
	* <code>C:\\Project\\Temp</code> (Windows) or <code>/home/temp</code> (Unix)
	* @param logLevel the logging minimal level as a String (INFO, WARNING or SEVERE)
	* @param rotate the timeout (in minutes) for rotation of the log file
	* @param mode if <code>true</code> initializes the standard log file, if <code>false</code> initializes the 
	* benchmark analysis file
	* @see #climbPath(String path, int levels)
	*/
	public void init(String path, String logLevel, int rotate, boolean mode) {

		// Setting global settings
		this.mode = mode ;		
		this.loggerPath = new String(path) ;
		if (Settings.DETAILS) System.out.println() ;
		if (Settings.DEBUG) System.out.println(leadingName + "Set log folder: " + loggerPath) ;
		if (logLevel.equalsIgnoreCase("SEVERE")) level = Level.SEVERE ;
		else if (logLevel.equalsIgnoreCase("WARNING")) level = Level.WARNING ;
		
		// Creating the Logger object
		try {
			String logFile = getLogFile(loggerPath, mode) ;
			createLogger(logFile, level, mode) ;
		} catch(Exception e) {
			System.err.println(leadingName + "Impossible to create the logging stream") ;
			System.err.println("Error message: " + e.getMessage() ) ;
		}
		
		// Starting the Rotate Thread
		this.rotateThread = new RotateThread(this, rotate) ;
		Thread thread = new Thread(rotateThread) ;
		thread.setName("Rotate Thread") ;
		thread.start() ;
	}

	/**
	* This method is called within the <code>init()</code> method to create the <code>Logger</code> and the
	* related <code>StreamHandler</code> objects. Before calling this method, the code within the 
	* <code>init()</code> method must produce a proper path for the log file, for example using the 
	* <code>getLogFile()</code> method.<br/>
	* Notice that this method could be private or protected, since usually is used only internally. We choose
	* to define it public to allow a more flexible way to write the code that will call the 
	* <code>init()</code> method.
	*
	* @param path the MANDATORY path reference for the log file
	* @param level the logging minimal level as a Logger.Level value (INFO, WARNING or SEVERE)
	* @param mode if <code>true</code> creates the standard log file, if <code>false</code> creates the 
	* benchmark analysis file
	* @see getLogFile(String optionalPath, boolean mode)
	*/
	public void createLogger(String path, Level level, boolean mode) {

		if (path != null) {
			
			// Creating the "rough "file stream
			FileOutputStream file = null ;
			try  								{	file = new FileOutputStream(path) ;								    }
			catch (FileNotFoundException e) 	{ 	System.err.println(leadingName + "Could not open LOG file") ; }

			// Creating a stream handler to format the log file
			SimpleFormatter formatter = new SimpleFormatter() ;
			StreamHandler handler = new StreamHandler(file, formatter) ;
			if (Settings.DETAILS) System.out.println(leadingName + "Set handler on file: " +  path + "\n") ;
			
			// Creating the logger object.
			// Notice that following code do not consider the option that we are using the Log4j mechanism,
			// since we are using this class instead of the Log4j module.
			Logger logger ;
			if (mode) logger = java.util.logging.Logger.getLogger(Settings.LOGGER) ;
				else logger = java.util.logging.Logger.getLogger(Settings.BENCHMARK) ;
			
			// Adding the stream handler to the logger
			try 				{ 	logger.addHandler(handler) ;														}
			catch(Exception e)	{ 	System.err.println(leadingName + "Impossible to add the Logging Handler") ;		}
			// Setting the minimal logging level
			logger.setLevel(Level.INFO) ;
			logger.info(leadingName + "Created log file: " + path) ;
			logger.info(leadingName + "Set logging level: " + level) ;
			logger.setLevel(level) ;

		} else {
			System.err.println("SystemLogger: log file not opened (found null path)") ;
		}
	}

	/**
	* Discards the old <code>Handler</code> used by the default <code>Logger</code> and creates a new one.
	* This method is called periodicaly by the <code>RotateThread</code>.
	*
	* @see RotateThread
	*/
	public void rotate() {

		// Getting the logger reference	
		Logger logger ;
		if (mode) logger = java.util.logging.Logger.getLogger(Settings.LOGGER) ;
			else logger = java.util.logging.Logger.getLogger(Settings.BENCHMARK) ;

		// Reporting the operation
		String path = getLogFile(loggerPath, mode) ;		
		String loggerName = mode ? "logger" : "benchmark" ;
		if (Settings.DETAILS) System.out.println(leadingName + "changing " + loggerName) ;

		// Closing the only one handler	and creating a new logger
		if (path != null) {
			Handler[] handlers = logger.getHandlers() ;
			if (handlers[0] != null) {
				handlers[0].close() ;
				logger.removeHandler(handlers[0]) ;
			}
			createLogger(path, level, mode) ;
		} else {
			logger.warning(leadingName + "keeping the old logger object") ;
			System.err.println(leadingName + "Keeping the old logger object");
		}
	}
	
	/**
	* Close the logger by closing all related handlers. The rotation thread will execute just another cycle,
	* then will exit. This method should be required only by the <code>SystemLogger</code> class.
	*
	*/
	public boolean close() {
	
		// Ending the rotation thread
		rotateThread.active = false ;
		rotateThread.running = false ;

		// Getting the logger reference	
		Logger logger ;
		if (mode) logger = java.util.logging.Logger.getLogger(Settings.LOGGER) ;
			else logger = java.util.logging.Logger.getLogger(Settings.BENCHMARK) ;		

		// Closing the only one handler		
		Handler[] handlers = logger.getHandlers() ;
		if (handlers[0] != null) {
			handlers[0].close() ;
			logger.removeHandler(handlers[0]) ;
		}
		// Reporting the operation result
		String loggerName = mode ? "logger" : "benchmark" ;		
		if (Settings.DETAILS) System.out.println(leadingName + "closed: " + loggerName) ;	
		return true ;
	}
	
// ************************************************************************************************************
// **********************************			ACCESSORS		***********************************************
// ************************************************************************************************************
	
	/**
	* Return the <code>parentDir</code> path used as base directory to locate the log file.
	*
	* @param logger should be <code>null</code>, since is not necessary any logger instance
	* @return the automatically generated parent directory for the logger
	*/
	public String getParentDir(Object logger) 	{		return parentDir ;									}
	
	/**
	* Return the <code>RotateThread</code> object used to rotate the log file.
	*
	* @return a reference to the <code>RotateThread</code> object
	*/
	public RotateThread getRotateThread() 		{		return rotateThread ;								}
	
	/**
	* Return the rotation time set for this logger, already formatted and ready to print.
	*
	* @param logger should be <code>null</code>, since is not necessary any logger instance
	* @return a string describing the rotation time set for this logger
	*/
	public String getRotationTime(Object logger) {
	
		long sleepTime = 0 ;
		if (rotateThread != null) sleepTime = rotateThread.getSleepTime() ;
		if (sleepTime > 0) return  (sleepTime/3600000) + " hours" ;
			else return "unset" ;
	}
	
// ************************************************************************************************************	
// **********************************	LOG FILE MANAGEMENT		***********************************************
// ************************************************************************************************************
	
	/**
	* Produce the complete qualified name (path and file name) for the log file. If the dir structure does not 
	* exist the method creates also the required subfolders. The logging file path should be set manually 
	* specifying a valid path with the parameter <code>optionalPath</code>. In current implementation 
	* the <code>optionalPath</code> is stored in the <code>loggerPath</code> class field.
	*
	* @param optionalPath the path that should be specified manually in the configuration file
	* @param mode if <code>true</code> provides the standard log file, if <code>false</code> provides the 
	* "Performances Analysis" file
	* @return the produced complete path of the log file
	*/
	protected String getLogFile(String optionalPath, boolean mode) {

		int climb = 3 ;									// Number of levels to climb from the parentDir path
		boolean done1 = false ;							// true if the "logs" folder has been created or found
		boolean done2 = false ;							// true if the "month" subfolder has been created or found
		String logs = Settings.LOGDIR ;					// Name of the base logging folder		

		// 1. Retrieving current month to create the related directory
		GregorianCalendar calendar = new GregorianCalendar() ;
		int month = calendar.get(Calendar.MONTH) + 1 ;				// January is month zero !!!

		// 2. Eventually generating automatically the parent dir for the log file
		if (optionalPath == null) {
			String runtime = getLocationPath() ;	// Eventually use: String runtime = getResourcePath() ;		
			if (runtime != null) parentDir = parseExtension(runtime) ; else return null ;
			parentDir = climbPath(parentDir, climb) ;
		} else {
			parentDir = new String(optionalPath) ;
		}

		// 3. Eventually creating the "logs" folder (child of the parentDir folder)
		boolean found = false ;		
		char fileSeparator = '/' ;
		//char fileSeparator = File.separatorChar ;		// Not necessary (it's an URL, not a path)
		File current = new File(parentDir) ;
		String[] list = current.list() ;
		for (int i = 0; i < list.length; i++) {
			if ( list[i].equalsIgnoreCase(logs) ) 	{ found = true ; break ; }
		}
		File logDir = new File(parentDir + fileSeparator + logs) ;
		if (!found) done1 = logDir.mkdir() ; else done1 = true ;
		if (Settings.DETAILS) System.out.println(leadingName + "Parent directory: " + parentDir) ;		

		// 4. Looking for the "logs\month" subfolder (for example: "logs\2" stays for february)
		found = false ;
		list = logDir.list() ;
		for (int i = 0; i < list.length; i++) {
			if ( list[i].equalsIgnoreCase("" + month) ) { found = true ; break ; }
		}
		File subfolder = new File(parentDir + fileSeparator + logs + fileSeparator + month) ;
		if (!found) done2 = subfolder.mkdir() ; else done2 = true ;

		// 5. Checking if the current TXT file already exists (same hour and minute)
		found = false ;
		list = subfolder.list() ;
		String name = getFileName(calendar, mode, "txt") ;				
		for (int i = 0; i < list.length; i++) {
			if ( list[i].equalsIgnoreCase(name) ) { found = true ; break ; }
		}
		
		// 6. Avoiding to rewrite previous file (in case of shutdown and reboot within a minute)
		if (found) name = getFileName(calendar, mode, "bis") ;				
		
		// 7. Returning the path for the log file
		String result = parentDir + fileSeparator + logs + fileSeparator + month + fileSeparator + name ;
		if ((done1) && (done2))	return result ; else return null ;
	}
	
	/**
	* Produce the file name (without the path) for the log file.
	*
	* @param calendar a valid <code>GregorianCalendar</code> instance
	* @param mode if <code>true</code> produces the name of standard log file, if <code>false</code> produces 
	* the name for the "Performances Analysis" file
	* @param ext the extension to be used for the log file
	* @return the log file name, for example: <code>d11_h10_m30.txt</code>
	*/
	protected String getFileName(GregorianCalendar calendar, boolean mode, String ext) {
		
		String name ;
		int minute = calendar.get(Calendar.MINUTE) ;
		int hour = calendar.get(Calendar.HOUR_OF_DAY) ;
		int day = calendar.get(Calendar.DAY_OF_MONTH) ;
		if (mode) name = "d" + day + "_h" + hour + "_m" + minute + "." + ext ;
			else name = "analysis_" + day + "_" + hour + "_" + minute + "." + ext ;
		return name ;
	}	
	
// ************************************************************************************************************	
// **********************************		PATH LOCATORS	***************************************************
// ************************************************************************************************************

	/**
	* Return the file path describing location of this class, using a platform indipendent syntax.
	*
	* @return a path of the type <code>/C:/Develop/webapps/test/WEB-INF/classes/sirti/webapp/util</code>
	*/
	protected String getLocationPath() {
	
		URL url = null ;
		String path = "" ;		
		
		try {
			url = getClass().getProtectionDomain().getCodeSource().getLocation() ;			
			path = url.getPath() ;
			if (Settings.DETAILS) System.out.println(leadingName + "Location path: " + path) ;
		} catch (Exception e) {
		  System.err.println(leadingName + "Error reading url: " + url) ;
		  if (Settings.DETAILS) System.err.println("Exception: " + e.getMessage()) ;
		  return null ;
	    }
		
		return path ;
	}
	
	/**
	* Return the file path describing location of this class, considering the class itself as a 
	* project resource, using <code>mypackage.MyClass.class.getResource()</code>. The returned path
	* can be absolute or relative, depending by the runtime platform. Furthermore, the file separator char 
	* used to produce the path depends by the runtime platform. For example:<br/><br/>
	*
	* Result on Windows: <code>C:\Programs\apache-tomcat-6.0.18\webapps\test\WEB-INF</code>.<br/>
	* Result on Unix: <code>/home/apache-tomcat-6.0.18/webapps/test/WEB-INF</code>.<br/><br/>
	*
	* For this reason when the cross-platorm feature is a crucial issue, the method <code>getLocationPath</code> 
	* should be used instead.
	*
	* @return the path as obtained by the <code>getResource()</code> method
	* @see #getLocationPath()
	*/
	protected String getResourcePath() {
		
		URL url = null ;
		String path = "" ;
		Class logger = this.getClass();
		
		try {
			// Finding the path (in the URL format) for this very class
			url = logger.getResource(logger.getSimpleName()  + ".class") ;
			path = url.getFile() ;
			if (Settings.DETAILS) System.out.println("SystemLogger: resource path: " + path) ;
		} catch (Exception e) {
		  System.err.println("SystemLogger: error reading url: " + url) ;
		  if (Settings.DETAILS) System.err.println("Exception: " + e.getMessage()) ;
		  return null ;
	    }
		
		return path ;
	}

	/**
	* Goes up in the directory tree, starting from the input <code>path</code> directory and climbing
	* the specified number of levels.<br/>
	* For example, if the startin path is <code>/home/apache/webapps/test/WEB-INF</code> and levels
	* equals 3, this method returns <code>/home/apache</code>.
	*
	* @param path the starting path, referring to a directory (not a file)
	* @param levels the number of levels to climb on the file system tree
	* @return the new path (after climbing)
	*/
	protected String climbPath(String path, int levels) {
		
		String dir = new String(path) ;
		for (int i = 0 ; i < levels ; i++) {
			File file = new File(dir) ;
			dir = file.getParent() ;
		}
		if (Settings.DETAILS) System.out.println(leadingName + "Climbed path: " + dir) ;
		return dir ;
	}	
	
// ************************************************************************************************************	
// **********************************		PATH INTROSPECTION		*******************************************
// ************************************************************************************************************
	
	/**
	* If the class is running inside a JAR, EAR or AAR the path returned by <code>getLocationPath()</code> or 
	* <code>getResourcePath()</code> will contain the full name of the package, for example:<br/><br/>
	*
	* <code>/C:/Develop/webapps/test/WEB-INF/classes/myservice.aar!</code><br/><br/>
	*
	* This method looks for the last <code>'/'</code> character in the input path and checks if this path ends
	* with an archive extension of the type JAR, EAR or AAR. If this is the case, the archive element
	* is removed from the path.<br/>
	*
	* @param path the input path as obtained by the <code>getLocationPath()</code> method
	* @return the input path unchanged OR the input path without the archive element
	* @see #getLocationPath()
	* @see #hasExtension(String path)
	*/
	protected String parseExtension(String path) {
		
		String result ;
		if (hasExtension(path)) {
			int start = path.lastIndexOf('/') ;
			result = path.substring(0, start) ;
			if (Settings.DETAILS) System.out.println(leadingName + "Parsed path: " + result) ;
		} else {
			result = new String(path) ;
		}
		return result ;
	}	
	
	/**
	* Check if a given path refers to a JAR, EAR or AAR archive.<br/>
	* This method compares the last path string element with the extensions specified in the <code>Settings</code>
	* class: if the path matches one of these extensions the method return <code>true</code>.<br/>
	*
	* @param path the input path as obtained by the <code>getLocationPath()</code> method
	* @return <code>true</code> if the path refers to a JAR, EAR or AAR archive
	* @see #getLocationPath()
	* @see #parseExtension(String path)
	*/
	protected boolean hasExtension(String path) {
	
		int index = 0 ;
		String[] archive = Settings.ARCHIVE ;
		for (int i = 0 ; i < archive.length ; i++) {
			index = path.lastIndexOf(archive[i]) ;
			if (index > 0) return true ;
		}
		return false ;
	}
	
} // END OF CLASS