/**
* Title: WSLogger
* Description:
* Copyright:    Copyright (c) 2009-2010
* Company:      Morpheus Quality Technologies
* @author Stefano Adriani
* @version 1.0
*/

package adriani.jws.template.util ;

import adriani.jws.template.Settings ;

import java.io.* ;
import java.util.* ;

/**
* This class has two different purposes: <em>static factory</em> and <em>logger instance</em>.<br/><br/>
*
* All the <code>static</code> attributes and methods written here are used to implement a 
* <strong>Factory</strong> which creates <code>WSLogger</code> single objects, used by all the other classes 
* to write messagge on the logging system. This section contains:<br/><br/>
*
* Static class attributes: <code>loggerManager</code> object, <code>benchLogger</code> object, 
* <code>table</code> hashtable, logPath, rotation time,  initialized flag.<br/>
* Static methods: The <code>initialize()</code> method (to be run once) and the <code>getLogger()</code> 
* method, the last representing the actual Factory for creating loggers.<br/>
* Static sccessors: allowing to access the static fields listed above.<br/><br/>
*
* Using the Factory, any other class can retrieve a <code>WSLogger</code> instance, which is a wrapper
* for the actual logger being used (JAVA or LOG4J). The logger instance offers:<br/><br/>
*
* Class attributes: one of the available logging objects (Java logging or Log4j).<br/>
* Methods: the standard logging interface (logInfo, logWarning, logSevere, isInfoEnabled).<br/>
*/
public class WSLogger {

	/**
	* Reference to the static <code>loggerManager</code> created at the first service invocation. 
	* This object MUST BE static to ensure that all the subclasses of the <code>GeneralService</code> class
	* do refer to the same <code>loggerManager</code> object. Dependingly by the service configuration, the 
	* <code>loggerManager</code> can be a <code>SystemLogger</code> (pure Java logger) or an 
	* <code>ExternalLogger</code> (Log4j logger).
	*/
	protected static 		LoggerManager 				loggerManager ;

	/**
	* Reference to the static <code>benchLogger</code> created at the first service invocation. 
	* This object MUST BE static for the same reason of the <code>loggerManager</code> object.<br/> 
	* The <code>benchLogger</code> logs on file detailed information about the single SQL queries, this
	* is different from the <code>serviceAnalyzer</code> object, which collects global and statistical data.
	* <br/><br/> 
	* <strong>Remark</strong>: this logger is always an istance of the <code>SystemLogger</code> class.
	*/
	protected static 		LoggerManager 				benchLogger ;
	
  	/** The logger path loaded from the configuration file for the current runtime enviroment.				*/
	protected static 		String 						logPath 		= "." ;		// Default value
  	/** The logger rotation time suggested loaded from the configuration file.								*/
	protected static 		int 						rotate 			= 720 ;		// Default value
  	/** The boolean flag depicting if this class has already been initialized.								*/
	protected static 		boolean						ready 			= false ;	// Default value
	/**
	* The boolean flag selecting the level on details message on STDOUT.									
	* We use this flag, and not the <code>Settings.DETAILS</code> flag because we want to keep separated
	* the verbosity of the business logic from the verbosity of the logger system itself.
	*/
	protected static 		boolean						verbose			= false ;	// Default value
	
	/** The hash table tracking all the <code>WSLogger</code> objects produced by the factory.				*/
	private static 			Hashtable 					table ;
	
  	/** Instance reference to the logger object relying on the Log4j mechanism. 							*/
	protected 				org.apache.log4j.Logger 	smartLogger ;
	
  	/** Instance reference to the logger object relying on the Log4j mechanism. 							*/
	protected 				java.util.logging.Logger 	javaLogger ;
	
  	/** Beginning of each logging message produced by this class.											*/
	protected static 		String 						leadingName 	= "WSLogger - " ;
	
	/**
	* The input <code>name</code> parameter is used only if the logging mechanism relies on the LOG4J
	* library. Otherwise, if the JAVA logging system is used, all the logger instances will share
	* the same logic name (defined by the <code>Settings.LOGGER</code> field).
	*
	* @param name the logic name to be used for the LOG4J logger object
	* @see Settings
	*/
	public WSLogger(String name) {
	
		if (Settings.JAVA) 		this.javaLogger		= java.util.logging.Logger.getLogger(Settings.LOGGER) ;
		if (Settings.LOG4J) 	this.smartLogger	= org.apache.log4j.Logger.getLogger(name) ;
	}
	
	/**
	* Method useful for debug and testing, the verify the scope of the various logger instances.
    */
    public void finalize() {

		if (verbose) System.out.println(leadingName + "Finalized object: " + this) ;
	}
	
// **********************************************************************************************************
// ************************************			FACTORY				*****************************************
// **********************************************************************************************************

	/**
	* Loads the logging parameters from the <code>configuration.properties</code> file, then creates the
	* required logger object, i.e. one subclass of the <code>LoggerManager</code> interface:<br/><br/>
	*
	* If <code>logger.type = LOG4J</code> the logger will be a <code>ExternalLogger</code> instance.<br/>
	* If <code>logger.type = JAVA</code> the logger will be a <code>SystemLogger</code> instance.<br/>
	*/
	public static void initialize() {
	
		// 1. Storing initialize status (to avoid multiple initializing calls)
		ready 	= true ;
		table 	= new Hashtable() ;

		// 2. Setting default values
		String level = "INFO" ;
		String logdir = "logs" ;
		String linuxPath = "/home/wsadm";
		String windowsPath = "C:\\Develop" ;

		// 3. Loading data from the configuration.properties file
		try {
			ResourceBundle rb	= ResourceBundle.getBundle(Settings.CONFIG) ;

			// Retrieving string properties
			logdir				= rb.getString("logger.dir") ;
			linuxPath			= rb.getString("path.linux") ;
			windowsPath			= rb.getString("path.windows") ;
			level				= rb.getString("logger.level") ;

			// Parsing boolean properties
			String type			= rb.getString("logger.type") ;
			Settings.LOG4J 		= type.equalsIgnoreCase("LOG4J") ;
			Settings.JAVA		= type.equalsIgnoreCase("JAVA") ;

			// Parsing integer properties
			String value		= rb.getString("logger.rotate") ;
			try 				{ rotate = Integer.parseInt(value.trim()) ;		}
			catch(Exception e) 	{ rotate = 720 ;								}	// Default value (12 hours)

			// Storing STATIC results
			Settings.LOGDIR		= logdir ;
			Settings.ROTATE		= rotate ;
			System.out.println(WSLogger.leadingName + "Loaded configuration file: " + Settings.CONFIG + ".properties") ;

		} catch(MissingResourceException e) {
			System.err.println(WSLogger.leadingName + "Log configuration properties file not found: using defaults") ;
			e.printStackTrace() ;
		}

		// 4. Retrieving the log path as defined in the Settings.CONFIG properties file
		if (File.separatorChar == '\\') logPath = windowsPath ;
		else if (File.separatorChar == '/') logPath = linuxPath ;

		// 5. Creating the logging mechanism
		if (Settings.JAVA) {
			loggerManager = new SystemLogger() ;
			loggerManager.init(logPath, level, rotate, true) ;
		} else if (Settings.LOG4J) {
			loggerManager = new ExternalLogger() ;
			loggerManager.init(logPath, level, rotate, true) ;
		}
		
		// 6. Eventually preparing the benchmark logger
		if (Settings.BENCH) {
			benchLogger = new SystemLogger() ;
			benchLogger.init(logPath, "INFO", rotate, false) ;		
		}
	}
	
	/**
	* The main static factory to be used by all other classes to ask and obtain a <code>WSLogger</code>
	* instance. Remember that a <code>WSLogger</code> instance MUST NOT be used to invoke the 
	* <code>initialize()</code> and <code>getLogger()</code> methods.<br/>
	* Instead, a single <code>WSLogger</code> instance MUST BE used only for actual logging, calling one 
	* of the standard logging interface: <code>logInfo()</code>, <code>logWarning()</code>, 
	* <code>logSevere()</code> and <code>isInfoEnabled()</code>.
	**/
	public static synchronized WSLogger getLogger(Class caller) {
	
		// Checking if we already have a logger for this package
		String name 	= caller.getPackage().getName() ;
		WSLogger logger = (WSLogger) table.get(name) ;
		
		// If there isn't any logger, we create a brand new one
		if (logger == null) {
			logger = new WSLogger(name) ;
			table.put(name, logger) ;
			if (verbose) System.out.println(leadingName + "Created WSLogger@" + printObject(logger) + " for " + caller ) ;
		}
		// Else we return the existing one
		else {
			if (verbose) System.out.println(leadingName + "Reusing WSLogger@" + printObject(logger) + " for " + caller ) ;
		}
			
		// Returning logger instance
		return logger ;
	}	
	
// **********************************************************************************************************
// **********************************		FACTORY	ACCESSORS		*****************************************
// **********************************************************************************************************
	
	/**
	* Factory accessor method.
    */
	public static boolean isReady() 					{			return ready ; 							}
	
	/**
	* Return current implementation of the <code>LoggerManager</code> object, which is responsible for
	* creating and initializing the log mechanism.
	*
	* @return a reference to the general <code>LoggerManager</code> object.
    */
	public static LoggerManager getLoggerManager() 		{			return loggerManager ; 					}
	
	/**
	* Return current implementation of the <code>LoggerManager</code> object, which is responsible for
	* creating and initializing the benchmark optional feature.
	*
	* @return a reference to the benchmark <code>LoggerManager</code> object.
    */
	public static LoggerManager getBenchmarkLogger() 	{			return benchLogger ; 					}

// **********************************************************************************************************
// **********************************		INSTANCE ACCESSORS		*****************************************
// **********************************************************************************************************
	
	/**
	* Return the <code>parentDir</code> path used as base directory to locate the log file.
	*
	* @return the current parent directory for logging on the file system
	*/
	public String getParentDir() {
	
		String path = "unset" ;
		if (loggerManager instanceof SystemLogger) {
			path = loggerManager.getParentDir(null) ;
		} else if (loggerManager instanceof ExternalLogger) {
			path = loggerManager.getParentDir(smartLogger) ;
		}
		return path ;
	}

	/**
	* Return the rotation time set for this logger, already formatted and ready to print.
	*
	* @return a string describing the rotation time set for this logger
	*/
	public String getRotationTime() {
	
		String time = "unset" ;
		if (loggerManager instanceof SystemLogger) {
			time = loggerManager.getRotationTime(null) ;
		} else if (loggerManager instanceof ExternalLogger) {
			time = loggerManager.getRotationTime(smartLogger) ;
		}
		return time ;	
	}
	
// **********************************************************************************************************
// ************************************			LOGGING				*****************************************
// **********************************************************************************************************

	/**
	* Log an INFO message on the default <code>Logger</code> and flush the relative Handler.
	*
	* @param msg the text message to be logged on the current logger.
	**/
	public void logInfo(String msg) {

		if (Settings.JAVA) {
			javaLogger.info(msg) ;
			flushLogger(javaLogger) ;
		}
		else if (Settings.LOG4J) {
			smartLogger.info(msg) ;
		}
	}

	/**
	* Log a WARNING message on the default <code>Logger</code> and flush the relative Handler.
	*
	* @param msg the text message to be logged on the current logger.
	**/
	public void logWarning(String msg) {

		if (Settings.JAVA) {
			javaLogger.warning(msg) ;
			flushLogger(javaLogger) ;
		}
		else if (Settings.LOG4J) {
			smartLogger.warn(msg) ;
		}
	}

	/**
	* Log a SEVERE message on the default <code>Logger</code> and flush the relative Handler.
	*
	* @param msg the text message to be logged on the current logger.
	*/
	public void logSevere(String msg) {

		if (Settings.JAVA) {
			javaLogger.severe(msg) ;
			flushLogger(javaLogger) ;
		}
		else if (Settings.LOG4J) {
			smartLogger.error(msg) ;
		}
	}

	/**
	* Flush the Handler bound to the given <code>Logger</code>. Used only when the <code>SystemLogger</code>
	* class is used to log messages instead of the Log4j mechanism.
	*
	* @param logger a reference to the logger object to be flushed.
	*/
	public void flushLogger(java.util.logging.Logger logger) {

		java.util.logging.Handler[] handlers = logger.getHandlers() ;
		if ( (handlers.length > 0) && (handlers[0] != null) ) handlers[0].flush() ;
	}

	/**
	* Improves performances by avoiding to evaluate the concatened string message when the log call will not
	* be executed (because not enabled for logger current level).
	* This method return always <code>false</code> if no logger is found.
	*
	* @return <code>true</code> if the INFO log level is enabled on the current Logger.
	*/
	public boolean isInfoEnabled() {

		if (Settings.JAVA) {
			return (javaLogger.getLevel() == java.util.logging.Level.INFO) ;
		}
		else if (Settings.LOG4J) {
			return smartLogger.isInfoEnabled() ;
		}
		return false ;
	}
	
// **********************************************************************************************************
// ************************					UTILITIES				*****************************************
// **********************************************************************************************************

	/**
	* Print the serialize reference number of the input object.
	
	* @param object a generic Java object, e.g. MyObject@1bd4722.
	* @param the serial number of the input object, e.g. 1bd4722.
	*/
	public static String printObject(Object object) {

		String fullName = object.toString() ;
		String[] result = fullName.split("@") ;
		if (result == null) return "unknown" ;
		else return result[1] ;
	}

}	// END OF CLASS